import { IGladiaTranscriptionResult } from "./IGladiaTranscript";
export declare enum TranscriptionStatus {
    queued = "queued",
    processing = "processing",
    done = "done",
    error = "error"
}
export declare enum TranscriptionKind {
    PreRecorded = "pre-recorded",
    Live = "live"
}
export interface ITranscriptionsResponse {
    items: IGetTranscriptionResponse[];
    first: string;
    current: string;
    next: string;
}
export interface IGladiaAudioMetadata {
    id: string;
    filename: string;
    extension: string;
    size: number;
    audio_duration: number;
    number_of_channels: number;
}
export interface IAudioUploadResponse {
    audio_url: string;
    audio_metadata: IGladiaAudioMetadata;
}
export interface IAudioTranscriptionResponse {
    id: string;
    result_url: string;
}
interface IDiarizationConfig {
    number_of_speakers?: number;
    min_speakers?: number;
    max_speakers?: number;
}
interface ITranslationConfig {
    model?: "base" | "enhanced";
    target_languages: string[];
}
interface ISubtitlesConfig {
    formats: ("srt" | "vtt")[];
}
interface ISummarizationConfig {
    type: "general" | "bullet_points" | "concise";
}
interface IAudioToLLMConfig {
    prompts: string[];
}
export declare const DEFAULT_AUDIO_TRANSCRIPTION_CONFIG: IAudioTranscriptionConfig;
export interface IAudioTranscriptionConfig {
    audio_url: string;
    sentences?: boolean;
    context_prompt?: string;
    custom_vocabulary?: string[];
    detect_language?: boolean;
    enable_code_switching?: boolean;
    language?: string;
    callback_url?: string;
    subtitles?: boolean;
    subtitles_config?: ISubtitlesConfig;
    diarization?: boolean;
    diarization_config?: IDiarizationConfig;
    translation?: boolean;
    translation_config?: ITranslationConfig;
    summarization?: boolean;
    summarization_config?: ISummarizationConfig;
    moderation?: boolean;
    audio_to_llm?: boolean;
    audio_to_llm_config?: IAudioToLLMConfig;
    custom_metadata?: Record<string, unknown>;
}
export interface IFile {
    id: string;
    filename: string;
    source: string;
    audio_duration: number;
    number_of_channels: number;
}
export interface IRequestParams {
    audio_url: string;
    subtitles: boolean;
    diarization: boolean;
    translation: boolean;
    summarization: boolean;
    sentences: boolean;
    moderation: boolean;
    audio_enhancer: boolean;
    detect_language: boolean;
    enable_code_switching: boolean;
}
export interface IGetTranscriptionResponse {
    id: string;
    request_id: string;
    kind: string;
    created_at: string;
    completed_at: string;
    status: TranscriptionStatus;
    file: IFile;
    request_params: IRequestParams;
    result?: IGladiaTranscriptionResult;
}
export interface IGladiaErrorResponse {
    timestamp: string;
    path: string;
    request_id: string;
    statusCode: number;
    message: string;
    validation_errors?: string[];
}
export declare function isIErrorResponse(data: any): boolean;
export {};
//# sourceMappingURL=interfaces.d.ts.map