/* eslint-disable capitalized-comments */
/* eslint-disable max-params */
import pkg from "lodash";
const { isArray } = pkg;
import { Timestamp } from "@firebase/firestore";
import { v4 } from "uuid";
import { iHasContainerRelationshipToAlgolia } from "./IHasContainerRelationship.js";
import { iHasParentChildRelationshipToAlgolia, sanitizeIHasParentChildRelationship, } from "./IHasParentChildRelationship.js";
import { iHasSiblingsRelationshipToAlgolia } from "./IHasSiblingsRelationship.js";
import { iHasUserCreataUpdateToAlgolia, sanitizeIHasUserCreataUpdate, } from "./IHasUsers.js";
import { ifSmaller } from "../extensions/index.js";
export function sanitizeICollectionDocument(item) {
    const changed1 = sanitizeIHasUserCreataUpdate(item);
    const changed2 = sanitizeIHasParentChildRelationship(item);
    return changed1 || changed2;
}
export function iCollectionDocumentToAlgolia(item) {
    const userAndGroup = iHasUserCreataUpdateToAlgolia(item);
    const relations = Object.assign(Object.assign(Object.assign({}, iHasParentChildRelationshipToAlgolia(item)), iHasContainerRelationshipToAlgolia(item)), iHasSiblingsRelationshipToAlgolia(item));
    const algoliaDoc = Object.assign(Object.assign(Object.assign({}, userAndGroup), relations), { id: item.id, documentReference: item.documentReference, notificationToken: item.notificationToken, tags: item.tags, metadata: ifSmaller(item.metadata, 0.3) });
    return algoliaDoc;
}
export function iCollectionDocumentRequiredSubset(item, collectionPath = "Library") {
    var _a, _b, _c, _d;
    return {
        id: item.id,
        createdBy: item.createdBy,
        users: (_a = item.users) !== null && _a !== void 0 ? _a : [],
        groups: (_b = item.groups) !== null && _b !== void 0 ? _b : [],
        groupClaims: (_c = item.groupClaims) !== null && _c !== void 0 ? _c : {},
        documentReference: (_d = item.documentReference) !== null && _d !== void 0 ? _d : {
            id: item.id,
            path: `${collectionPath}/${item.id}`,
        },
    };
}
export var WorkerState;
(function (WorkerState) {
    WorkerState["Idle"] = "Idle";
    WorkerState["Pending"] = "Pending";
    WorkerState["Running"] = "Running";
    WorkerState["Completed"] = "Completed";
    WorkerState["Failed"] = "Failed";
})(WorkerState || (WorkerState = {}));
export var WorkerAction;
(function (WorkerAction) {
    WorkerAction["None"] = "None";
    WorkerAction["VideoUploadedPipe"] = "videoUploadedPipe";
    WorkerAction["NormalizeAudio"] = "NormalizeAudio";
    WorkerAction["Transcode"] = "Transcode";
    WorkerAction["Translate"] = "Translate";
    WorkerAction["TranslateEditSpecification"] = "TranslateEditSpecification";
    WorkerAction["Transcribe"] = "Transcribe";
    WorkerAction["Render"] = "Render";
    WorkerAction["AnalyzeVideo"] = "AnalyzeVideo";
    WorkerAction["AnalyzeAudio"] = "AnalyzeAudio";
    WorkerAction["AnalyzeImage"] = "AnalyzeImage";
})(WorkerAction || (WorkerAction = {}));
export function getBackgroundTasks(actualData) {
    if (isArray(actualData === null || actualData === void 0 ? void 0 : actualData.backgroundTasks)) {
        return actualData === null || actualData === void 0 ? void 0 : actualData.backgroundTasks;
    }
    return [];
}
export function workerStateInProgress(state) {
    return state === WorkerState.Pending || state === WorkerState.Running;
}
export function getTasksSorted(actualData, asc) {
    var _a;
    if (!getBackgroundTasks(actualData))
        return [];
    const tasks = (_a = getBackgroundTasks(actualData)) === null || _a === void 0 ? void 0 : _a.sort((a, b) => compareTimestamps(a.createdAt, b.createdAt));
    if (asc)
        return tasks;
    if (!asc)
        return tasks.reverse();
}
function compareTimestamps(a, b) {
    if (a && b) {
        const dateA = a instanceof Date ? a : new Timestamp(a.seconds, a.nanoseconds).toDate();
        const dateB = b instanceof Date ? b : new Timestamp(b.seconds, b.nanoseconds).toDate();
        if (dateA.getTime() > dateB.getTime())
            return 1;
        if (dateA.getTime() < dateB.getTime())
            return -1;
    }
    return 0;
}
export function taskInProgress(data) {
    return getTasksInProgress(data).length > 0;
}
export function getTasksInProgress(actualData) {
    var _a;
    return ((_a = getBackgroundTasks(actualData)
        .filter((task) => task.state === WorkerState.Pending ||
        task.state === WorkerState.Running)
        .sort((a, b) => {
        if (a.state === WorkerState.Running &&
            b.state !== WorkerState.Running) {
            return -1;
        }
        if (b.state === WorkerState.Running &&
            a.state !== WorkerState.Running) {
            return 1;
        }
        if (a.state === WorkerState.Pending &&
            b.state !== WorkerState.Pending) {
            return -1;
        }
        if (b.state === WorkerState.Pending &&
            a.state !== WorkerState.Pending) {
            return 1;
        }
        return 0;
    })) !== null && _a !== void 0 ? _a : []);
}
/**
 * Retrieves a list of background tasks filtered by the specified action and sorts them.
 *
 * @param action - The action to filter tasks by.
 * @param actualData - Optional collection document containing the tasks.
 * @returns An array of background tasks filtered by the specified action and sorted.
 */
export function getTasksByAction(action, actualData) {
    var _a;
    return ((_a = getBackgroundTasks(actualData).filter((task) => task.action === action)) !== null && _a !== void 0 ? _a : []).sort((a, b) => {
        if (workerStateInProgress(a.state) && !workerStateInProgress(b.state))
            return -1;
        if (!workerStateInProgress(a.state) && workerStateInProgress(b.state))
            return 1;
        return 0;
    });
}
export function getTaskByAction(action, actualData, taskId) {
    const tasks = getBackgroundTasks(actualData).filter((task) => task.action === action && (taskId ? task.id === taskId : true));
    if (!(tasks === null || tasks === void 0 ? void 0 : tasks.length))
        return undefined;
    if (tasks.length > 1) {
        const inProgress = tasks.find((task) => workerStateInProgress(task.state));
        if (inProgress)
            return inProgress;
    }
    return tasks[0];
}
/**
 * Retrieves the index of a task within the backgroundTasks array of the provided ICollectionDocument
 * that matches the specified WorkerAction.
 *
 * @param action - The action to search for within the backgroundTasks array.
 * @param actualData - The ICollectionDocument object containing the backgroundTasks array.
 *                     If null or undefined, the function returns -1.
 * @returns The index of the task that matches the specified action, or -1 if no match is found or if actualData is null/undefined.
 */
export function getTaskIndexByAction(action, actualData, taskId) {
    var _a;
    if (!actualData)
        return -1;
    const index = ((_a = actualData === null || actualData === void 0 ? void 0 : actualData.backgroundTasks) !== null && _a !== void 0 ? _a : []).findIndex((task) => task.action === action && taskId ? task.id === taskId : true);
    return index;
}
/**
 * Determines if the background tasks have changed between two ICollectionDocument instances.
 *
 * @param posterior - The posterior ICollectionDocument instance to compare.
 * @param prior - The prior ICollectionDocument instance to compare.
 * @param action - An optional WorkerAction to filter tasks by.
 * @returns A boolean indicating whether the background tasks have changed.
 */
export function backgroundTasksChanged(posterior, prior, action) {
    const posteriorTasks = action
        ? getTasksByAction(action, posterior)
        : getBackgroundTasks(posterior);
    const priorTasks = action
        ? getTasksByAction(action, prior)
        : getBackgroundTasks(prior);
    let changed = false;
    for (const posteriorTask of posteriorTasks) {
        const priorTask = priorTasks.find((t) => t.action === posteriorTask.action);
        if (backgroundTaskChanged(posteriorTask, priorTask)) {
            changed = true;
            break;
        }
    }
    return changed;
}
/**
 * Determines if a background task has changed state.
 *
 * This function compares the prior and posterior background tasks to check if there has been a change in their states.
 *
 * @param posteriorTask - The background task in its current state.
 * @param priorTask - The background task in its previous state.
 * @returns `true` if there is a change in the state of the background task, otherwise `false`.
 */
export function backgroundTaskChanged(posteriorTask, priorTask) {
    if (!priorTask && posteriorTask)
        return true;
    if (priorTask && !posteriorTask)
        return true;
    if (priorTask && posteriorTask) {
        if (priorTask.state !== posteriorTask.state)
            return true;
    }
    return false;
}
/**
 * Determines if any background task has been completed between two states of a collection document.
 *
 * @param posterior - The posterior state of the collection document.
 * @param prior - The prior state of the collection document.
 * @param action - (Optional) A specific worker action to filter tasks by.
 * @returns `true` if any background task has been completed; otherwise, `false`.
 */
export function anyBackgroundTaskCompleted(posterior, prior, action) {
    const posteriorTasks = action
        ? getTasksByAction(action, posterior)
        : getBackgroundTasks(posterior);
    const priorTasks = action
        ? getTasksByAction(action, prior)
        : getBackgroundTasks(prior);
    for (const posteriorTask of posteriorTasks) {
        const priorTask = priorTasks.find((t) => t.action === posteriorTask.action);
        if (backgroundTaskCompleted(posteriorTask, priorTask)) {
            return true;
        }
    }
    return false;
}
/**
 * Determines if a background task has been completed.
 *
 * @param posteriorTask - The task that is being checked for completion.
 * @param priorTask - The task that was previously being executed.
 * @returns `true` if the posterior task is completed and has changed from the prior task, otherwise `false`.
 */
export function backgroundTaskCompleted(posteriorTask, priorTask) {
    if (posteriorTask && backgroundTaskChanged(posteriorTask, priorTask)) {
        if (posteriorTask.state === WorkerState.Completed)
            return true;
    }
    return false;
}
/**
 * Determines if there is a rising edge in background tasks between the prior and posterior states.
 *
 * @param action - The action to filter tasks by.
 * @param stateRise - The state that indicates a rising edge.
 * @param posterior - The current state of the collection document.
 * @param prior - The previous state of the collection document (optional).
 * @param blockingStates - An array of states that block the rising edge (optional).
 * @param compareId -  If true, the transition is only considered if it has the same id.
 * @returns `true` if a rising edge is detected, otherwise `false`.
 */
export function backgroundTasksRisingEdge(action, stateRise, posterior, prior, blockingStates, compareId = false) {
    const posteriorTasks = getTasksByAction(action, posterior);
    const priorTasks = getTasksByAction(action, prior);
    const risen = posteriorTasks
        .map((posterior) => {
        const prior = compareId
            ? priorTasks.find((t) => t.id === posterior.id)
            : priorTasks.find((t) => t.action === action);
        if (backgroundTaskRisingEdge(action, stateRise, posterior, prior, blockingStates, compareId)) {
            return Object.assign({}, posterior);
        }
        return undefined;
    })
        .filter((task) => task !== undefined);
    return risen.at(0);
}
/**
 * Determines if a background task has transitioned to a specified state (rising edge).
 *
 * @param action - The action associated with the worker.
 * @param stateRise - The state that signifies the rising edge.
 * @param current - The current background task.
 * @param previous - The previous background task.
 * @param blockingStates - An array of states that block the transition.
 * @param compareId - If true, the transition is only considered if it has the same id.
 * @returns `true` if the background task has transitioned to the specified state, otherwise `false`.
 */
export function backgroundTaskRisingEdge(action, stateRise, current, previous, blockingStates, compareId = false) {
    const isCurrent = current !== undefined && current !== null;
    const isPrevious = previous !== undefined && previous !== null;
    if (!isCurrent || current.action !== action)
        return false;
    if (compareId && isPrevious && previous.id !== current.id)
        return false;
    if ((previous === null || previous === void 0 ? void 0 : previous.state) && (blockingStates === null || blockingStates === void 0 ? void 0 : blockingStates.includes(previous.state)))
        return false;
    if (current.state === stateRise) {
        if ((previous === null || previous === void 0 ? void 0 : previous.state) !== stateRise) {
            console.log(`____|^^^^  RISING EDGE  action:${action}; state:${stateRise}; prior:${previous === null || previous === void 0 ? void 0 : previous.state}`);
            return true;
        }
    }
    return false;
}
/**
 * Upserts a background task into the collection document.
 *
 * This function checks if the `backgroundTasks` array exists in the `actualData` object.
 * If it does not exist, it initializes it as an empty array.
 *
 * It then searches for an existing task in the `backgroundTasks` array that matches the
 * action of the provided `task` and has a state of `WorkerState.Pending`.
 *
 * If such a task is not found, it assigns a new ID to the task (if it doesn't already have one),
 * sets the creation date, and adds the task to the beginning of the `backgroundTasks` array.
 *
 * If a matching task is found, it updates the task in the array with the provided `task`.
 *
 * @param actualData - The collection document containing the background tasks.
 * @param task - The background task to be upserted.
 * @returns `true` if the task was added or updated, `false` otherwise.
 */
export function upsertBackgroundTask(actualData, task) {
    if (!isArray(actualData.backgroundTasks))
        actualData.backgroundTasks = [];
    const index = actualData.backgroundTasks.findIndex((t) => t.action === task.action && t.id === task.id);
    if (index === -1) {
        if (!task.id || task.id.length === 0)
            task.id = v4().substring(0, 8);
        task.createdAt = new Date();
        actualData.backgroundTasks.unshift(task);
        return true;
    }
    actualData.backgroundTasks[index] = task;
    return true;
}
/**
 * Updates the state of a background task in the given document.
 *
 * This function can set the task to a specific state or advance it to the next state.
 * It updates the `updatedAt` timestamp and sets the `finishedAt` timestamp if the task is completed.
 * It can also update the task's message and trace reference.
 *
 * @param action - The action associated with the worker task.
 * @param doc - The document containing the background tasks.
 * @param options - Optional parameters:
 *   - taskId: The ID of the task to update. If not provided, the first task matching the action is updated.
 *   - newState: The new state to set for the task.
 *   - advanceToNextState: If true, advances the task to the next state.
 *   - message: An optional message to associate with the task.
 *   - traceRef: An optional trace reference to associate with the task.
 * @returns `true` if the task state was successfully updated, `false` otherwise.
 */
export function updateBackgroundTaskState(action, doc, options) {
    const index = getTaskIndexByAction(action, doc, options === null || options === void 0 ? void 0 : options.taskId);
    if (!isArray(doc.backgroundTasks) || index === -1)
        return false;
    const task = doc.backgroundTasks[index];
    task.updatedAt = new Date();
    if (options === null || options === void 0 ? void 0 : options.advanceToNextState) {
        task.state = nextState(task.state);
    }
    else if ((options === null || options === void 0 ? void 0 : options.newState) !== undefined) {
        task.state = options.newState;
    }
    if (task.state === WorkerState.Completed) {
        task.finishedAt = new Date();
    }
    if (options === null || options === void 0 ? void 0 : options.message)
        task.message = options.message;
    if (options === null || options === void 0 ? void 0 : options.traceRef)
        task.traceReference = options.traceRef;
    return true;
}
// /**
//  * Adds a background task to the collection document. If the backgroundTasks array
//  * does not exist, it initializes it. If the task does not have an ID, it generates
//  * a new one. The task is then timestamped and added to the beginning of the array.
//  *
//  * @param actualData - The collection document to which the background task will be added.
//  * @param task - The background task to be added.
//  */
// export function createBackgroundTask(
//   actualData: ICollectionDocument,
//   task: IBackgroundTask
// ) {
//   if (!isArray(actualData.backgroundTasks)) actualData.backgroundTasks = [];
//   if (task.id === undefined || task.id.length === 0)
//     task.id = v4().substring(0, 8);
//   task.createdAt = new Date();
//   actualData.backgroundTasks.unshift(task);
// }
// /**
//  * Upserts a background task into the collection document.
//  *
//  * This function checks if the `backgroundTasks` array exists in the `actualData` object.
//  * If it does not exist, it initializes it as an empty array.
//  *
//  * It then searches for an existing task in the `backgroundTasks` array that matches the
//  * action of the provided `task` and has a state of `WorkerState.Pending`.
//  *
//  * If such a task is not found, it assigns a new ID to the task (if it doesn't already have one),
//  * sets the creation date, and adds the task to the beginning of the `backgroundTasks` array.
//  *
//  * If a matching task is found, it updates the task in the array with the provided `task`.
//  *
//  * @param actualData - The collection document containing the background tasks.
//  * @param task - The background task to be upserted.
//  * @returns `true` if the task was added or updated, `false` otherwise.
//  */
// export function upsertBackgroundTask(
//   actualData: ICollectionDocument,
//   task: IBackgroundTask
// ) {
//   if (!isArray(actualData.backgroundTasks)) actualData.backgroundTasks = [];
//   const index = actualData.backgroundTasks.findIndex(
//     (t) => t.action === task.action && t.state === WorkerState.Pending
//   );
//   if (index === -1) {
//     if (task.id === undefined || task.id.length === 0)
//       task.id = v4().substring(0, 8);
//     task.createdAt = new Date();
//     actualData.backgroundTasks.unshift(task);
//     return true;
//   }
//   if (actualData.backgroundTasks[index]) {
//     actualData.backgroundTasks[index] = task;
//     return true;
//   }
//   return false;
// }
// /**
//  * Set the state of a background task within a collection document.
//  *
//  * @param action - The action associated with the worker task.
//  * @param state - The new state to set for the background task.
//  * @param actualData - The collection document containing the background tasks.
//  * @param message - Optional message to associate with the task update.
//  * @param traceRef - Optional trace reference to associate with the task update.
//  * @returns `true` if the background task state was successfully updated, `false` otherwise.
//  */
// export function setBackgroundTaskState(
//   action: WorkerAction,
//   state: WorkerState,
//   actualData: ICollectionDocument,
//   message?: string | null,
//   traceRef?: string | null
// ) {
//   const index = getTaskIndexByAction(action, actualData);
//   if (!isArray(actualData.backgroundTasks) || index === -1) return false;
//   if (actualData.backgroundTasks[index]) {
//     actualData.backgroundTasks[index].state = state;
//     actualData.backgroundTasks[index].updatedAt = new Date();
//     if (message) actualData.backgroundTasks[index].message = message;
//     if (traceRef) actualData.backgroundTasks[index].traceReference = traceRef;
//     return true;
//   }
//   return false;
// }
// /**
//  * Updates the state to the NEXT state of a background task in the given document based on the provided action.
//  *
//  * @param action - The action performed by the worker.
//  * @param doc - The document containing the background tasks.
//  * @param message - Optional message to update the task with.
//  * @param traceRef - Optional trace reference to update the task with.
//  * @returns `true` if the task state was successfully updated, `false` otherwise.
//  */
// export function nextBackgroundTaskState(
//   action: WorkerAction,
//   doc: ICollectionDocument,
//   message?: string,
//   traceRef?: string | null
// ) {
//   const index = getTaskIndexByAction(action, doc);
//   if (!isArray(doc.backgroundTasks) || index === -1) return false;
//   doc.backgroundTasks[index].updatedAt = new Date();
//   doc.backgroundTasks[index].state = nextState(
//     doc.backgroundTasks[index].state
//   );
//   if (doc.backgroundTasks[index].state === WorkerState.Completed) {
//     doc.backgroundTasks[index].finishedAt = new Date();
//   }
//   if (message) doc.backgroundTasks[index].message = message;
//   if (traceRef) doc.backgroundTasks[index].traceReference = traceRef;
//   return true;
// }
// /**
//  * Updates the state of a background task in the given document.
//  *
//  * @param action - The action associated with the worker task.
//  * @param newState - The new state to set for the worker task.
//  * @param doc - The document containing the background tasks.
//  * @param message - Optional message to associate with the task update.
//  * @param traceRef - Optional trace reference to associate with the task update.
//  * @returns `true` if the task state was successfully updated, `false` otherwise.
//  */
// export function updateBackgroundTaskState(
//   action: WorkerAction,
//   newState: WorkerState,
//   doc: ICollectionDocument,
//   message?: string | null,
//   traceRef?: string | null
// ) {
//   const index = getTaskIndexByAction(action, doc);
//   if (!isArray(doc.backgroundTasks) || index === -1) return false;
//   if (doc.backgroundTasks[index]) {
//     doc.backgroundTasks[index].updatedAt = new Date();
//     if (newState === WorkerState.Completed)
//       doc.backgroundTasks[index].finishedAt = new Date();
//     doc.backgroundTasks[index].state = newState;
//     doc.backgroundTasks[index].message = message;
//     doc.backgroundTasks[index].traceReference = traceRef;
//     return true;
//   }
//   return false;
// }
function toBackgroundTaskLog(actualData) {
    var _a;
    actualData.backgroundTaskLog = [...((_a = actualData.backgroundTaskLog) !== null && _a !== void 0 ? _a : [])];
    if (actualData.backgroundTask)
        actualData.backgroundTaskLog.unshift(Object.assign(Object.assign({}, actualData.backgroundTask), { finishedAt: new Date() }));
}
function nextState(state) {
    switch (state) {
        case WorkerState.Pending:
            return WorkerState.Running;
        case WorkerState.Running:
            return WorkerState.Completed;
        case WorkerState.Completed:
            return WorkerState.Idle;
        default:
            return WorkerState.Idle;
    }
}
//# sourceMappingURL=ICollectionDocument.js.map