import { VideoResolution, VideoRatioType, VideoCodecType, VideoImageFormatType, PixelFormat, VideoFileType, } from "../enums/index.js";
export const DEFAULT_VIDEO_SETTINGS = {
    frameRate: 25,
    durationInFrames: 1,
    videoCodec: VideoCodecType.H264,
    videoImageFormat: VideoImageFormatType.JPEG,
    pixelFormat: PixelFormat.yuv420p,
    proresProfile: null,
    fileType: VideoFileType.mp4,
    isLandscape: true,
    resolution: VideoResolution.HD,
    ratio: VideoRatioType.R16To9,
    ratioFraction: [16, 9],
    width: 1920,
    height: 1080,
    alphaChannel: false,
};
export function updateVideoSettingsFromMetadata(videoSettings, videoMetadata) {
    var _a, _b, _c, _d;
    const updatedVideoSettings = Object.assign({}, videoSettings);
    if ((_a = videoMetadata.frameRate) === null || _a === void 0 ? void 0 : _a.value) {
        updatedVideoSettings.frameRate = videoMetadata.frameRate.value;
    }
    if ((_b = videoMetadata.duration) === null || _b === void 0 ? void 0 : _b.inFrames) {
        updatedVideoSettings.durationInFrames = videoMetadata.duration.inFrames;
    }
    if (((_c = videoMetadata.dimensions) === null || _c === void 0 ? void 0 : _c.width) && ((_d = videoMetadata.dimensions) === null || _d === void 0 ? void 0 : _d.height)) {
        updatedVideoSettings.width = videoMetadata.dimensions.width;
        updatedVideoSettings.height = videoMetadata.dimensions.height;
        if (videoMetadata.dimensions.ratio)
            updatedVideoSettings.ratio = videoMetadata.dimensions.ratio;
        if (videoMetadata.dimensions.ratioFraction)
            updatedVideoSettings.ratioFraction =
                videoMetadata.dimensions.ratioFraction;
        updatedVideoSettings.isLandscape =
            updatedVideoSettings.width > updatedVideoSettings.height;
    }
    if (videoMetadata.codec) {
        updatedVideoSettings.videoCodec = videoMetadata.codec
            .name;
        updatedVideoSettings.pixelFormat = videoMetadata.codec
            .colorspace;
    }
    return updatedVideoSettings;
}
//# sourceMappingURL=IVideoSettings.js.map