var _a;
import { vec2 } from "gl-matrix";
import { immerable } from "immer";
import { Box } from "./box.js"; // Assuming the Box class is in the same directory
import { Aspect } from "../../enums/index.js";
var CssUnit;
(function (CssUnit) {
    CssUnit["millimeters"] = "mm";
    CssUnit["centimeters"] = "cm";
    CssUnit["inches"] = "in";
    CssUnit["pixel"] = "px";
    CssUnit["points"] = "pt";
    CssUnit["picas"] = "pc";
    CssUnit["em"] = "em";
    CssUnit["rem"] = "rem";
    CssUnit["ex"] = "ex";
    CssUnit["ch"] = "ch";
    CssUnit["viewportWidth"] = "vw";
    CssUnit["viewportHeight"] = "vh";
    CssUnit["vMin"] = "vmin";
    CssUnit["vMax"] = "vmax";
    CssUnit["percent"] = "%";
})(CssUnit || (CssUnit = {}));
export class BoxStyle {
    constructor(box) {
        this[_a] = true; // Option 1
        this.unit = CssUnit.percent;
        this.box = box;
    }
    /**
     * Generates the CSS properties required to style an HTML element
     * based on the Box model's properties, using projectToParentSpace.
     *
     * @returns {CSSProperties} The calculated CSS properties.
     */
    getStyleInPercent(applyScalingEffect = true) {
        if (applyScalingEffect) {
            switch (this.box.scalingEffect) {
                case Aspect.Fit:
                    this.box.fit();
                    break;
                case Aspect.Fill:
                    this.box.fill();
                    break;
                default:
                    break;
            }
        }
        const scale = 100; // Scale to percentage
        const scaleToPercent = (value) => roundToNearest(value * 100, 0.01);
        const style = {};
        style.position = "absolute";
        // Compute width and height from the box size
        const sizeScaled = vec2.scale(vec2.create(), this.box.sizeInParentSpace, scale);
        style.width = `${roundToNearest(sizeScaled[0])}${this.unit}`;
        style.height = `${roundToNearest(sizeScaled[1])}${this.unit}`;
        // Compute the top-left position from the box position
        const topLeft = this.box.projectToParentSpace([-0.5, 0.5]);
        const inPixelCoordinates = Box.toPixelCoordinateSystem(topLeft);
        style.left = `${scaleToPercent(inPixelCoordinates[0])}${this.unit}`;
        style.top = `${scaleToPercent(inPixelCoordinates[1])}${this.unit}`;
        if (this.box.rotate) {
            // Use Pixel Coordinate System for rotation
            const originOfRotation = Box.invertY(vec2.add(vec2.create(), this.box.anchor, Box.topLeftToOrigin));
            style.transformOrigin = `${roundToNearest(originOfRotation[0])}${this.unit} ${roundToNearest(originOfRotation[1])}${this.unit}`;
            // Apply rotation if any
            if (this.box.rotation !== 0) {
                style.transform = `rotate(${this.box.rotation}deg)`;
            }
        }
        return style;
    }
}
_a = immerable;
function roundToNearest(value, nearest = 0.01) {
    return Math.round(value / nearest) * nearest;
}
//# sourceMappingURL=boxStyle.js.map