import "reflect-metadata";
import { immerable } from "immer";
import { AssetKey, IClip, ILayer, ITransition, IVideoSettings } from "../interfaces/index.js";
import { TimeLineElement } from "./timeLineElement.js";
import { Layer } from "./layer/layer.js";
import { AudioLayer } from "./layer/audio.layer.js";
import { FrameRange } from "./frameRange.js";
import { LottieLayer } from "./layer/lottie.layer.js";
import { EditSpecification } from "./editSpecification.js";
import { VideoLayer } from "./layer/video.layer.js";
import { SubtitleLayer } from "./layer/subtitle.layer.js";
import { ImageLayer } from "./layer/image.layer.js";
import { MediaAssetVariant } from "./mediaAsset.js";
export declare class Clip extends TimeLineElement {
    __CLIP__: string;
    [immerable]: boolean;
    name?: string;
    layers: (Layer | AudioLayer | VideoLayer | ImageLayer | SubtitleLayer | LottieLayer)[];
    get primaryLayer(): Layer | AudioLayer | VideoLayer | ImageLayer | SubtitleLayer | LottieLayer | undefined;
    set primaryLayer(value: Layer | AudioLayer | VideoLayer | ImageLayer | SubtitleLayer | LottieLayer | undefined);
    get languages(): string[];
    get absoluteRange(): FrameRange;
    set absoluteRange(value: FrameRange);
    get innerBoundaryRange(): FrameRange | undefined;
    set innerBoundaryRange(value: FrameRange | undefined);
    get outerBoundaryRange(): FrameRange;
    set outerBoundaryRange(value: FrameRange);
    private get rangeInsideClip();
    get libId(): string | null;
    set libId(value: string | null);
    constructor(partial?: Partial<Clip>);
    setDuration(value: number, factor?: number): void;
    setTransitionsFromIClip(iClip: IClip, fps: number, start: boolean, end: boolean): void;
    clearStartEndTransitions(): void;
    trimmLeft(delta: number, moveToRight?: boolean, absMoveToRight?: boolean): void;
    trimmRight(delta: number): void;
    clearLayers(): void;
    setLayers(items: Layer[]): void;
    addLayer(layer: Layer, inheritTransitions?: boolean): void;
    shouldTrim(layer: Layer): boolean;
    getTransitionPaddingStart(layer: Layer): number;
    getTransitionPaddingEnd(layer: Layer): number;
    static getRange(clip: Clip, lastClip?: Clip): FrameRange;
    getRange(previousClip?: Clip): FrameRange;
    getNextClipRange(lastFrame: number, nextClip: Clip): FrameRange;
    toIClip(fps: number): IClip;
    getILayers(): ILayer[];
    print(): void;
    static iLayerWithTransitions(layer: ILayer, transitions: ITransition[]): ILayer;
    static fromFrames(frames: number): Clip;
    static fromSeconds(seconds: number, fps?: number): Clip;
    static fromClip(clip: Clip): Clip;
    static fromIClip(data: IClip, fps: number): Clip;
    static fromLayer(layer: Layer, fps?: number): Clip;
    static setAboluteRanges(): void;
    static emptyClip(durationInFrames?: number): Clip;
}
export declare function clipsFromEditSpecification(editSpecification: EditSpecification, videoSettings: IVideoSettings): Clip[];
export declare function setClipTiming(clips: Clip[]): void;
export declare function withClipTiming(clips: Clip[]): Clip[];
export declare function setClipTransitions(clips: Clip[], editSpecification: EditSpecification): void;
export declare function withClipTransitions(clips: Clip[], editSpecification: EditSpecification): Clip[];
export declare function longestDuration(iLayers: ILayer[]): number;
export declare function clipFromILayers(iLayers: ILayer[], assets: MediaAssetVariant<AssetKey>[], videoSettings: IVideoSettings, durationInSeconds: number, defaults: IClip | null, first?: boolean, last?: boolean): Clip | undefined;
export declare function editSpecificationToClips(data: string | EditSpecification | object, videoSettings?: IVideoSettings, caller?: string): Clip[];
export declare function clipsToEditSpecification(clips: Clip[], origin: EditSpecification, durationInFrames?: number): EditSpecification;
//# sourceMappingURL=clip.d.ts.map