var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import _ from "lodash";
import { Type, instanceToInstance, plainToInstance } from "class-transformer";
import { stringify } from "flatted";
import { immerable, produce } from "immer";
import { LayerType, TimingAction } from "../enums/index.js";
import { TimeLineElement } from "./timeLineElement.js";
import { Layer } from "./layer/layer.js";
import { distinctTransitions } from "./transition.js";
import { AudioLayer } from "./layer/audio.layer.js";
import { FrameRange } from "./frameRange.js";
import { LottieLayer } from "./layer/lottie.layer.js";
import { LowerThirdLayer } from "./layer/lowerThird.layer.js";
import { SlideInTextLayer } from "./layer/slideInText.layer.js";
import { TitleLayer } from "./layer/title.layer.js";
import { EditSpecification } from "./editSpecification.js";
import { verbose } from "../constants.js";
import { Frame } from "./frame.js";
import { VideoLayer } from "./layer/video.layer.js";
import { layerFromILayer } from "./layer/layer.factory.js";
import { SubtitleLayer } from "./layer/subtitle.layer.js";
import { ImageLayer } from "./layer/image.layer.js";
export class Clip extends TimeLineElement {
    get primaryLayer() {
        return this.layers[0];
    }
    set primaryLayer(value) {
        if (value) {
            this.layers[0] = value;
        }
    }
    get languages() {
        // Return all languages from all layers distinct
        const result = this.layers
            .map((layer) => layer.language)
            .filter((value, index, self) => self.indexOf(value) === index);
        return result;
    }
    // #region OVERRIDES
    get absoluteRange() {
        return this._absoluteRange;
    }
    set absoluteRange(value) {
        this._absoluteRange = value;
        this.layers.forEach((layer) => {
            layer.setAbsoluteTimingFromParent(this);
            layer.fixCutRange();
        });
    }
    get innerBoundaryRange() {
        var _b;
        return (_b = this.primaryLayer) === null || _b === void 0 ? void 0 : _b.innerBoundaryRange;
    }
    set innerBoundaryRange(value) {
        if (this.primaryLayer)
            this.primaryLayer.innerBoundaryRange = value;
    }
    get outerBoundaryRange() {
        var _b, _c;
        return (_c = (_b = this.primaryLayer) === null || _b === void 0 ? void 0 : _b.outerBoundaryRange) !== null && _c !== void 0 ? _c : FrameRange.positive;
    }
    set outerBoundaryRange(value) {
        if (this.primaryLayer)
            this.primaryLayer.outerBoundaryRange = value;
    }
    // #endregion
    get rangeInsideClip() {
        return new FrameRange(0, this.relativeRange.duration);
    }
    get libId() {
        var _b, _c;
        return (_c = (_b = this.primaryLayer) === null || _b === void 0 ? void 0 : _b.libId) !== null && _c !== void 0 ? _c : null;
    }
    set libId(value) {
        if (this.primaryLayer)
            this.primaryLayer.libId = value;
    }
    // Constructor() {
    // }
    constructor(partial) {
        var _b;
        super(partial);
        this.__CLIP__ = "Clip";
        this[_a] = true; // Option 1
        this.layers = [];
        if (partial) {
            if (this.name)
                this.name = partial.name;
            if (this.layers)
                this.layers = (_b = partial.layers) !== null && _b !== void 0 ? _b : [];
        }
    }
    setDuration(value, factor = 1) {
        const duration = Math.round(value * factor);
        const newRelativeRange = this.relativeRange.withDurationFromStart(duration, false);
        this.relativeRange = newRelativeRange;
        this.absoluteRange.duration = duration;
    }
    setTransitionsFromIClip(iClip, fps, start, end) {
        super.setTransitionsFromIClip(iClip, fps, start, end);
    }
    clearStartEndTransitions() {
        var _b;
        super.clearStartEndTransitions();
        (_b = this.primaryLayer) === null || _b === void 0 ? void 0 : _b.clearStartEndTransitions();
        // This.layers.forEach((layer) => {
        //   layer.clearStartEndTransitions();
        // });
    }
    trimmLeft(delta, moveToRight = false, absMoveToRight = false) {
        var _b;
        super.trimmLeft(delta, moveToRight, absMoveToRight);
        (_b = this.primaryLayer) === null || _b === void 0 ? void 0 : _b.trimmLeft(delta, moveToRight, absMoveToRight);
        this.layers
            .filter((layer) => layer !== this.primaryLayer && layer.lockedAtParent)
            .forEach((layer) => {
            layer.trimmLeft(delta, moveToRight, absMoveToRight);
        });
    }
    trimmRight(delta) {
        var _b;
        super.trimmRight(delta);
        (_b = this.primaryLayer) === null || _b === void 0 ? void 0 : _b.trimmRight(delta);
        this.layers
            .filter((layer) => layer !== this.primaryLayer && layer.lockedAtParent)
            .forEach((layer) => {
            layer.trimmRight(delta);
        });
    }
    clearLayers() {
        this.layers = [];
    }
    setLayers(items) {
        this.layers = items;
    }
    addLayer(layer, inheritTransitions = false) {
        const item = layer; // _.cloneDeep(layer);
        const timingAction = this.shouldTrim(item)
            ? TimingAction.Trim
            : TimingAction.Expand;
        const startPadding = this.getTransitionPaddingStart(item);
        const endPadding = this.getTransitionPaddingEnd(item);
        item.setRelativeTiming(this.relativeRange, startPadding, endPadding, timingAction);
        if (inheritTransitions) {
            const opacityIn = this.firstStartTransitionByProperyName("opacity");
            const opacityOut = this.firstEndTransitionByProperyName("opacity");
            const volumeIn = this.firstStartTransitionByProperyName("volume");
            const volumeOut = this.firstEndTransitionByProperyName("volume");
            const transitions = [];
            if (opacityIn)
                transitions.push(opacityIn);
            if (opacityOut)
                transitions.push(opacityOut);
            if (volumeIn)
                transitions.push(volumeIn);
            if (volumeOut)
                transitions.push(volumeOut);
            item.transitions = transitions;
        }
        this.layers.push(item);
    }
    shouldTrim(layer) {
        return ((layer instanceof AudioLayer ||
            layer instanceof SlideInTextLayer ||
            layer instanceof TitleLayer ||
            layer instanceof LottieLayer ||
            layer instanceof LowerThirdLayer) &&
            layer.relativeRange.isInside(this.relativeRange.startAt(0)));
    }
    getTransitionPaddingStart(layer) {
        var _b, _c;
        if (this.shouldTrim(layer)) {
            return (_c = (_b = this.inTransition) === null || _b === void 0 ? void 0 : _b.halfDuration) !== null && _c !== void 0 ? _c : 0;
        }
        return 0;
    }
    getTransitionPaddingEnd(layer) {
        var _b, _c;
        if (this.shouldTrim(layer)) {
            return (_c = (_b = this.outTransition) === null || _b === void 0 ? void 0 : _b.halfDuration) !== null && _c !== void 0 ? _c : 0;
        }
        return 0;
    }
    static getRange(clip, lastClip) {
        const self = Clip.fromClip(clip);
        return self.getRange(lastClip);
    }
    getRange(previousClip) {
        var _b, _c, _d, _e, _f, _g, _h, _j;
        if (!previousClip)
            return this.relativeRange.clone();
        let start = (_c = (_b = previousClip.absoluteRange) === null || _b === void 0 ? void 0 : _b.end) !== null && _c !== void 0 ? _c : 0;
        if ((_d = previousClip.primaryLayer) === null || _d === void 0 ? void 0 : _d.absoluteRange)
            start = (_e = previousClip.primaryLayer) === null || _e === void 0 ? void 0 : _e.absoluteRange.end;
        if (previousClip.outTransition) {
            const overlap = Math.max(0, (_g = (_f = previousClip.outTransition) === null || _f === void 0 ? void 0 : _f.relativeRange.duration) !== null && _g !== void 0 ? _g : 0);
            start = Math.max(0, start - overlap);
        }
        else if (this.inTransition) {
            const overlap = Math.max(0, (_j = (_h = this.inTransition) === null || _h === void 0 ? void 0 : _h.relativeRange.duration) !== null && _j !== void 0 ? _j : 0);
            start = Math.max(0, start - overlap);
        }
        const sequenceRange = new FrameRange(start, start + this.relativeRange.duration);
        return sequenceRange;
    }
    getNextClipRange(lastFrame, nextClip) {
        var _b;
        let startFrame = lastFrame;
        if (this.primaryLayer) {
            startFrame = this.absoluteRange.addRelative(this.primaryLayer.relativeRange, TimingAction.Expand).end;
        }
        if (this.outTransition) {
            const offset = Math.max(0, (_b = this.outTransition.relativeRange.duration) !== null && _b !== void 0 ? _b : 0);
            startFrame = Math.max(0, startFrame - offset);
        }
        const sequenceRange = new FrameRange(startFrame, startFrame + nextClip.relativeRange.duration);
        return sequenceRange;
    }
    toIClip(fps) {
        var _b;
        const reult = {
            id: (_b = this.id) !== null && _b !== void 0 ? _b : null,
            name: this.name,
            duration: this.relativeRange.duration / fps,
            layers: this.getILayers(),
            transition: null,
        };
        return reult;
    }
    getILayers() {
        const result = this.layers.map((layer) => {
            const iLayer = layer.getILayer();
            return iLayer;
        });
        return result;
    }
    print() {
        const ilayers = this.getILayers();
        const str = stringify(ilayers, null, 2);
        console.log("Clip", this);
        console.log("Clip.layers", str);
    }
    static iLayerWithTransitions(layer, transitions) {
        var _b;
        const result = layer;
        result.transitions = distinctTransitions([
            ...((_b = layer.transitions) !== null && _b !== void 0 ? _b : []),
            ...transitions,
        ]);
        return result;
    }
    static fromFrames(frames) {
        const clip = new Clip();
        clip.setDuration(frames);
        return clip;
    }
    static fromSeconds(seconds, fps = 1) {
        const clip = new Clip();
        clip.setDuration(seconds, fps);
        return clip;
    }
    static fromClip(clip) {
        const result = plainToInstance(Clip, clip);
        return result;
    }
    static fromIClip(data, fps) {
        const clip = new Clip();
        clip.setDuration(data.duration, fps);
        return clip;
    }
    static fromLayer(layer, fps = 1) {
        const clip = new Clip();
        clip.setDuration(layer.relativeRange.duration, fps);
        clip.addLayer(layer);
        return clip;
    }
    static setAboluteRanges() { }
    static emptyClip(durationInFrames) {
        const clip = new Clip();
        clip.setDuration(durationInFrames !== null && durationInFrames !== void 0 ? durationInFrames : 25);
        return clip;
    }
}
_a = immerable;
__decorate([
    Type(() => Layer, {
        discriminator: {
            property: "type",
            subTypes: [
                { value: AudioLayer, name: LayerType.Audio },
                { value: VideoLayer, name: LayerType.Video },
                { value: ImageLayer, name: LayerType.Image },
                { value: SubtitleLayer, name: LayerType.Subtitle },
                { value: LottieLayer, name: LayerType.Lottie },
            ],
        },
    }),
    __metadata("design:type", Array)
], Clip.prototype, "layers", void 0);
// #region EDIT SPECIFICATION
export function clipsFromEditSpecification(editSpecification, videoSettings) {
    if (verbose)
        console.log("EditlyService.editSpecificationToClips");
    if (!editSpecification.clips) {
        throw new Error("EditlyService.editSpecificationToClips editSpecification.clips is null");
    }
    const clips = editSpecification.clips.map((iClip, index) => {
        var _b;
        if (!iClip.layers) {
            throw new Error(`EditlyService.editSpecificationToClips #${index} iClip.layers is null`);
        }
        const durationInSeconds = (_b = iClip.duration) !== null && _b !== void 0 ? _b : longestDuration(iClip.layers);
        const clip = clipFromILayers(iClip.layers, editSpecification.assets, videoSettings, durationInSeconds, editSpecification.defaults, index === 0, index === editSpecification.clips.length - 1);
        if (clip && iClip.id)
            clip.id = iClip.id;
        if (clip && iClip.name)
            clip.name = iClip.name;
        if (!(clip instanceof Clip))
            throw new Error("EditlyService.editSpecificationToClips clip is null");
        return clip;
    });
    const truecyClips = clips.filter((x) => x !== undefined);
    setClipTiming(truecyClips);
    if (verbose) {
        // TruecyClips.forEach((clip) => {
        //   clip.print();
        // });
    }
    return truecyClips;
}
export function setClipTiming(clips) {
    let lastFrame = 0;
    clips.forEach((clip, index) => {
        if (clip) {
            const previousClip = clips[index - 1];
            let newAbsoluteRange = clip.relativeRange.add(lastFrame);
            if (previousClip)
                newAbsoluteRange = previousClip.getNextClipRange(lastFrame, clip);
            clip.absoluteRange = newAbsoluteRange;
            clip.layers.forEach((layer) => {
                layer.setAbsoluteTimingFromParent(clip);
                layer.fixCutRange();
            });
            lastFrame = newAbsoluteRange.end;
        }
    });
}
export function withClipTiming(clips) {
    let lastFrame = 0;
    const result = clips.map((clip, index) => {
        const previousClip = clips[index - 1];
        let newAbsoluteRange = clip.relativeRange.add(lastFrame);
        if (previousClip)
            newAbsoluteRange = previousClip.getNextClipRange(lastFrame, clip);
        const updatedClip = produce(clip, (draft) => {
            draft.absoluteRange = newAbsoluteRange;
            draft.layers.forEach((layer) => {
                layer.setAbsoluteTimingFromParent(clip);
                layer.fixCutRange();
            });
        });
        lastFrame = newAbsoluteRange.end;
        return updatedClip;
    });
    return result;
}
export function setClipTransitions(clips, editSpecification) {
    if (!editSpecification.defaults || !editSpecification.videoSettings)
        return;
    clips.forEach((clip, index) => {
        if (clip) {
            const addStartTransition = index > 0;
            const addEndTransition = index < clips.length - 1;
            const setTrans = clip.transitions.length === 0;
            if (setTrans)
                clip.setTransitionsFromIClip(editSpecification.defaults, editSpecification.videoSettings.frameRate, addStartTransition, addEndTransition);
        }
    });
}
export function withClipTransitions(clips, editSpecification) {
    if (!editSpecification.defaults || !editSpecification.videoSettings)
        return clips;
    return clips.map((clip, index) => {
        const addStartTransition = index > 0;
        const addEndTransition = index < clips.length - 1;
        const setTrans = clip.transitions.length === 0;
        if (setTrans) {
            return produce(clip, (draft) => {
                draft.setTransitionsFromIClip(editSpecification.defaults, editSpecification.videoSettings.frameRate, addStartTransition, addEndTransition);
            });
        }
        return clip;
    });
}
export function longestDuration(iLayers) {
    var _b;
    let duration = 0;
    for (const layer of iLayers) {
        if (layer.lockedAtParent)
            continue;
        duration = Math.max(duration, (_b = layer.duration) !== null && _b !== void 0 ? _b : 0);
    }
    return duration;
}
// eslint-disable-next-line max-params
export function clipFromILayers(iLayers, assets, videoSettings, durationInSeconds, defaults, first = false, last = false) {
    if (verbose)
        console.log(`Clip.fromILayers #1 iLayers ${iLayers === null || iLayers === void 0 ? void 0 : iLayers.length}, first: ${first}, last: ${last}`);
    const clip = new Clip();
    const durationInFrames = Frame.framesFromSeconds(durationInSeconds, videoSettings.frameRate);
    clip.setDuration(durationInFrames);
    if (iLayers.length === 0)
        return clip;
    const layers = iLayers.map((iLayer, index) => {
        return layerFromILayer(_.cloneDeep(iLayer), assets, videoSettings, durationInSeconds);
    });
    const primaryLayer = layers.at(0);
    if (primaryLayer) {
        // if (
        //   primaryLayer instanceof VideoLayer &&
        //   primaryLayer.transitions.length > 0
        // ) {
        //   clip.setTransitionsFromLayer(primaryLayer);
        // } else if (defaults)
        if (defaults)
            clip.setTransitionsFromIClip(defaults, videoSettings.frameRate, !first, !last);
    }
    if (verbose)
        console.log(`Clip.fromILayers #2 Layers: ${layers === null || layers === void 0 ? void 0 : layers.length}`, clip.constructor.name);
    let i = 0;
    for (const layer of layers) {
        clip.addLayer(layer, i === 0);
        i++;
    }
    const lastFrame = Layer.getlastFrameFromLayers(clip.layers, clip.relativeRange);
    if (clip.relativeRange.end < lastFrame)
        clip.relativeRange.end = lastFrame;
    if (verbose) {
        console.log(`Clip.fromILayers #3`, clip.constructor.name, clip);
    }
    return clip;
}
export function editSpecificationToClips(data, videoSettings, caller) {
    let editSpecification;
    if (verbose)
        console.log(`Clip.calculateClips ${caller}`);
    if (data instanceof EditSpecification) {
        editSpecification = plainToInstance(EditSpecification, data);
        if (verbose)
            console.log(`Clip.calculateClips instanceof EditSpecification`);
    }
    else if (typeof data === "object") {
        editSpecification = plainToInstance(EditSpecification, data);
        if (verbose)
            console.log(`Clip.calculateClips typeof object`);
    }
    else if (typeof data === "string") {
        editSpecification = EditSpecification.fromJSON(data);
        if (verbose)
            console.log(`Clip.calculateClips typeof string`);
    }
    else {
        throw new Error("Clip.calculateClips jsonData is not valid");
    }
    const clips = clipsFromEditSpecification(editSpecification, videoSettings !== null && videoSettings !== void 0 ? videoSettings : editSpecification.videoSettings);
    if (verbose)
        console.log(`Clip.calculateClips Clips ${clips.length}`);
    return clips;
}
export function clipsToEditSpecification(clips, origin, durationInFrames) {
    const result = instanceToInstance(origin);
    result.clips = clips.map((clip) => {
        return clip.toIClip(origin.fps);
    });
    if (durationInFrames) {
        result.durationInFrames = durationInFrames;
        result.videoSettings.durationInFrames = durationInFrames;
    }
    return result;
}
// #endregion
//# sourceMappingURL=clip.js.map