var _a;
import { immerable } from "immer";
import { CssUnit } from "../enums/index.js";
export class CssValue {
    constructor() {
        this[_a] = true; // Option 1
        this.value = 0;
        this.unit = CssUnit.percent;
        this.toString = () => {
            return `${Math.round(this.value * 100) / 100}${this.unit}`;
        };
    }
    static fromPercent(valueInPercent) {
        const result = new CssValue();
        result.value = valueInPercent;
        result.unit = CssUnit.percent;
        return result;
    }
    static fromPixel(pixel) {
        const result = new CssValue();
        result.value = pixel;
        result.unit = CssUnit.pixel;
        return result;
    }
    static getCanvas() {
        let canvas;
        // Check if in browser or Node.js environment
        // eslint-disable-next-line no-negated-condition
        if (typeof document !== "undefined") {
            // Browser environment
            canvas = document.createElement("canvas");
        }
        else {
            // Node.js environment
            throw new Error("Node.js environment not supported.");
            // CreateCanvas(512, 512); // Dimensions are arbitrary for text measurement
        }
        return canvas;
    }
    /**
     * Uses canvas.measureText to compute and return the width of the given text of given font in pixels.
     *
     * @param {String} text The text to be rendered.
     * @param {String} font The css font descriptor that text is to be rendered with (e.g. "bold 14px verdana").
     *
     * @see https://stackoverflow.com/questions/118241/calculate-text-width-with-javascript/21015393#21015393
     */
    static measureTextWidth(text, font = "16px Arial") {
        const canvas = CssValue.getCanvas();
        if (!canvas)
            return 0;
        const context = canvas.getContext("2d");
        context.font = font;
        return context.measureText(text).width;
    }
    static getTextWidth(text, fontFamily, fontSize, fontWeight) {
        return CssValue.measureTextWidth(text, `${fontWeight} ${fontSize} ${fontFamily}`);
    }
    static getTextWidthFromITextSettings(text, settings) {
        return CssValue.measureTextWidth(text, `${settings.fontWeight} ${settings.fontSize} ${settings.fontFamily}`);
    }
    /**
     * Uses canvas.measureText to compute and return the width of the given text of given font in pixels.
     *
     * @param {String} text The text to be rendered.
     * @param {String} font The css font descriptor that text is to be rendered with (e.g. "bold 14px verdana").
     *
     * @see https://stackoverflow.com/questions/118241/calculate-text-width-with-javascript/21015393#21015393
     */
    static measureTextHeight(text, font) {
        const canvas = CssValue.getCanvas();
        if (!canvas)
            return 0;
        const context = canvas.getContext("2d");
        context.font = font;
        const metrics = context.measureText(text);
        return metrics.fontBoundingBoxAscent + metrics.fontBoundingBoxDescent;
    }
    static getTextHeight(text, fontFamily, fontSize, fontWeight) {
        return CssValue.measureTextHeight(text, `${fontWeight} ${fontSize} ${fontFamily}`);
    }
    static getTextHeightFromITextSettings(text, settings) {
        return CssValue.measureTextHeight(text, `${settings.fontWeight} ${settings.fontSize} ${settings.fontFamily}`);
    }
}
_a = immerable;
//# sourceMappingURL=cssValue.js.map