import { immerable } from "immer";
import { Anchor, TimingAction } from "../enums/index.js";
import { IFrameRange } from "../interfaces/index.js";
interface ISplit {
    first: FrameRange;
    last: FrameRange;
}
export declare class FrameRange implements IFrameRange {
    [immerable]: boolean;
    private callback;
    private args;
    use(callback: (...args: any[]) => void, ...args: any[]): FrameRange;
    execute(condition?: boolean): FrameRange;
    start: number;
    end: number;
    anchor?: Anchor | null;
    get center(): number;
    get duration(): number;
    set duration(value: number);
    set durationFromEnd(value: number);
    static fromSeconds(start: number, end: number, fps: number, anchor?: Anchor | null): FrameRange;
    static fromMilliseconds(start: number, end: number, fps: number, anchor?: Anchor | null): FrameRange;
    static fromIFrameRange(item: IFrameRange): FrameRange;
    static get positive(): FrameRange;
    static get negative(): FrameRange;
    static get zero(): FrameRange;
    static get max(): FrameRange;
    static get minClipRange(): FrameRange;
    constructor(start?: number, end?: number, anchor?: Anchor | null, zeroLength?: boolean);
    aAndNotB(b: FrameRange): FrameRange[];
    aIntersectB(b: FrameRange): FrameRange;
    aXorB(b: FrameRange): FrameRange[];
    union(b: FrameRange | FrameRange[]): FrameRange[];
    static unionOfAll(ranges: FrameRange[]): FrameRange[];
    private combineFrameRanges;
    toIFrameRange(): IFrameRange;
    withPadding(paddingStart: number | null, paddingEnd?: number | null, zeroLength?: boolean | undefined): FrameRange;
    withDurationFromStart(duration: number, limit?: boolean): FrameRange;
    scale(factor: number): FrameRange;
    scaleDuration(scale: number): FrameRange;
    scaleDurationFromStart(factor: number): FrameRange;
    scaleDurationFromCenter(factor: number): FrameRange;
    scaleDurationFromEnd(factor: number): FrameRange;
    shiftLeft(value: number): FrameRange;
    shiftRight(value: number): FrameRange;
    trimmLeft(value: number): FrameRange;
    trimmRight(value: number): FrameRange;
    add(value: number, minStart?: number, maxEnd?: number, action?: TimingAction): FrameRange;
    addStart(value: number): FrameRange;
    addEnd(value: number): FrameRange;
    addRelative(relative: FrameRange, timing?: TimingAction): FrameRange;
    subtract(value: number): FrameRange;
    subtractStart(value: number): FrameRange;
    subtractEnd(value: number): FrameRange;
    split(): ISplit;
    withDurationFromEnd(duration: number): FrameRange;
    withAnchor(anchor?: Anchor | undefined | null): FrameRange;
    clearAnchor(): FrameRange;
    getFadeIn(fadeDuration: number, padding?: number): Array<number>;
    getFadeOut(fadeDuration: number, padding?: number): Array<number>;
    getFadeInOut(fadeDuration: number, padding?: number): Array<number>;
    getFadeInOutAtStart(fadeDuration: number, visibleDuration: number, padding?: number): Array<number>;
    getFadeInOutAtEnd(fadeDuration: number, visibleDuration: number, padding?: number): Array<number>;
    getFadeInOutWithOther(other: FrameRange): Array<number>;
    getFade(): Array<number>;
    getCutCenter(): Array<number>;
    getCutCenterInOutWithOther(other: FrameRange): Array<number>;
    getRelativeFrame(frame: number): number;
    getRelativeFrameClamp(frame: number): number;
    startAt(start?: number): FrameRange;
    endDuration(duration: number): void;
    calculateNormPosition(frame: number): number;
    getIntersection(other: FrameRange): FrameRange | null;
    isEmpty(): boolean;
    isDefault(): boolean;
    isValid(): boolean;
    isIntersecting(other: FrameRange): boolean;
    isBefore(other: FrameRange): boolean;
    isBehind(other: FrameRange): boolean;
    isInside(other: FrameRange | undefined): boolean;
    startIsEqual(other: FrameRange | undefined): boolean;
    endIsEqual(other: FrameRange | undefined): boolean;
    isEqual(other: FrameRange | undefined): boolean;
    containsFrame(frame: number): boolean;
    isInDurationRange(min?: number, max?: number): boolean;
    deltaStart(other: FrameRange): number;
    deltaEnd(other: FrameRange): number;
    addPadding(padding: number, paddingEnd?: number | null): FrameRange;
    addPaddingLeft(padding: number): FrameRange;
    tryAddPaddingLeft(padding: number): FrameRange | undefined;
    addPaddingRight(padding: number): FrameRange;
    tryAddPaddingRight(padding: number): FrameRange | undefined;
    round(): FrameRange;
    pushOtherBehind(other: FrameRange, padding?: number): FrameRange;
    pushOtherInFront(other: FrameRange, padding?: number): FrameRange;
    limitDuration(maxDuration?: number): FrameRange;
    pushInside(other: FrameRange, action?: TimingAction): FrameRange;
    clamp(other: FrameRange): FrameRange;
    pushApart(other?: FrameRange): FrameRange;
    expand(other: FrameRange, padding?: number): FrameRange;
    clone(): FrameRange;
    toString: () => string;
    static compareStartAndEnd(a: FrameRange, b: FrameRange): number;
    static compareStart(a: FrameRange, b: FrameRange): number;
    static compareEnd(a: FrameRange, b: FrameRange): number;
    static compareDuration(a: FrameRange, b: FrameRange): number;
    static combineRelativeAndCut(relativeRange: FrameRange, cutRange?: FrameRange | null | undefined): FrameRange;
    static empty(): FrameRange;
    static fromStart(startFrame: number, duration: number, anchor?: Anchor | null): FrameRange;
    static fromCenter(center: number, duration: number, anchor?: Anchor | null): FrameRange;
    static fromEnd(lastFrame: number, duration: number, anchor?: Anchor | null): FrameRange;
    static first(frames: FrameRange[]): FrameRange | null;
    static last(frames: FrameRange[]): FrameRange | null;
    static withStart(frames: FrameRange[], start: number): FrameRange | null;
    static withEnd(frames: FrameRange[], end: number): FrameRange | null;
    static areEqual(a: FrameRange, b: FrameRange): boolean;
    static arrayEquals(a: FrameRange[], b: FrameRange[]): boolean;
    /**
     * A \ B (also written A − B[]) is the set of all things that belong to A but not B[]
     */
    static differences(a: FrameRange, bItems: FrameRange[]): FrameRange[];
    private static differencesRecursive;
    /**
     * A \ B (also written A − B) is the set of all things that belong to A but not B
     */
    static difference(a: FrameRange, b: FrameRange): FrameRange[];
    static distinctFrameRanges(ranges: FrameRange[]): FrameRange[];
    static distinctPartition(segments: any[], frameRangePropertyName?: string): FrameRange[];
}
export declare function millisecondsToFps(milliseconds: number, fps: number): number;
export declare function framesToMilliseconds(frames: number, fps: number): number;
export {};
//# sourceMappingURL=frameRange.d.ts.map