var _a;
import "reflect-metadata";
import { immerable } from "immer";
import { FrameRange } from "./frameRange.js";
import { FrameRangeCollection } from "./frameRangeCollection.js";
export class FrameRangeItemCollection {
    constructor() {
        this[_a] = true; // Option 1
        this.collection = [];
    }
    get empty() {
        return this.collection.length === 0;
    }
    get lastItem() {
        return this.collection[this.collection.length - 1];
    }
    get lastFrame() {
        var _b, _c;
        return (_c = (_b = this.lastItem) === null || _b === void 0 ? void 0 : _b.range.end) !== null && _c !== void 0 ? _c : 0;
    }
    get lastFrameScaled() {
        var _b, _c;
        return (_c = (_b = this.lastItem) === null || _b === void 0 ? void 0 : _b.scaledRange.end) !== null && _c !== void 0 ? _c : 0;
    }
    append(item, atEnd = true) {
        if (atEnd) {
            const shifted = Object.assign(Object.assign({}, item), { range: FrameRange.fromStart(this.lastFrame + (this.empty ? 0 : 1), item.range.duration).withAnchor(item.range.anchor), scaledRange: FrameRange.fromStart(this.lastFrameScaled + (this.empty ? 0 : 1), item.scaledRange.duration).withAnchor(item.scaledRange.anchor) });
            this.collection.push(shifted);
            return;
        }
        this.collection.push(item);
    }
    findIndexContainsFrame(frame, scaled = false) {
        return this.collection.findIndex((x) => (scaled ? x.scaledRange : x.range).containsFrame(frame));
    }
    includingFrame(frame, scaled = false) {
        return this.collection.filter((x) => {
            return (scaled ? x.scaledRange : x.range).containsFrame(frame);
        });
    }
    leftOfFrame(frame, scaled = false) {
        return this.collection.filter((x) => (scaled ? x.scaledRange.end : x.range.end) < frame);
    }
    leftOfAndIncludingFrame(frame, scaled = false) {
        const left = this.leftOfFrame(frame, scaled);
        const including = this.includingFrame(frame, scaled);
        return [...left, ...including];
    }
    rightOfFrame(frame, scaled = false) {
        return this.collection.filter((x) => (scaled ? x.scaledRange.start : x.range.start) > frame);
    }
    rightOfAndIncludingFrame(frame, scaled = false) {
        const right = this.rightOfFrame(frame, scaled);
        const including = this.includingFrame(frame, scaled);
        return [...right, ...including];
    }
    toFrameRangeCollection() {
        const collection = new FrameRangeCollection();
        const ranges = this.collection.map((x) => x.range);
        const scaledRanges = this.collection.map((x) => x.scaledRange);
        collection.collection = ranges !== null && ranges !== void 0 ? ranges : [];
        collection.collectionScaled = scaledRanges !== null && scaledRanges !== void 0 ? scaledRanges : [];
        return collection;
    }
}
_a = immerable;
//# sourceMappingURL=frameRangeItemCollection.js.map