import "reflect-metadata";
import { LayerType, TimingAction } from "../../enums/index.js";
import { LanguageKey, IProperty, IVariant, ILayer, IVideoSettings, AssetKey } from "../../interfaces/index.js";
import { PlaybackMode } from "../../lottieWrapper.js";
import { LottieData } from "../../types/index.js";
import { NormalizedSize } from "../normalizedSize.js";
import { FrameRange } from "../frameRange.js";
import { LottieAnimation } from "../lottieAnimation.js";
import { ElementProps } from "../../interfaces/elementProps.js";
import { Position } from "../position.js";
import { Layer, MediaAssetVariant } from "../index.js";
import { FrameRangeItemCollection, IFrameRangeScaledItem } from "../frameRangeItemCollection.js";
export declare class LottieLayer extends Layer {
    path: string | null;
    duration: number;
    playbackMode: PlaybackMode;
    __LAYER__: string;
    private _lottieAnimation;
    get lottieAnimation(): LottieAnimation;
    get minDuration(): number | undefined;
    get maxDuration(): number | undefined;
    translateX: number;
    private _propertiesLocale;
    private get _propertiesLocaleMap();
    get propertiesLocale(): IVariant<LanguageKey, IProperty[]>[];
    set propertiesLocale(value: IVariant<LanguageKey, IProperty[]>[]);
    get languageKey(): LanguageKey;
    get properties(): Map<string, IProperty>;
    set properties(value: Map<string, IProperty>);
    getPropertiesLocale(language: string, createIfMissing?: boolean): Map<string, IProperty>;
    setPropertiesLocale(language: string, value: Map<string, IProperty>): void;
    get languages(): string[];
    get textProperties(): IProperty[];
    get imageProperties(): IProperty[];
    get colorProperties(): IProperty[];
    constructor(layer: ILayer, asset: MediaAssetVariant<AssetKey> | undefined, path: string | null, lottieData: LottieData | string, duration: number, playbackMode: PlaybackMode, position?: Position | null, size?: NormalizedSize | null, type?: LayerType, name?: string);
    static fromILayer(layer: ILayer, assets: MediaAssetVariant<AssetKey>[], videoSettings: IVideoSettings, duration: number): Layer;
    getILayer(): ILayer;
    get lottieSizeInPixel(): number;
    setRelativeTiming(parentInsideRange: FrameRange, startOffset?: number, endOffset?: number, timingAction?: TimingAction): void;
    getTitle(): string;
    toInfoString(): string;
    getPauseRange(): FrameRange | undefined;
    getBreakRanges(startOffset?: number): FrameRange[] | undefined;
    propertiesToAllLocale(): boolean;
    copyPropertiesToLocale(targetLanguageCode?: string, sourceLanguageCode?: string): boolean;
    copyPropertiesLocaleByLanguages(languageCodes: string[], sourceLanguageCode?: string): boolean;
    upsertPropertiesLocaleByLanguages(languageCodes: string[], sourceLanguageCode?: string): boolean;
    getProperties(sourceLanguageCode?: string, createIfMissing?: boolean): IProperty[];
    setProperties(properties: IProperty[], languageCode?: string): void;
    setProperty(name: string, value: IProperty, languageCode?: string): void;
    updatePropertyValueAllLanguages(propertyName: string, value: any): void;
    updatePropertyValue(propertyName: string, value: any, languageCode?: string): void;
    fillProperties(lottieAnimation: LottieAnimation): void;
    hydrateLottieAnimationByProperties(animation: LottieAnimation): LottieAnimation;
    fetchAndFill(): Promise<void>;
    clone(): LottieLayer;
    nextFrame(frame: number, absoluteRange: FrameRange, lottieDuration: number, breaks?: FrameRange[] | null): number;
    clipPauseSequenceScaled(sequenceRange: FrameRange): FrameRangeItemCollection<IFrameRangeScaledLottieItem> | undefined;
    private clipPauseSequence;
}
export declare const LottieComponent: ({ sequenceRange, index, layer, path, selectors, }: ElementProps) => import("react/jsx-runtime").JSX.Element;
export declare function fetchDataAsync(path: string): Promise<LottieData>;
export interface IFrameRangeScaledLottieItem extends IFrameRangeScaledItem {
    type: "animation" | "pause";
}
//# sourceMappingURL=lottie.layer.d.ts.map