var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsxs as _jsxs, Fragment as _Fragment, jsx as _jsx } from "react/jsx-runtime";
/* eslint-disable spaced-comment */
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import _ from "lodash";
import { useEffect, useMemo, useState } from "react";
import { parse } from "flatted";
import { Freeze, Sequence, useCurrentFrame, delayRender, cancelRender, continueRender, useVideoConfig, } from "remotion";
import { Type, plainToClass } from "class-transformer";
import { devModeContext, disableContext } from "./devModeContext.js";
import { LayerType, TimingAction } from "../../enums/index.js";
import { PlaybackMode, getLottieSizeInPixel } from "../../lottieWrapper.js";
import { Frame } from "../frame.js";
import { FrameRange } from "../frameRange.js";
import { LottieAnimation } from "../lottieAnimation.js";
import { VariantMap } from "../variantMap.js";
import { Position } from "../position.js";
import { Layer } from "../index.js";
import RenderGuard from "../components/renderGuard.component.js";
import { Box, BoxStyle } from "../boxModel/index.js";
import { FrameRangeItemCollection, } from "../frameRangeItemCollection.js";
export class LottieLayer extends Layer {
    get lottieAnimation() {
        return this._lottieAnimation;
    }
    get minDuration() {
        return this.lottieAnimation.frameDurationWithoutPause + 1;
    }
    get maxDuration() {
        return this.outerBoundaryRange.duration;
    }
    get _propertiesLocaleMap() {
        const map = new VariantMap(this._propertiesLocale.map((item) => {
            return {
                key: item.key,
                value: [...item.value],
            };
        }));
        map.setDefault([]);
        map.onChange = (collection) => {
            this._propertiesLocale = collection.map((iVariant) => {
                return {
                    key: iVariant.key,
                    value: [...iVariant.value],
                };
            });
        };
        return map;
    }
    get propertiesLocale() {
        return this._propertiesLocale;
    }
    set propertiesLocale(value) {
        this._propertiesLocale = value;
    }
    get languageKey() {
        return { languageCode: this.language };
    }
    get properties() {
        var _a;
        const propertiesArray = (_a = this._propertiesLocaleMap.get(this.languageKey)) !== null && _a !== void 0 ? _a : [];
        return new Map(propertiesArray.map((x) => [x.name, x]));
    }
    set properties(value) {
        const propertiesArray = [...value.values()];
        this._propertiesLocaleMap.set(this.languageKey, propertiesArray);
    }
    getPropertiesLocale(language, createIfMissing) {
        var _a;
        const propertiesArray = (_a = this._propertiesLocaleMap.get({ languageCode: language }, createIfMissing)) !== null && _a !== void 0 ? _a : [];
        return new Map(propertiesArray.map((x) => [x.name, x]));
    }
    setPropertiesLocale(language, value) {
        const propertiesArray = [...value.values()];
        this._propertiesLocaleMap.set({ languageCode: language }, propertiesArray);
    }
    get languages() {
        const { keys } = this._propertiesLocaleMap;
        return keys.map((x) => x.languageCode);
    }
    get textProperties() {
        return [...this.properties.values()].filter((x) => x.type === "text");
    }
    get imageProperties() {
        return [...this.properties.values()].filter((x) => x.type === "image");
    }
    get colorProperties() {
        return [...this.properties.values()].filter((x) => x.type === "color");
    }
    // eslint-disable-next-line max-params
    constructor(layer, asset, path, lottieData, duration, playbackMode, position = null, size = null, type = LayerType.Lottie, name = "") {
        super(type, layer, asset, position, size, name);
        this.path = path;
        this.duration = duration;
        this.playbackMode = playbackMode;
        this.__LAYER__ = "LottieLayer";
        this.translateX = 0;
        this._propertiesLocale = [];
        const lottieDataObj = (typeof lottieData === "string"
            ? parse(lottieData)
            : _.cloneDeep(lottieData));
        this._lottieAnimation = new LottieAnimation(lottieDataObj, path !== null && path !== void 0 ? path : undefined);
        this.fillProperties(this._lottieAnimation);
    }
    static fromILayer(layer, assets, videoSettings, duration) {
        var _a, _b, _c;
        const iLayer = _.cloneDeep(layer);
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (videoSettings.frameRate <= 0)
            throw new Error(`LottieClip fps:${videoSettings.frameRate} must be bigger than 0`);
        const durationInFrames = iLayer.duration && iLayer.duration > 0
            ? Frame.framesFromSeconds(iLayer.duration, videoSettings.frameRate)
            : Frame.framesFromSeconds(duration, videoSettings.frameRate);
        const lottieData = LottieAnimation.LottieDataFrom(iLayer.lottieData);
        iLayer.size = { width: lottieData.w, height: lottieData.h };
        const position = Position.fromIPosition(iLayer.position, Position.centerCenter());
        const normalizedSize = Layer.aspectLayerSize(iLayer, videoSettings);
        const lottieLayer = new LottieLayer(iLayer, asset, path, lottieData, durationInFrames, PlaybackMode.StayAtLastFrame, position, normalizedSize);
        lottieLayer.name = (_b = iLayer.name) !== null && _b !== void 0 ? _b : "LottieAnimation";
        lottieLayer._propertiesLocale = (_c = iLayer.propertiesLocale) !== null && _c !== void 0 ? _c : [];
        return lottieLayer;
    }
    getILayer() {
        var _a;
        const base64 = this.lottieAnimation.toBase64();
        const layer = Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.Lottie, path: (_a = this.path) !== null && _a !== void 0 ? _a : null, propertiesLocale: [...this._propertiesLocale], lottieData: base64 !== null && base64 !== void 0 ? base64 : null });
        return layer;
    }
    get lottieSizeInPixel() {
        if (this.lottieAnimation.lottieData)
            return getLottieSizeInPixel(this.lottieAnimation.lottieData).width;
        return 0;
    }
    setRelativeTiming(parentInsideRange, startOffset = 0, endOffset = 0, timingAction = TimingAction.Expand) {
        const parentWithOffset = parentInsideRange
            .addPaddingLeft(startOffset)
            .addPaddingRight(endOffset);
        switch (timingAction) {
            case TimingAction.Trim:
                if (!this.relativeRange.isInside(parentWithOffset) &&
                    timingAction === TimingAction.Trim)
                    this.relativeRange = parentWithOffset;
                break;
            default:
                this.relativeRange = parentInsideRange.addRelative(this.relativeRange, timingAction);
                break;
        }
        // For (const child of this.children) {
        //   child.relativeRange = layerRange;
        // }
    }
    getTitle() {
        return this.name;
    }
    toInfoString() {
        return `LOTTIE LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.path}`;
    }
    getPauseRange() {
        const marker = this.lottieAnimation.getPauseMarker();
        if (marker && marker.dr > 0)
            return new FrameRange(marker.tm, marker.tm + marker.dr);
        return undefined;
    }
    getBreakRanges(startOffset) {
        return this.lottieAnimation.getBreaksRanges(startOffset);
    }
    propertiesToAllLocale() {
        let copied = false;
        this._propertiesLocaleMap.keys.forEach((key) => {
            if (key.languageCode !== this.language)
                if (this.copyPropertiesToLocale(key.languageCode))
                    copied = true;
        });
        return copied;
    }
    copyPropertiesToLocale(targetLanguageCode, sourceLanguageCode) {
        return this._propertiesLocaleMap.upsert({ languageCode: targetLanguageCode !== null && targetLanguageCode !== void 0 ? targetLanguageCode : this.language }, this.getProperties(sourceLanguageCode));
    }
    copyPropertiesLocaleByLanguages(languageCodes, sourceLanguageCode) {
        let copied = false;
        languageCodes.forEach((target) => {
            if (this.copyPropertiesToLocale(target, sourceLanguageCode))
                copied = true;
        });
        return copied;
    }
    upsertPropertiesLocaleByLanguages(languageCodes, sourceLanguageCode) {
        let upserted = false;
        const missing = languageCodes.filter((x) => {
            var _a;
            return !this._propertiesLocaleMap.exists({ languageCode: x }) ||
                ((_a = this._propertiesLocaleMap.get({ languageCode: x })) === null || _a === void 0 ? void 0 : _a.length) === 0;
        });
        missing.forEach((target) => {
            if (this.copyPropertiesToLocale(target, sourceLanguageCode))
                upserted = true;
        });
        return upserted;
    }
    getProperties(sourceLanguageCode, createIfMissing) {
        const properties = sourceLanguageCode
            ? this.getPropertiesLocale(sourceLanguageCode, createIfMissing)
            : this.properties;
        return Array.from(properties).map(([key, property]) => ({
            name: key,
            value: property.value,
            path: property.path,
            type: property.type,
            width: property.width,
            height: property.height,
            functionDecorators: property.functionDecorators,
        }));
    }
    setProperties(properties, languageCode) {
        properties.forEach((property) => {
            this.setProperty(property.name, property, languageCode);
        });
    }
    setProperty(name, value, languageCode) {
        if (languageCode) {
            const tempProperties = this.getPropertiesLocale(languageCode);
            tempProperties.set(name, value);
            this.setPropertiesLocale(languageCode, tempProperties);
        }
        else {
            const tempProperties = this.properties;
            tempProperties.set(name, value);
            this.properties = tempProperties;
            this.copyPropertiesToLocale();
        }
    }
    updatePropertyValueAllLanguages(propertyName, value) {
        this.languages.forEach((languageCode) => {
            this.updatePropertyValue(propertyName, value, languageCode);
        });
    }
    updatePropertyValue(propertyName, value, languageCode) {
        var _a, _b;
        if (languageCode) {
            const tempProperties = this.getPropertiesLocale(languageCode);
            const origin = (_a = tempProperties.get(propertyName)) !== null && _a !== void 0 ? _a : {};
            tempProperties.set(propertyName, Object.assign(Object.assign({}, origin), { value }));
            this.setPropertiesLocale(languageCode, tempProperties);
        }
        else {
            const tempProperties = this.properties;
            const origin = (_b = tempProperties.get(propertyName)) !== null && _b !== void 0 ? _b : {};
            tempProperties.set(propertyName, Object.assign(Object.assign({}, origin), { value }));
            this.properties = tempProperties;
        }
    }
    fillProperties(lottieAnimation) {
        const tempProperties = this.properties;
        tempProperties.clear();
        lottieAnimation.getTextProperties().forEach((iProperty) => {
            if (!tempProperties.has(iProperty.name))
                tempProperties.set(iProperty.name, iProperty);
        });
        const colorProps = lottieAnimation.getColorProperties();
        colorProps.forEach((iProperty) => {
            if (!tempProperties.has(iProperty.name))
                tempProperties.set(iProperty.name, iProperty);
        });
        const imageProps = lottieAnimation.getImageProperties();
        imageProps.forEach((iProperty) => {
            if (!tempProperties.has(iProperty.name))
                tempProperties.set(iProperty.name, iProperty);
        });
        this.properties = tempProperties;
        this.copyPropertiesToLocale();
    }
    hydrateLottieAnimationByProperties(animation) {
        const hydratedAnimation = _.cloneDeep(animation);
        this.textProperties.forEach((prop) => {
            hydratedAnimation.setTextProperty(prop.name, prop.value);
        });
        this.imageProperties.forEach((prop) => {
            hydratedAnimation.setImageProperty(prop.name, prop.value);
        });
        this.colorProperties.forEach((prop) => {
            hydratedAnimation.setColorProperty(prop);
        });
        return hydratedAnimation;
    }
    fetchAndFill() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.path)
                return;
            const response = yield fetch(this.path);
            const data = (yield response.json());
            const lottieAnimation = new LottieAnimation(data, this.path);
            this.fillProperties(lottieAnimation);
        });
    }
    clone() {
        return plainToClass(LottieLayer, Object.assign({}, this));
    }
    nextFrame(frame, absoluteRange, lottieDuration, breaks) {
        var _a;
        const sumOfBrakes = (_a = breaks === null || breaks === void 0 ? void 0 : breaks.reduce((accumulator, x) => {
            return accumulator + x.duration;
        }, 0)) !== null && _a !== void 0 ? _a : 0;
        if (lottieDuration <= 0)
            return frame;
        const relativeFrame = absoluteRange.getRelativeFrameClamp(frame);
        const sumOfAnimated = lottieDuration - sumOfBrakes;
        const strech = absoluteRange.duration - lottieDuration;
        const scale = sumOfBrakes === 0 ? 1 : (strech + sumOfBrakes) / sumOfBrakes;
        const clipBrakeSequence = this.clipPauseSequenceScaled(absoluteRange);
        if (!clipBrakeSequence)
            return relativeFrame;
        const activeParts = clipBrakeSequence.includingFrame(relativeFrame, true);
        const pastPauses = clipBrakeSequence
            .leftOfFrame(relativeFrame, true)
            .filter((x) => x.type === "pause");
        const summOfPastPauses = pastPauses.reduce((accumulator, x) => {
            return accumulator + x.range.duration;
        }, 0);
        const summOfPastPausesScaled = pastPauses.reduce((accumulator, x) => {
            return accumulator + x.scaledRange.duration;
        }, 0);
        const deltaSumOfPastPauses = summOfPastPausesScaled - summOfPastPauses;
        if (activeParts.length > 0) {
            const activePart = activeParts[0];
            switch (activePart.type) {
                case "animation":
                    return relativeFrame - deltaSumOfPastPauses;
                case "pause":
                    return activePart.range.start;
                default:
                    return relativeFrame;
            }
        }
        // if (breaks && breaks.length > 0) {
        //   const clipBrakeSequence = this.clipPauseSequenceScaled(absoluteRange);
        //   if (!clipBrakeSequence) return relativeFrame;
        //   const index = clipBrakeSequence.findIndexContainsFrame(
        //     relativeFrame,
        //     true
        //   );
        //   const origin = clipBrakeSequence.collection.at(index);
        //   if (!origin || !scaled) return 0;
        //   let outFrameRelative = 0;
        //   const isAnimated = index % 2 === 1;
        //   if (isAnimated) {
        //     outFrameRelative = origin!..start;
        //   } else {
        //     outFrameRelative =
        //       origin!.start + scaled!.getRelativeFrame(relativeFrame);
        //   }
        //   return outFrameRelative;
        // }
        return relativeFrame;
    }
    clipPauseSequenceScaled(sequenceRange) {
        const breakRanges = this.getBreakRanges();
        const lottieDuration = this.lottieAnimation.frameDuration;
        const sumOfBreaks = this.lottieAnimation.sumOfBrakes;
        const strech = sequenceRange.duration - lottieDuration;
        const scale = sumOfBreaks === 0 ? 1 : (strech + sumOfBreaks) / sumOfBreaks;
        const clipPauseSequence = this.clipPauseSequence(FrameRange.fromStart(0, lottieDuration), breakRanges, scale);
        return clipPauseSequence;
    }
    clipPauseSequence(range, breaks, scale) {
        if (!breaks || breaks.length === 0)
            return undefined;
        const clipPauseSequence = new FrameRangeItemCollection();
        //const clipPauseSequence = new FrameRangeCollectionContinuous();
        const breaksWithPadding = breaks.map((x) => x.withPadding(-1));
        const animatedParts = FrameRange.differences(range, breaksWithPadding);
        animatedParts.forEach((part, index) => {
            const animation = {
                range: part,
                scaledRange: part,
                type: "animation",
            };
            clipPauseSequence.append(animation);
            if (index < animatedParts.length - 1) {
                const originPause = breaks.at(index);
                const scaledPause = originPause.scaleDuration(scale);
                const pause = {
                    range: originPause,
                    scaledRange: scaledPause,
                    type: "pause",
                };
                clipPauseSequence.append(pause);
            }
        });
        return clipPauseSequence;
    }
}
__decorate([
    Type(() => LottieAnimation),
    __metadata("design:type", LottieAnimation)
], LottieLayer.prototype, "_lottieAnimation", void 0);
export const LottieComponent = ({ sequenceRange, index, layer, path, selectors, }) => {
    const { fps, width, height, durationInFrames } = useVideoConfig();
    const [Lottie, setLottie] = useState();
    useEffect(() => {
        const loadLottie = () => __awaiter(void 0, void 0, void 0, function* () {
            if (detectEnvironment() === "browser") {
                const { Lottie } = yield import("@remotion/lottie");
                setLottie(() => Lottie);
            }
        });
        loadLottie();
    }, []);
    const detectEnvironment = () => {
        if (typeof window !== "undefined" &&
            typeof window.document !== "undefined") {
            return "browser";
            // eslint-disable-next-line max-statements-per-line
        }
        if (typeof process !== "undefined" &&
            process.versions &&
            process.versions.node) {
            return "node";
        }
        return "unknown";
    };
    const frame = useCurrentFrame();
    const lottieLayer = useMemo(() => {
        const self = layer;
        self.spacialBox.parent = new Box({
            size: [width, height],
        });
        return self;
    }, [layer, height, width]);
    const spacialBoxStyle = useMemo(() => new BoxStyle(lottieLayer.spacialBox).getStyleInPercent(), [lottieLayer.spacialBox]);
    const layerDisabled = useMemo(() => {
        if (lottieLayer.renderDisabled || lottieLayer.renderDisabledLocal)
            return true;
        return false;
    }, [lottieLayer]);
    const [handleFetchData] = useState(() => delayRender("Loading Lottie animation"));
    const [handleProperties] = useState(() => delayRender("hydrate Properties"));
    const [handleBreakRanges] = useState(() => delayRender("calculate breaks"));
    const [lottieAnimation, setLottieAnimation] = useState(null);
    const [breakRanges, setBreakRanges] = useState(null);
    const absoluteRange = sequenceRange.clone().round();
    useEffect(() => {
        if (!path ||
            (lottieAnimation && lottieAnimation.path === path) ||
            (lottieAnimation &&
                lottieAnimation.name === lottieLayer.lottieAnimation.name)) {
            continueRender(handleFetchData);
            return;
        }
        if (lottieLayer.lottieAnimation) {
            const animation = lottieLayer.lottieAnimation;
            const hydratedAnimation = lottieLayer.hydrateLottieAnimationByProperties(animation);
            console.log("hydrate animation from lottieLayer.lottieAnimation");
            setLottieAnimation(hydratedAnimation);
            continueRender(handleFetchData);
            return;
        }
        fetch(path)
            .then((data) => data.json())
            .then((json) => {
            if (json.error) {
                cancelRender(`Fetch returned error ${path}, ${json.error}`);
            }
            const animation = new LottieAnimation(json, path);
            const hydratedAnimation = lottieLayer.hydrateLottieAnimationByProperties(animation);
            setLottieAnimation(hydratedAnimation);
            continueRender(handleFetchData);
        })
            .catch((err) => {
            cancelRender(`Fetch Failed ${path}, ${err}`);
        });
    }, [handleFetchData, lottieAnimation, lottieLayer, path]);
    useEffect(() => {
        if (lottieAnimation) {
            const hydratedAnimation = lottieLayer.hydrateLottieAnimationByProperties(lottieAnimation);
            if (!_.isEqual(hydratedAnimation.lottieData, lottieAnimation.lottieData)) {
                setLottieAnimation(hydratedAnimation);
            }
        }
        continueRender(handleProperties);
    }, [handleProperties, lottieAnimation, lottieLayer]);
    useEffect(() => {
        var _a;
        setBreakRanges((_a = lottieAnimation === null || lottieAnimation === void 0 ? void 0 : lottieAnimation.getBreaksRanges()) !== null && _a !== void 0 ? _a : null);
        continueRender(handleBreakRanges);
    }, [handleBreakRanges, lottieAnimation]);
    const lottieAnimationValid = lottieAnimation && lottieAnimation.frameDuration > 0;
    // if (!lottieAnimationValid) return null;
    const [handleFreezeFrame] = useState(() => delayRender("calculate freezeFrame"));
    const [freezeFrame, setFreezeFrame] = useState(0);
    const [freezeFrameIsValid, setFreezeFrameIsValid] = useState(false);
    useEffect(() => {
        if (!lottieAnimation)
            return;
        const freeze = lottieLayer.nextFrame(frame, absoluteRange, lottieAnimation.frameDuration, breakRanges);
        setFreezeFrame(freeze);
        setFreezeFrameIsValid(freeze > 0 && freeze < lottieAnimation.frameDuration);
        continueRender(handleFreezeFrame);
    }, [
        handleFreezeFrame,
        lottieLayer,
        frame,
        absoluteRange,
        lottieAnimation,
        breakRanges,
    ]);
    const valid = absoluteRange.isValid() && freezeFrameIsValid;
    const message = `absoluteRange.isValid()=${absoluteRange.isValid()} freezeFrameIsValid=${freezeFrameIsValid}`;
    const info = (_jsx(devModeContext.Consumer, { children: (devMode) => {
            var _a, _b, _c;
            if (!devMode)
                return null;
            const breakRangesString = breakRanges === null || breakRanges === void 0 ? void 0 : breakRanges.reduce((accumulator, x) => {
                return accumulator + x.toString() + " ";
            }, "");
            return (_jsx(_Fragment, { children: _jsxs("div", { style: Object.assign(Object.assign({}, spacialBoxStyle), { zIndex: 100 + ((_a = lottieLayer.zIndex) !== null && _a !== void 0 ? _a : 1), top: (((_b = lottieLayer.zIndex) !== null && _b !== void 0 ? _b : 1) % 3) * 350, left: ((_c = lottieLayer.zIndex) !== null && _c !== void 0 ? _c : 1) * 20, fontSize: "20px" }), children: [_jsxs("p", { children: ["frame: ", frame] }), _jsxs("p", { children: ["freezeFrame: ", freezeFrame] }), _jsxs("p", { children: ["absoluteRange: ", absoluteRange.toString()] }), _jsxs("p", { children: ["lottieAnimation.frameDuration:", lottieAnimation === null || lottieAnimation === void 0 ? void 0 : lottieAnimation.frameDuration] }), _jsxs("p", { children: ["breakRanges: ", breakRangesString] }), _jsxs("p", { children: ["zIndex: ", lottieLayer.zIndex] })] }) }));
        } }));
    return (_jsx(RenderGuard, { disabled: layerDisabled || !valid, message: message, children: _jsx(disableContext.Consumer, { children: (disable) => {
                if (disable === null || disable === void 0 ? void 0 : disable.includes("LottieComponent"))
                    return null;
                return (_jsx(Sequence, { name: "lottie-sequence", from: absoluteRange.start, durationInFrames: absoluteRange.duration, children: _jsxs("div", { className: "lottie-layer", style: Object.assign(Object.assign({}, spacialBoxStyle), { zIndex: lottieLayer.zIndex }), children: [_jsx(Freeze, { frame: freezeFrame, children: Lottie ? (_jsx(Lottie, { animationData: lottieAnimation.lottieData, className: "remotion-lottie", direction: "forward", loop: false })) : (_jsx("div", { children: "Loading animation..." }) // Fallback while Lottie is loading
                                ) }), info] }, "Layer_" + layer.id) }, "lottieSequence_" + layer.id));
            } }) }));
};
export function fetchDataAsync(path) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield fetch(path);
        return (yield response.json());
    });
}
function groupPropertiesByName(variantMap) {
    const groupedProperties = {};
    variantMap.collection.forEach((variant) => {
        const { languageCode } = variant.key;
        const propertiesMap = variant.value;
        propertiesMap.forEach((property, propertyName) => {
            const propertyDetails = {
                languageCode,
                propertyName,
                value: property.value,
                path: property.path,
                type: property.type,
                functionDecorators: property.functionDecorators,
                options: property.options,
                width: property.width,
                height: property.height,
            };
            if (propertyDetails.type === "text") {
                if (!groupedProperties[property.name]) {
                    groupedProperties[property.name] = [];
                }
                groupedProperties[property.name].push(propertyDetails);
            }
        });
    });
    return groupedProperties;
}
//# sourceMappingURL=lottie.layer.js.map