import "reflect-metadata";
import { LayerType, TimingAction } from "../../enums/index.js";
import { PlaybackMode } from "../../lottieWrapper.js";
import { LottieLayer } from "./lottie.layer.js";
import { SlideInTextLayer } from "./slideInText.layer.js";
import { NormalizedSize } from "../normalizedSize.js";
import { Transition } from "../transition.js";
import { Frame } from "../frame.js";
import { FrameRange } from "../frameRange.js";
import { Position } from "../position.js";
export class LowerThirdLayer extends LottieLayer {
    get textWidth() {
        return Math.max(this.textLine1Width, this.textLine2Width);
    }
    // eslint-disable-next-line max-params
    constructor(layer, asset, path, lottieData, duration, textLine1, textLine2, position, size, settingsLine1, settingsLine2) {
        super(layer, asset, path, lottieData, duration, PlaybackMode.StayAtLastFrame, position, size, LayerType.LowerThird);
        this.path = path;
        this.duration = duration;
        this.textLine1 = textLine1;
        this.textLine2 = textLine2;
        this.settingsLine1 = settingsLine1;
        this.settingsLine2 = settingsLine2;
        this.__LAYER__ = "LowerThirdLayer";
        this.textLine1Width = 0;
        this.textLine2Width = 0;
        this.left = 180;
        this.right = 210;
        this.children = [];
        const slideTextPosition = Position.leftTop(this.left, 0)
            .relative()
            .xPixel();
        const t1 = new SlideInTextLayer(layer, asset, textLine1, duration, 25, slideTextPosition, new NormalizedSize(1, 0.3), settingsLine1, "LowerThirdClip__Line1");
        this.children.push(t1);
        this.textLine1Width = t1.textWidth;
        if (settingsLine2) {
            const t2 = new SlideInTextLayer(layer, asset, textLine2, duration, 25, slideTextPosition, new NormalizedSize(1, 0.2), settingsLine2, "LowerThirdClip__Line2");
            this.children.push(t2);
            this.textLine2Width = t2.textWidth;
        }
        this.translateX =
            0 -
                Math.max(this.lottieSizeInPixel - this.left - this.right - this.textWidth, 0);
    }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _a, _b, _c, _d;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (!path && !iLayer.subtitlesLocale)
            throw new Error("Subtitle needs a path or subtitle!");
        if (videoSettings.frameRate <= 0)
            throw new Error(`LowerThirdClip fps:${videoSettings.frameRate} must be bigger than 0`);
        const frameDuration = iLayer.duration
            ? Frame.framesFromSeconds(iLayer.duration, videoSettings.frameRate)
            : Frame.framesFromSeconds(duration, videoSettings.frameRate);
        const settingsLine1 = {
            fontSize: "5rem",
            textColor: "#000",
            fontWeight: "bolder",
            fontFamily: "DINPro-Bold",
            charSpacing: null,
        };
        const settingsLine2 = {
            fontSize: "4rem",
            textColor: "#000",
            fontWeight: "normal",
            fontFamily: "DINPro-Regular",
            charSpacing: null,
        };
        const position = Position.fromIPosition(iLayer.position);
        const size = { width: 1, height: 1 };
        const lowerThirdLayer = new LowerThirdLayer(iLayer, asset, path, {}, frameDuration, (_b = iLayer.text) !== null && _b !== void 0 ? _b : "", (_c = iLayer.text2) !== null && _c !== void 0 ? _c : "", position, null, settingsLine1, settingsLine2);
        lowerThirdLayer.durationInSeconds = iLayer.duration;
        if (iLayer.relativeRange)
            lowerThirdLayer.relativeRange = FrameRange.fromIFrameRange(iLayer.relativeRange);
        if (iLayer.id)
            lowerThirdLayer.id = iLayer.id;
        lowerThirdLayer.libId = (_d = iLayer.libId) !== null && _d !== void 0 ? _d : null;
        lowerThirdLayer.transitions = Transition.fromITransitions(iLayer.transitions);
        return lowerThirdLayer;
    }
    setRelativeTiming(parentInsideRange, startOffset = 0, endOffset = 0, timingAction = TimingAction.Trim) {
        const parentWithOffset = parentInsideRange
            .addPaddingLeft(startOffset)
            .addPaddingRight(endOffset);
        switch (timingAction) {
            case TimingAction.Trim:
                if (!this.relativeRange.isInside(parentWithOffset) &&
                    timingAction === TimingAction.Trim)
                    this.relativeRange = parentWithOffset;
                break;
            default:
                this.relativeRange = parentInsideRange.addRelative(this.relativeRange, timingAction);
                break;
        }
        for (const child of this.children) {
            if (!child.relativeRange.isInside(parentWithOffset) &&
                timingAction === TimingAction.Trim)
                child.relativeRange = parentWithOffset;
        }
    }
    getILayer(fps = 1) {
        var _a, _b;
        return Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.LowerThird, text: (_a = this.textLine1) !== null && _a !== void 0 ? _a : null, text2: (_b = this.textLine2) !== null && _b !== void 0 ? _b : null });
    }
    getTitle() {
        return `${this.textLine1}`;
    }
}
//# sourceMappingURL=lowerThird.layer.js.map