/* eslint-disable default-param-last */
import "reflect-metadata";
import { LayerType } from "../../enums/index.js";
import { Layer } from "./layer.js";
import { NormalizedSize } from "../normalizedSize.js";
import { VideoSettings } from "../videoSettings.js";
import { CssValue } from "../cssValue.js";
import { Frame } from "../frame.js";
import { Position } from "../position.js";
export class TitleLayer extends Layer {
    get minDuration() {
        return undefined;
    }
    get maxDuration() {
        return this.outerBoundaryRange.duration;
    }
    get textWidth() {
        if (this.textSettings)
            return CssValue.getTextWidthFromITextSettings(this.text, this.textSettings);
        return 0;
    }
    // eslint-disable-next-line max-params
    constructor(layer, asset, text, duration, position = null, size = null, textSettings = null, name = "") {
        super(LayerType.Title, layer, asset, position, size, name);
        this.text = text;
        this.duration = duration;
        this.textSettings = textSettings;
        this.__LAYER__ = "TitleLayer";
        // This.relativeRange = FrameRange.fromStart(0, duration);
    }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _a, _b, _c, _d, _e, _f, _g;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (!path && !iLayer.subtitlesLocale)
            throw new Error("Title needs a path or subtitle!");
        const viewPort = VideoSettings.sizeFromVideoSettings(videoSettings);
        if (videoSettings.frameRate <= 0)
            throw new Error(`TitleClip fps:${videoSettings.frameRate} must be bigger than 0`);
        const durationInSec = (_b = iLayer.duration) !== null && _b !== void 0 ? _b : duration;
        const frameDuration = Frame.framesFromSeconds(durationInSec, videoSettings.frameRate);
        let fontSize = "5rem";
        if (iLayer.fontSize)
            fontSize = `${iLayer.fontSize * 100}vh`;
        const textSettings = {
            fontSize,
            textColor: (_c = iLayer.textColor) !== null && _c !== void 0 ? _c : "#000",
            fontWeight: (_d = iLayer.fontWeight) !== null && _d !== void 0 ? _d : "bolder",
            fontFamily: "DINPro-Light",
            charSpacing: null,
        };
        const textWidth = CssValue.getTextWidthFromITextSettings((_e = iLayer.text) !== null && _e !== void 0 ? _e : "", textSettings) *
            1.01;
        const textHeight = CssValue.getTextHeightFromITextSettings((_f = iLayer.text) !== null && _f !== void 0 ? _f : "", textSettings) *
            1.01;
        const size = NormalizedSize.fromViewPort({ width: textWidth, height: textHeight }, viewPort);
        const position = Position.fromIPosition(iLayer.position, Position.center(0.5, 0.4));
        const titleLayer = new TitleLayer(iLayer, asset, (_g = iLayer.text) !== null && _g !== void 0 ? _g : "", frameDuration, position, size, textSettings, "TitleClip");
        titleLayer.durationInSeconds = durationInSec;
        return titleLayer;
    }
    getILayer() {
        var _a;
        return Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.Title, text: (_a = this.text) !== null && _a !== void 0 ? _a : null });
    }
    getTitle() {
        return `${this.text.slice(-20)}`;
    }
    toInfoString() {
        return `VIDEO LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.getTitle()}`;
    }
    getProperties() {
        const result = [];
        return result;
    }
    setProperty(name, value) { }
    updatePropertyValue(name, value) { }
}
//# sourceMappingURL=title.layer.js.map