import { jsxs as _jsxs, jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import { Sequence, Video, OffthreadVideo, staticFile, useCurrentFrame, useVideoConfig, } from "remotion";
import { useMemo } from "react";
import { LayerType } from "../../enums/index.js";
import { Layer } from "./layer.js";
import { Transition } from "../transition.js";
import { devModeContext, disableContext } from "./devModeContext.js";
import { BoxModel } from "../boxModel.js";
import { Frame } from "../frame.js";
import { FrameRange } from "../frameRange.js";
import RenderGuard from "../components/renderGuard.component.js";
import { clamp } from "../../extensions/number.extentions.js";
import { Box, BoxStyle } from "../boxModel/index.js";
export class VideoLayer extends Layer {
    get minDuration() {
        var _a, _b;
        return (_b = (_a = this.innerBoundaryRange) === null || _a === void 0 ? void 0 : _a.duration) !== null && _b !== void 0 ? _b : 25;
    }
    get maxDuration() {
        var _a;
        return (_a = this.outerBoundaryRange.duration) !== null && _a !== void 0 ? _a : this.duration;
    }
    get volume() {
        return this._volume;
    }
    set volume(value) {
        this._volume = clamp(value, 0, 1);
    }
    // eslint-disable-next-line max-params
    constructor(layer, asset, path, duration, position = null, size = null, name = "") {
        super(LayerType.Video, layer, asset, position, size, name);
        this.path = path;
        this.duration = duration;
        this.__LAYER__ = "VideoLayer";
        this._volume = 1;
    }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _a, _b, _c, _d;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (!path)
            throw new Error("VideoClip needs a path!");
        if (videoSettings.frameRate <= 0)
            throw new Error(`VideoClip fps:${videoSettings.frameRate} must be bigger than 0`);
        const durationInSeconds = (_b = iLayer.duration) !== null && _b !== void 0 ? _b : duration;
        const durationInFrames = Frame.framesFromSeconds(durationInSeconds, videoSettings.frameRate);
        const normalizedSize = Layer.aspectLayerSize(iLayer, videoSettings);
        const videoLayer = new VideoLayer(iLayer, asset, path, durationInFrames, null, normalizedSize);
        videoLayer.proxyPath =
            (_d = (_c = asset === null || asset === void 0 ? void 0 : asset.value.proxyFile) === null || _c === void 0 ? void 0 : _c.cloudDownloadPath) !== null && _d !== void 0 ? _d : iLayer.proxyPath;
        videoLayer.videoSize = iLayer.size;
        if (iLayer.mixVolume || iLayer.mixVolume === 0)
            videoLayer.volume = iLayer.mixVolume;
        return videoLayer;
    }
    getILayer() {
        var _a, _b, _c;
        const iLayer = this.toBaseILayer();
        return Object.assign(Object.assign({}, iLayer), { type: LayerType.Video, path: (_a = this.path) !== null && _a !== void 0 ? _a : null, proxyPath: (_b = this.proxyPath) !== null && _b !== void 0 ? _b : null, mixVolume: this.volume, size: (_c = this.videoSize) !== null && _c !== void 0 ? _c : null });
    }
    pathSelector(isPlayer) {
        return this.proxyPath && isPlayer ? this.proxyPath : this.path;
    }
    getTitle() {
        if (!this.path)
            return "video";
        const url = new URL(this.path);
        const filename = url.pathname.substring(url.pathname.lastIndexOf("/") + 1);
        return `${filename}`;
    }
    getProperties() {
        const result = [];
        return result;
    }
    setProperty(name, value) { }
    updatePropertyValue(name, value) { }
    toInfoString() {
        return `VIDEO LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.path}`;
    }
}
export const VideoComponent = ({ sequenceRange, index, layer, path, offthreadVideo, selectors, }) => {
    const missingFile = "https://firebasestorage.googleapis.com/v0/b/user-generated-content-app.appspot.com/o/Pirate.png?alt=media&token=fcff4f9f-2c89-4754-90c8-a48a4da57227";
    const frame = useCurrentFrame();
    const { fps, width, height, durationInFrames } = useVideoConfig();
    const videoLayer = useMemo(() => {
        const self = layer;
        self.spacialBox.parent = new Box({
            size: [width, height],
        });
        return self;
    }, [layer, width, height]);
    const spacialBoxStyle = useMemo(() => new BoxStyle(videoLayer.spacialBox).getStyleInPercent(), [videoLayer.spacialBox]);
    const layerDisabled = useMemo(() => videoLayer.renderDisabled || videoLayer.renderDisabledLocal, [videoLayer.renderDisabled, videoLayer.renderDisabledLocal]);
    const absoluteRange = useMemo(() => sequenceRange.round().pushInside(FrameRange.positive), [sequenceRange]);
    const cutRange = useMemo(() => { var _a; return (_a = layer.cut) === null || _a === void 0 ? void 0 : _a.round(); }, [layer.cut]);
    const source = useMemo(() => path
        ? (path === null || path === void 0 ? void 0 : path.startsWith("http"))
            ? path
            : staticFile(`/${path}`)
        : undefined, [path]);
    const volumeTransitions = useMemo(() => {
        return videoLayer.getTransitions("volume", absoluteRange);
    }, [videoLayer, absoluteRange]);
    const opacityTransitions = useMemo(() => {
        return videoLayer.getTransitions("opacity", absoluteRange);
    }, [videoLayer, absoluteRange]);
    const activeVolumeTransitionFn = (frame) => {
        return Transition.activeTransitionFromTransitions(volumeTransitions, frame);
    };
    const activeVolumeFn = (frame) => {
        var _a, _b;
        return (_b = (_a = activeVolumeTransitionFn(frame)) === null || _a === void 0 ? void 0 : _a.calculateNormPosition(frame)) !== null && _b !== void 0 ? _b : 1;
    };
    const transVolumeFn = (frame) => {
        return Transition.calculateNumberFromTransitions(volumeTransitions, frame);
    };
    const volumeFn = (frame) => {
        const vol = videoLayer.volume * transVolumeFn(frame);
        return vol;
    };
    const volumeFixedFn = (frame, range) => {
        return volumeFn(frame).toFixed(2);
    };
    const volumeXFn = (frame) => {
        var _a, _b;
        return (_b = (_a = activeVolumeTransitionFn(frame)) === null || _a === void 0 ? void 0 : _a.calculateNormPosition(frame)) !== null && _b !== void 0 ? _b : 0.5;
    };
    const opacityFn = (frame) => {
        return Transition.calculateNumberFromTransitions(opacityTransitions, frame);
    };
    const activeOpacityTransitionFn = (frame) => {
        return Transition.activeTransitionFromTransitions(opacityTransitions, frame);
    };
    const opacityXFn = (frame) => {
        var _a, _b;
        return ((_b = (_a = activeOpacityTransitionFn(frame)) === null || _a === void 0 ? void 0 : _a.calculateNormPosition(frame)) !== null && _b !== void 0 ? _b : 0.5);
    };
    const isValid = source && absoluteRange.isValid();
    const message = `layerDisabled=${layerDisabled} absoluteRange.isValid()=${absoluteRange.isValid()} source=${source}`;
    const size = 300;
    const info = (_jsx(devModeContext.Consumer, { children: (devMode) => {
            var _a, _b, _c, _d, _e;
            if (!devMode)
                return null;
            return (_jsx(_Fragment, { children: _jsxs("div", { style: Object.assign(Object.assign({ opacity: activeVolumeTransitionFn(frame) ||
                            activeOpacityTransitionFn(frame)
                            ? 1
                            : 0 }, spacialBoxStyle), { zIndex: 100 + ((_a = videoLayer.zIndex) !== null && _a !== void 0 ? _a : 1), top: (((_b = videoLayer.zIndex) !== null && _b !== void 0 ? _b : 1) % 3) * 350, left: ((_c = videoLayer.zIndex) !== null && _c !== void 0 ? _c : 1) * 20 }), children: [_jsxs("p", { style: {
                                fontSize: 30,
                                textShadow: "0px 0px 4px black",
                            }, children: [activeOpacityTransitionFn(frame) ? "X" : "0", " opacity:", opacityFn(frame).toFixed(2), (_d = activeOpacityTransitionFn(frame)) === null || _d === void 0 ? void 0 : _d.toString()] }), _jsxs("p", { style: {
                                fontSize: 30,
                                textShadow: "0px 0px 4px black",
                            }, children: [activeVolumeTransitionFn(frame) ? "X" : "0", " volume:", " ", volumeFixedFn(frame, absoluteRange), " ", (_e = activeVolumeTransitionFn(frame)) === null || _e === void 0 ? void 0 : _e.toString()] }), _jsx(PlotComponent, { x: 40, y: 0, height: size, width: size, labelX: "time", labelY: "opacity", valueX: opacityXFn(frame), valueY: opacityFn(frame), color: "red" }), _jsx(PlotComponent, { x: 70, y: 0, height: size, width: size, labelX: "time", labelY: "volume", valueX: volumeXFn(frame), valueY: volumeFn(frame), color: "#ff00c3" })] }) }));
        } }));
    return (_jsx(RenderGuard, { disabled: layerDisabled || !isValid, message: message, children: _jsx(disableContext.Consumer, { children: (disable) => {
                if (disable === null || disable === void 0 ? void 0 : disable.includes("VideoComponent"))
                    return null;
                return (_jsx(Sequence, { name: offthreadVideo ? "offthread-" : "video-sequence", from: absoluteRange.start, durationInFrames: absoluteRange.duration, premountFor: 100, children: (() => {
                        var _a, _b;
                        if (offthreadVideo) {
                            if (disable === null || disable === void 0 ? void 0 : disable.includes("<OffthreadVideo>"))
                                return null;
                            return (_jsx(OffthreadVideo, { name: "OffthreadVideo-" + index, src: source, startFrom: cutRange === null || cutRange === void 0 ? void 0 : cutRange.start, endAt: cutRange === null || cutRange === void 0 ? void 0 : cutRange.end, volume: (f) => volumeFn(f + absoluteRange.start), transparent: videoLayer.alphaChannel, style: Object.assign({ opacity: opacityFn(frame), zIndex: (_a = videoLayer.zIndex) !== null && _a !== void 0 ? _a : 1 }, spacialBoxStyle), onError: (e) => {
                                    console.error("error in OffthreadVideo", e);
                                } }, "OffthreadVideo-" + index));
                        }
                        if (disable === null || disable === void 0 ? void 0 : disable.includes("<Video>"))
                            return null;
                        return (_jsxs(_Fragment, { children: [_jsx(Video, { name: "Video-" + index, poster: missingFile, src: source, startFrom: cutRange === null || cutRange === void 0 ? void 0 : cutRange.start, endAt: cutRange === null || cutRange === void 0 ? void 0 : cutRange.end, volume: (f) => volumeFn(f + absoluteRange.start), style: Object.assign({ pointerEvents: "none", opacity: opacityFn(frame), zIndex: (_b = videoLayer.zIndex) !== null && _b !== void 0 ? _b : 1 }, spacialBoxStyle), onError: (e) => {
                                        console.error("error in Video", e);
                                    } }, "Video-" + index), info] }));
                    })() }, "videoSequence-" + index));
            } }) }));
};
export const PlotComponent = ({ x, y, width, height, labelX, labelY, valueX, valueY, color, }) => {
    const box = BoxModel.fromTopLeftWidthHeight(y, x, width, height);
    const scaledX = valueX * 400;
    const scaledY = 400 - valueY * 400;
    return (_jsx("div", { style: Object.assign({}, box.topLeftWidthHeight), children: _jsxs("svg", { width: "100%", height: "100%", viewBox: "0 0 400 400", xmlns: "http://www.w3.org/2000/svg", children: [_jsx("line", { x1: "0", y1: "200", x2: "400", y2: "200", stroke: "#325AAE", strokeWidth: "4" }), _jsx("line", { x1: "200", y1: "0", x2: "200", y2: "400", stroke: "#325AAE", strokeWidth: "4" }), _jsx("text", { x: "10", y: "50", fontWeight: "lighter", fontSize: "40", fill: "#a80505", children: labelY }), _jsx("text", { x: "250", y: "250", fontWeight: "lighter", fontSize: "40", fill: "#a80505", children: labelX }), _jsx("circle", { cx: scaledX, cy: scaledY, r: "8", fill: color !== null && color !== void 0 ? color : "red" })] }) }));
};
//# sourceMappingURL=video.layer.js.map