var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
import "reflect-metadata";
import ISO6391 from "iso-639-1";
import { plainToInstance, Type } from "class-transformer";
import { immerable } from "immer";
import { MediaType, getTasksInProgress, WorkerState, } from "../interfaces/index.js";
import { keepJsonStringAtFirebase, stringifyAtFirebase } from "../index.js";
import { CollectionDocument } from "./collectionDocument.js";
import { MediaAsset } from "./mediaAsset.js";
import { WebVTT } from "./webVTT.js";
export class LibraryItem extends CollectionDocument {
    get videoSettings() {
        var _b;
        return (_b = this.asset.videoSettings) !== null && _b !== void 0 ? _b : undefined;
    }
    set videoSettings(value) {
        this.asset.videoSettings = value;
    }
    get iSize() {
        var _b;
        return (_b = this.videoSettings) === null || _b === void 0 ? void 0 : _b.iSize;
    }
    get stateMessage() {
        const tasks = getTasksInProgress(this);
        const task = tasks.at(0);
        if (!task)
            return "";
        if (task.state === WorkerState.Pending) {
            return `Pending`;
        }
        if (task.state === WorkerState.Running) {
            return `${task.name}`;
        }
        return "";
    }
    get languageCode() {
        var _b, _c;
        return (_c = (_b = this.asset.languageCode) !== null && _b !== void 0 ? _b : this._languageCode) !== null && _c !== void 0 ? _c : null;
    }
    set languageCode(value) {
        if (value && value.length > 0 && ISO6391.validate(value)) {
            this._languageCode = value;
            this.asset.languageCode = value;
        }
    }
    get variantLanguageCodes() {
        var _b;
        if (!((_b = this.asset.sourceFile) === null || _b === void 0 ? void 0 : _b.variantMap.hasValues))
            return [];
        return this.asset.sourceFile.variantMap.keys.map((k) => k.languageCode);
    }
    get assetFiles() {
        var _b, _c, _d, _e, _f;
        const files = new Map();
        if (this.asset.sourceFile) {
            if (this.asset.sourceFile.cloudDownloadPath) {
                files.set(`Original ${(_b = this.asset.languageCode) === null || _b === void 0 ? void 0 : _b.toUpperCase()}`, this.asset.sourceFile);
            }
            if (this.asset.sourceFile.variantMap.hasValues) {
                this.asset.sourceFile.variantMap.collection.forEach((variant) => {
                    if (variant.value.cloudDownloadPath &&
                        !Array.from(files.entries()).some(([name, file]) => file.cloudDownloadPath === variant.value.cloudDownloadPath)) {
                        files.set(`Translation ${variant.key.languageCode.toUpperCase()}`, variant.value);
                    }
                });
            }
        }
        if ((_c = this.asset.proxyFile) === null || _c === void 0 ? void 0 : _c.cloudDownloadPath)
            files.set("Proxy", this.asset.proxyFile);
        if ((_d = this.asset.proxyFileLow) === null || _d === void 0 ? void 0 : _d.cloudDownloadPath)
            files.set("Proxy LOW", this.asset.proxyFileLow);
        if ((_e = this.asset.audioFile) === null || _e === void 0 ? void 0 : _e.cloudDownloadPath)
            files.set("Audio", this.asset.audioFile);
        if ((_f = this.asset.normalizedAudioFile) === null || _f === void 0 ? void 0 : _f.cloudDownloadPath)
            files.set("Normalized", this.asset.normalizedAudioFile);
        return Array.from(files.entries()).map(([name, file]) => ({ name, file }));
    }
    constructor(item) {
        super();
        this[_a] = true;
        this.name = "";
        this.state = "";
        this.type = MediaType.Video;
        this.asset = MediaAsset.default();
        this._languageCode = null;
    }
    initVideoSettings() { }
    updateSubtitle(languageCode, webVtt) {
        try {
            return this.asset.updateSubtitle(languageCode, webVtt);
        }
        catch (error) {
            console.error("ERROR updateSubtitle", error);
            throw error;
        }
    }
    toILibraryItem() {
        return Object.assign(Object.assign({}, super.toICollectionDocument()), { name: this.name, state: this.state, type: this.type, dataJson: this.data ? JSON.stringify(this.data) : undefined, data: undefined, asset: this.asset.toIMediaAsset() });
    }
    static toILibraryItem(obj) {
        return plainToInstance(LibraryItem, obj).toILibraryItem();
    }
    static fromILibraryItem(item) {
        try {
            return plainToInstance(LibraryItem, item);
        }
        catch (error) {
            console.error(`ERROR fromILibraryItem ${error.message}`, item);
            throw error;
        }
    }
    static default() {
        return LibraryItem.fromILibraryItem({
            id: "id",
            name: "name",
            type: MediaType.Video,
            state: "ok",
            asset: MediaAsset.default(),
        });
    }
    static getText(item) {
        const metadataText = LibraryItem.getMetadataText(item);
        const analysisText = LibraryItem.getAnalysisText(item);
        const subtitleText = LibraryItem.getSubtitleText(item);
        let text = "";
        if (metadataText)
            text += metadataText + "\n";
        if (analysisText)
            text += analysisText + "\n";
        if (subtitleText)
            text += subtitleText + "\n";
        return text;
    }
    static getMetadataText(item) {
        if (!item.metadata)
            return undefined;
        let text = "";
        for (const [key, value] of Object.entries(item.metadata)) {
            if (value && typeof value === "string" && value.length > 0) {
                text += `${key}: ${value}\n`;
            }
        }
        return text;
    }
    static getAnalysisText(item) {
        if (!item.asset.analysis)
            return undefined;
        let text = "";
        for (const analysis of item.asset.analysis) {
            if (item) {
                text += `${analysis.information}\n`;
            }
        }
        return text;
    }
    static getAnalysisTexts(item) {
        if (!item.asset.analysis)
            return undefined;
        let text = "";
        for (const analysis of item.asset.analysis) {
            if (item) {
                text += `${analysis.information}\n`;
            }
        }
        return text;
    }
    static getSubtitleText(item) {
        if (!item.asset.sourceFile.jsonData)
            return undefined;
        const webVtt = WebVTT.fromJson(item.asset.sourceFile.jsonData);
        return webVtt.toString();
    }
    static getSubtitleTimerangeTexts(item, minTextLength) {
        if (!item.asset.sourceFile.jsonData)
            return undefined;
        const webVtt = WebVTT.fromJson(item.asset.sourceFile.jsonData);
        return webVtt.toTimerangeTexts(minTextLength);
    }
}
_a = immerable;
__decorate([
    Type(() => MediaAsset),
    __metadata("design:type", MediaAsset)
], LibraryItem.prototype, "asset", void 0);
__decorate([
    stringifyAtFirebase,
    keepJsonStringAtFirebase,
    __metadata("design:type", Object)
], LibraryItem.prototype, "data", void 0);
//# sourceMappingURL=library-item.js.map