var _a;
import { immerable } from "immer";
import "reflect-metadata";
import { parse, stringify } from "flatted";
import { FrameRange } from "./frameRange.js";
import { FunctionDescription } from "./function-description.js";
export class LottieAnimation {
    get lottieDataWithoutBase64() {
        return LottieAnimation.removeBase64Data(this.lottieData);
    }
    get name() {
        return this.lottieData.nm;
    }
    get frameDuration() {
        return this.lottieData.op - this.lottieData.ip;
    }
    get seconds() {
        return this.frameDuration / this.frameRate;
    }
    get frameDurationWithoutPause() {
        return this.frameDuration - this.sumOfBrakes;
    }
    get frameRate() {
        return this.lottieData.fr;
    }
    get sumOfBrakes() {
        var _b, _c;
        const sumOfBreaks = (_c = (_b = this.getBreaksRanges()) === null || _b === void 0 ? void 0 : _b.reduce((accumulator, x) => {
            return accumulator + x.duration;
        }, 0)) !== null && _c !== void 0 ? _c : 0;
        return sumOfBreaks;
    }
    constructor(data, path = undefined) {
        this[_a] = true; // Option 1
        this.path = "";
        this.lottieData = {
            fr: 25,
            w: 10,
            h: 10,
            ip: 0,
            op: 10,
        };
        this.path = path;
        if (data)
            this.lottieData = LottieAnimation.removeBase64Data(data);
    }
    getBreaksRanges(startOffset) {
        const markers = this.getBreakMarkers();
        return markers === null || markers === void 0 ? void 0 : markers.map((marker) => {
            return FrameRange.fromStart(startOffset !== null && startOffset !== void 0 ? startOffset : 0 + marker.tm, marker.dr);
        });
    }
    shrink() { }
    // #region TextProperties
    getTextProperties() {
        if (this.lottieData.shrinked)
            return [];
        const properties = this.getTextPropertiesFromLayers(this.lottieData.layers, ["lottieData"]);
        const assetProperties = this.getTextPropertiesFromAssets(this.lottieData.assets);
        return [...properties, ...assetProperties];
    }
    getTextPropertiesFromLayers(layers, path = []) {
        const properties = [];
        if (Array.isArray(layers)) {
            layers === null || layers === void 0 ? void 0 : layers.forEach((layer, idx) => {
                var _b, _c, _d;
                if (layer.ty === 5 && layer.t && layerStartWith(layer, "property")) {
                    const actPath = [...path, `layers`, `[${idx}]`, "t", "d", "k", "[0]"];
                    const value = (_c = (_b = layer.t) === null || _b === void 0 ? void 0 : _b.d) === null || _c === void 0 ? void 0 : _c.k[0];
                    if (value && ((_d = value.s) === null || _d === void 0 ? void 0 : _d.t)) {
                        properties.push({
                            name: layer.nm,
                            value: value.s.t,
                            path: actPath.join("."),
                            type: "text",
                            functionDecorators: parseFunctionDescription(layer.nm),
                        });
                    }
                }
            });
        }
        return properties;
    }
    getTextPropertiesFromAssets(assets, path = []) {
        let properties = [];
        const actPath = [...path, "assets"];
        if (Array.isArray(assets)) {
            assets === null || assets === void 0 ? void 0 : assets.forEach((asset, idx) => {
                properties = [
                    ...properties,
                    ...this.getTextPropertiesFromLayers(asset.layers, [
                        ...actPath,
                        `${idx}`,
                    ]),
                ];
            });
        }
        return properties;
    }
    setTextProperty(name, newValue) {
        if (Array.isArray(this.lottieData.layers)) {
            this.setTextProperyInLayers(name, newValue, this.lottieData.layers);
        }
        if (Array.isArray(this.lottieData.assets)) {
            this.setTextProperyInAssets(name, newValue, this.lottieData.assets);
        }
    }
    setTextProperyInAssets(name, newValue, assets) {
        assets === null || assets === void 0 ? void 0 : assets.forEach((asset) => {
            this.setTextProperyInLayers(name, newValue, asset.layers);
        });
    }
    setTextProperyInLayers(name, newValue, layers) {
        layers === null || layers === void 0 ? void 0 : layers.forEach((layer) => {
            var _b, _c, _d;
            if (layer && layer.ty === 5 && layer.nm === name) {
                const value = (_c = (_b = layer.t) === null || _b === void 0 ? void 0 : _b.d) === null || _c === void 0 ? void 0 : _c.k[0];
                if (value && ((_d = value.s) === null || _d === void 0 ? void 0 : _d.t)) {
                    value.s.t = newValue;
                }
            }
        });
    }
    // #endregion
    // #region ColorProperties
    getColorProperties() {
        let assetItemsColorInfos = [];
        if (this.lottieData.assets && this.lottieData.assets.length > 0) {
            let index = 0;
            const colorInfos = this.lottieData.assets.map((asset) => LottieAnimation.findColorsInObject(asset, `${index++}`, () => true, [
                "assets",
            ]));
            assetItemsColorInfos = assetItemsColorInfos.concat(...colorInfos);
        }
        const layersColorInfos = LottieAnimation.findColorsInArray(this.lottieData.layers, "layers");
        const properties = [...layersColorInfos, ...assetItemsColorInfos].map((colorInfo) => {
            const fnDecorators = parseFunctionDescription(colorInfo.layer.nm);
            const colors = fnDecorators.filter((x) => x.name.toLowerCase().includes("color"));
            const options = colors.map((x) => x.parameters[0]);
            return {
                name: `Color of ${FunctionDescription.getName(fnDecorators)}`,
                value: colorInfo.value,
                path: colorInfo.path.join("."),
                type: "color",
                functionDecorators: fnDecorators,
                options,
            };
        });
        return properties;
    }
    static findColorsInArray(collection, collectionName = "layer", whereFn = () => true, path = []) {
        let results = [];
        if (!collection || collection.length <= 0)
            return results;
        if (path.length > 128)
            return results;
        for (let i = 0; i < collection.length; i++) {
            const item = collection[i];
            const layerPath = [...path, `${collectionName}`];
            if (Array.isArray(item)) {
                this.findColorsInArray(item, `[${i}]`, whereFn, layerPath);
                continue;
            }
            const objectType = getLottieObjectType(item);
            if (!objectType || typeof item !== "object" || !item.ty)
                continue;
            const objResults = this.findColorsInObject(item, `[${i}]`, whereFn, layerPath);
            results = results.concat(objResults);
        }
        return results;
    }
    static findColorsInObject(item, objName, whereFn = () => true, path = []) {
        let results = [];
        const layerPath = [...path, `${objName}`];
        if (path.length > 128 || typeof item !== "object")
            return results;
        const objectType = getLottieObjectType(item);
        if (objectType && layerContainsName(item, "color") && whereFn(item)) {
            const index = getColorIndex(item);
            const textColor = getTextFontColor(item, index);
            if (textColor) {
                results.push({
                    path: layerPath.concat(textColor.path),
                    value: textColor.value,
                    layer: item,
                });
            }
            const strokeColor = getStrokeColorValue(item, index);
            if (strokeColor) {
                results.push({
                    path: layerPath.concat(strokeColor.path),
                    value: strokeColor.value,
                    layer: item,
                });
            }
            const fillColor = getFillColorValue(item, index);
            if (fillColor) {
                results.push({
                    path: layerPath.concat(fillColor.path),
                    value: fillColor.value,
                    layer: item,
                });
            }
        }
        // eslint-disable-next-line guard-for-in
        for (const key in item) {
            const subObj = item[key];
            if (!subObj)
                continue;
            if (Array.isArray(subObj)) {
                results = results.concat(this.findColorsInArray(subObj, key, whereFn, layerPath));
                continue;
            }
            if (typeof subObj === "object" && subObj.ty) {
                results = results.concat(this.findColorsInObject(item[key], key, whereFn, layerPath.concat([key])));
            }
        }
        return results;
    }
    setColorProperty(property) {
        if (property.type !== "color")
            return;
        let colorValue = null;
        if (Array.isArray(property.value)) {
            colorValue = property.value;
        }
        if (typeof property.value === "string") {
            colorValue = hexToNormalizedArray(property.value);
        }
        if (!colorValue)
            return;
        const nestedValue = getNestedProperty(this.lottieData, property.path);
        if (!nestedValue)
            return;
        setNestedProperty(this.lottieData, property.path, colorValue);
    }
    // #endregion
    // #region ImageProperties
    getImageProperties() {
        let assetItemsImageInfos = [];
        if (this.lottieData.assets && this.lottieData.assets.length > 0) {
            let index = 0;
            const imageInfos = this.lottieData.assets.map((asset) => LottieAnimation.findImagesInObject(asset, `${index++}`, () => true, [
                "assets",
            ]));
            assetItemsImageInfos = assetItemsImageInfos.concat(...imageInfos);
        }
        // Const layersImageInfos = LottieAnimation.findImagesInArray(
        //   this.lottieData.layers,
        //   'layers'
        // );
        const properties = [...assetItemsImageInfos].map((imageInfo) => {
            return {
                name: `image_${imageInfo.imageIndex}`,
                value: imageInfo.value,
                path: imageInfo.path.join("."),
                type: "image",
                width: imageInfo.width,
                height: imageInfo.height,
            };
        });
        return properties;
    }
    static findImagesInObject(item, objName, whereFn = () => true, path = []) {
        const results = [];
        const layerPath = [...path, `${objName}`];
        if (path.length > 128 || typeof item !== "object")
            return results;
        if (typeof item.id === "string" && whereFn(item)) {
            const isImageAsset = item.id.startsWith("image_");
            if (isImageAsset) {
                const imageIndex = parseInt(item.id.split("_")[1], 10);
                const path = item.p;
                if (isImageAsset) {
                    results.push({
                        path: layerPath.concat(["p"]),
                        imageIndex,
                        value: path,
                        width: item.w,
                        height: item.h,
                        layer: item,
                    });
                }
            }
        }
        // Recursive search
        // for (const key in item) {
        //   const subObj = item[key];
        //   if (!subObj) continue;
        //   if (Array.isArray(subObj)) {
        //     results = results.concat(
        //       this.findImagesInArray(subObj, key, whereFn, layerPath)
        //     );
        //   }
        // }
        return results;
    }
    static findImagesInArray(collection, collectionName = "assets", whereFn = () => true, path = []) {
        let results = [];
        if (!collection || collection.length <= 0)
            return results;
        if (path.length > 128)
            return results;
        for (let i = 0; i < collection.length; i++) {
            const item = collection[i];
            const layerPath = [...path, `${collectionName}`];
            if (Array.isArray(item)) {
                this.findColorsInArray(item, `[${i}]`, whereFn, layerPath);
                continue;
            }
            if (typeof item.id === "string") {
                const id = item.id;
                const isImageAsset = id.startsWith("image_");
                if (!isImageAsset) {
                    continue;
                }
            }
            else {
                continue;
            }
            const objResults = this.findImagesInObject(item, `[${i}]`, whereFn, layerPath);
            results = results.concat(objResults);
        }
        return results;
    }
    setImageProperty(name, newValue) {
        if (Array.isArray(this.lottieData.assets)) {
            this.setImageProperyInAssets(name, newValue, this.lottieData.assets);
        }
    }
    setImageProperyInAssets(name, newValue, assets) {
        assets === null || assets === void 0 ? void 0 : assets.forEach((asset) => {
            if (asset.id &&
                typeof asset.id === "string" &&
                asset.id.startsWith("image_") &&
                asset.p &&
                typeof asset.p === "string") {
                if (asset.id === name) {
                    asset.p = newValue;
                    asset.u = "";
                }
            }
        });
    }
    // #endregion
    getPauseMarker() {
        var _b, _c;
        return (_c = (_b = this.lottieData) === null || _b === void 0 ? void 0 : _b.markers) === null || _c === void 0 ? void 0 : _c.find((x) => x.cm.toLowerCase().startsWith("pause"));
    }
    getBreakMarkers() {
        var _b, _c;
        return (_c = (_b = this.lottieData) === null || _b === void 0 ? void 0 : _b.markers) === null || _c === void 0 ? void 0 : _c.filter((x) => x.cm.toLowerCase().startsWith("pause"));
    }
    static encodeToBase64(inputString) {
        const encoder = new TextEncoder();
        const data = encoder.encode(inputString);
        return btoa(String.fromCharCode(...data));
    }
    static decodeFromBase64(base64String) {
        const binaryString = atob(base64String);
        const decoder = new TextDecoder();
        return decoder.decode(new Uint8Array(binaryString.length).map((_, i) => binaryString.charCodeAt(i)));
    }
    toBase64() {
        return LottieAnimation.encodeToBase64(stringify(this.lottieData));
    }
    static fromBase64(base64) {
        let data;
        try {
            data = parse(LottieAnimation.decodeFromBase64(base64));
        }
        catch (error) {
            console.error(error);
        }
        return new LottieAnimation(data);
    }
    static removeBase64Data(lottieData) {
        let stripped = LottieAnimation.removeBase64ImageData(lottieData);
        stripped = LottieAnimation.removeBase64FontData(stripped);
        return stripped;
    }
    static removeBase64FontData(lottieData) {
        var _b;
        const data = Object.assign({}, lottieData);
        if (((_b = data.fonts) === null || _b === void 0 ? void 0 : _b.list) && Array.isArray(data.fonts.list)) {
            data.fonts.list = data.fonts.list.map((font) => {
                if (font.fPath &&
                    typeof font.fPath === "string" &&
                    font.fPath.startsWith("data:") &&
                    font.fPath.length > 1024) {
                    const header = LottieAnimation.dataHeader(font.fPath);
                    font.fPath = `${header}FONT_BASE64DATA_HERE`;
                }
                return font;
            });
        }
        return data;
    }
    static removeBase64ImageData(lottieData) {
        const data = Object.assign({}, lottieData);
        if (data.assets && Array.isArray(data.assets)) {
            data.assets = data.assets.map((asset) => {
                if (asset.id.startsWith("image_") &&
                    asset.p &&
                    typeof asset.p === "string") {
                    const p = asset.p;
                    if (p.length > 1024 &&
                        p.startsWith("data:") &&
                        p.substring(0, 128).includes("base64,")) {
                        const header = LottieAnimation.dataHeader(p);
                        asset.p = `${header}IMAGE_BASE64DATA_HERE`;
                    }
                }
                return asset;
            });
        }
        return data;
    }
    static dataHeader(p) {
        const index = p.indexOf("base64,");
        const header = p.substring(0, index + 7);
        return header;
    }
    static LottieDataFrom(data) {
        let { lottieData } = new LottieAnimation();
        try {
            if (typeof data === "string") {
                if (isBase64(data)) {
                    lottieData = LottieAnimation.fromBase64(data).lottieData;
                }
                else {
                    lottieData = parse(data);
                }
            }
            if (typeof data === "object" && data !== null) {
                lottieData = data;
            }
        }
        catch (error) {
            console.error(error);
        }
        return lottieData;
    }
}
_a = immerable;
function isBase64(input) {
    // Regular expression to match a valid Base64 string
    const base64RegExp = /^[A-Za-z0-9+/]*={0,2}$/;
    return base64RegExp.test(input);
}
export function parseFunctionDescription(name) {
    const funcRegex = /(([a-zA-Z0-9]*)\([a-zA-Z_0-9 ,.#[\]]*\))/g;
    const functions = name.match(funcRegex);
    if (!functions)
        return [];
    const functionDescriptions = [];
    for (const func of functions) {
        functionDescriptions.push(new FunctionDescription("", func));
    }
    return functionDescriptions;
}
export function layerStartWith(layer, name) {
    var _b;
    return (_b = layer === null || layer === void 0 ? void 0 : layer.nm) === null || _b === void 0 ? void 0 : _b.toString().toLowerCase().startsWith(name.toLowerCase());
}
export function layerContainsName(layer, name) {
    var _b;
    return (_b = layer === null || layer === void 0 ? void 0 : layer.nm) === null || _b === void 0 ? void 0 : _b.toString().toLowerCase().includes(name.toLowerCase());
}
export var LottieObjectType;
(function (LottieObjectType) {
    LottieObjectType["Shape"] = "4|sh";
    LottieObjectType["Text"] = "5|t";
    LottieObjectType["Solid"] = "1|s";
    LottieObjectType["Image"] = "2|i";
    LottieObjectType["Null"] = "3|n";
    LottieObjectType["Camera"] = "6|c";
    LottieObjectType["PreComp"] = "0|p";
    LottieObjectType["Adjustment"] = "7|a";
    LottieObjectType["Audio"] = "8|au";
    LottieObjectType["Light"] = "9|l";
    // Shape Item Types
    LottieObjectType["Rectangle"] = "rc";
    LottieObjectType["Ellipse"] = "el";
    LottieObjectType["Path"] = "sh";
    LottieObjectType["Fill"] = "fl";
    LottieObjectType["Stroke"] = "st";
    LottieObjectType["Group"] = "gr";
    LottieObjectType["TrimPaths"] = "tm";
    // eslint-disable-next-line @typescript-eslint/no-duplicate-enum-values
    LottieObjectType["RoundCorners"] = "rc";
    LottieObjectType["Repeater"] = "rp";
    LottieObjectType["GradientFill"] = "gf";
    LottieObjectType["GradientStroke"] = "gs";
    LottieObjectType["Star"] = "sr";
    LottieObjectType["Polystar"] = "ps";
    LottieObjectType["MergePaths"] = "mm";
})(LottieObjectType || (LottieObjectType = {}));
function getLottieObjectType(layer) {
    var _b;
    if (!layer)
        return null;
    const type = (_b = layer === null || layer === void 0 ? void 0 : layer.ty) === null || _b === void 0 ? void 0 : _b.toString();
    if (type) {
        for (const value of Object.values(LottieObjectType)) {
            if (value.split("|").includes(type)) {
                return value;
            }
        }
    }
    return null;
}
var LottieColorType;
(function (LottieColorType) {
    LottieColorType["Text"] = "5|t";
    LottieColorType["Fill"] = "fl";
    LottieColorType["Stroke"] = "st";
    LottieColorType["GradientFill"] = "gf";
    LottieColorType["GradientStroke"] = "gs";
    LottieColorType["Solid"] = "1|s";
})(LottieColorType || (LottieColorType = {}));
function isColorContainer(layerType) {
    return getLottieColorObjectType({ ty: layerType }) !== null;
}
function isLottieObjectType(layer, type) {
    return getLottieObjectType(layer) === type;
}
function getLottieColorObjectType(layer) {
    var _b;
    const type = (_b = layer === null || layer === void 0 ? void 0 : layer.ty) === null || _b === void 0 ? void 0 : _b.toString();
    if (type) {
        for (const value of Object.values(LottieColorType)) {
            if (value.toString().split("|").includes(type)) {
                return value;
            }
        }
    }
    return null;
}
function getColorIndex(layer) {
    let index;
    const layerName = layer.nm;
    const fnDecorators = parseFunctionDescription(layerName);
    const colorOptions = FunctionDescription.getColorOptions(fnDecorators);
    const indices = colorOptions.map((x) => x.index);
    if (indices.length > 0 && indices.at(0) !== 0)
        index = indices.at(0);
    return index;
}
function getTextFontColor(layer, index) {
    var _b, _c, _d, _e, _f, _g, _h;
    if (isLottieObjectType(layer, LottieObjectType.Text)) {
        const k = (_c = (_b = layer.t) === null || _b === void 0 ? void 0 : _b.d) === null || _c === void 0 ? void 0 : _c.k;
        if (!k)
            return null;
        const idx = index && index < k.length ? index : 0;
        const value = (_h = (_g = (_f = (_e = (_d = layer.t) === null || _d === void 0 ? void 0 : _d.d) === null || _e === void 0 ? void 0 : _e.k) === null || _f === void 0 ? void 0 : _f[idx]) === null || _g === void 0 ? void 0 : _g.s) === null || _h === void 0 ? void 0 : _h.fc;
        if (!value)
            return null;
        return {
            value,
            type: LottieObjectType.Text,
            path: ["t", "d", "k", `[${idx}]`, "s", "fc"],
        };
    }
    return null;
}
function getStrokeColorValue(layer, index) {
    var _b, _c, _d;
    if (isLottieObjectType(layer, LottieObjectType.Stroke)) {
        const k = (_b = layer === null || layer === void 0 ? void 0 : layer.c) === null || _b === void 0 ? void 0 : _b.k;
        const type = getLottieObjectType(layer);
        if (k && Array.isArray(k) && k.length > 0) {
            const idx = index && index < k.length ? index : 0;
            if (typeof k[idx] === "object") {
                return {
                    type,
                    value: ((_c = k[idx]) === null || _c === void 0 ? void 0 : _c.s) || null,
                    path: ["c", "k", `[${idx}]`, "s"],
                };
            }
            return {
                type,
                value: k || null,
                path: ["c", "k"],
            };
        }
        return { type, value: ((_d = layer === null || layer === void 0 ? void 0 : layer.c) === null || _d === void 0 ? void 0 : _d.k) || null, path: ["c", "k"] };
    }
    return null;
}
function getFillColorValue(layer, index) {
    var _b, _c;
    if (isLottieObjectType(layer, LottieObjectType.Fill)) {
        const k = (_b = layer === null || layer === void 0 ? void 0 : layer.c) === null || _b === void 0 ? void 0 : _b.k;
        const type = getLottieObjectType(layer);
        if (k && Array.isArray(k) && k.length > 0) {
            const idx = index && index >= 0 && index < k.length ? index : 0;
            return {
                type,
                value: k[idx].s || null,
                path: ["c", "k", `[${idx}]`, "s"],
            };
        }
        return { type, value: ((_c = layer === null || layer === void 0 ? void 0 : layer.c) === null || _c === void 0 ? void 0 : _c.k) || null, path: ["c", "k"] };
    }
    return null;
}
export function normalizedArrayToHex(arr) {
    // Convert and normalize RGB components
    let r = Math.round(arr[0] * 255)
        .toString(16)
        .padStart(2, "0");
    let g = Math.round(arr[1] * 255)
        .toString(16)
        .padStart(2, "0");
    let b = Math.round(arr[2] * 255)
        .toString(16)
        .padStart(2, "0");
    // Initialize alpha as fully opaque
    let a = "";
    // Convert and normalize alpha channel if it exists
    if (arr.length === 4) {
        a = Math.round(arr[3] * 255)
            .toString(16)
            .padStart(2, "0");
        if (r[0] === r[1] && g[0] === g[1] && b[0] === b[1] && a[0] === a[1]) {
            r = r[0];
            g = g[0];
            b = b[0];
            a = a[0];
        }
        return `#${r}${g}${b}${a}`.toUpperCase();
    }
    if (r[0] === r[1] && g[0] === g[1] && b[0] === b[1]) {
        r = r[0];
        g = g[0];
        b = b[0];
    }
    return `#${r}${g}${b}`.toUpperCase();
}
export function hexToNormalizedArray(hex) {
    // Check if the hex is a valid color #fff or #ffffff or #ffffff00
    const hexRegex = /^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3}|[A-Fa-f0-9]{8})$/;
    if (!hexRegex.test(hex)) {
        return [1, 1, 1, 1];
    }
    // Remove the hash (#) if it exists
    hex = hex.replace("#", "");
    // Handle shorthand formats
    if (hex.length === 3 || hex.length === 4) {
        hex = Array.from(hex)
            .map((x) => x + x)
            .join("");
    }
    // Extract and normalize RGB components
    const r = parseInt(hex.substring(0, 2), 16) / 255;
    const g = parseInt(hex.substring(2, 4), 16) / 255;
    const b = parseInt(hex.substring(4, 6), 16) / 255;
    // Initialize alpha as 1 (fully opaque)
    let a = 1;
    // Extract and normalize alpha channel if it exists
    if (hex.length === 8) {
        a = parseInt(hex.substring(6, 8), 16) / 255;
    }
    if (a === 1)
        return [r, g, b];
    return [r, g, b, a];
}
export function formatHex(hex) {
    return normalizedArrayToHex(hexToNormalizedArray(hex));
}
class LottieTextFinder {
    constructor(lottieData) {
        this.lottieData = lottieData;
    }
    findTextLayers(layers) {
        let textLayers = [];
        for (const layer of layers) {
            if (layer.ty === 5) {
                textLayers.push(layer);
            }
            if (layer.ty === 0 && layer.layers) {
                textLayers = textLayers.concat(this.findTextLayers(layer.layers));
            }
        }
        return textLayers;
    }
    findTextLayersInAssets(assets) {
        let textLayers = [];
        for (const asset of assets) {
            if (asset.layers) {
                textLayers = textLayers.concat(this.findTextLayers(asset.layers));
            }
        }
        return textLayers;
    }
    findAllTextLayers() {
        let allTextLayers = this.findTextLayers(this.lottieData.layers);
        allTextLayers = allTextLayers.concat(this.findTextLayersInAssets(this.lottieData.assets));
        return allTextLayers.map((layer) => layer.t);
    }
    filterNonTextLayersAndAssets() {
        if (this.lottieData.layers)
            this.lottieData.layers = this.lottieData.layers.filter((layer) => {
                if (layer.ty === 5)
                    return true;
                if (layer.ty === 0 &&
                    layer.layers &&
                    this.findTextLayers(layer.layers).length > 0)
                    return true;
                return false;
            });
        if (this.lottieData.assets)
            this.lottieData.assets = this.lottieData.assets.filter((asset) => {
                if (asset.layers && this.findTextLayers(asset.layers).length > 0)
                    return true;
                return false;
            });
    }
}
export function getNestedProperty(obj, path) {
    if (!path || typeof path !== "string")
        return undefined;
    const keys = path.split(".");
    let current = obj;
    for (const key of keys) {
        if (!current) {
            return undefined;
        }
        if (key.startsWith("[") && key.endsWith("]")) {
            const index = parseInt(key.slice(1, -1), 10);
            current = current[index];
        }
        else {
            current = current[key];
        }
    }
    return current;
}
export function setNestedProperty(obj, path, value) {
    if (!obj)
        return;
    if (!path || typeof path !== "string")
        return;
    const keys = path.split(".");
    const lastKey = keys.pop();
    if (!lastKey)
        return;
    const parent = getNestedProperty(obj, keys.join("."));
    if (!parent)
        return;
    if (lastKey.startsWith("[") && lastKey.endsWith("]")) {
        const index = parseInt(lastKey.slice(1, -1), 10);
        parent[index] = value;
    }
    else {
        parent[lastKey] = value;
    }
}
//# sourceMappingURL=lottieAnimation.js.map