var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import { immerable } from "immer";
import { plainToClass, Type } from "class-transformer";
import { compareJsonStrings } from "../extensions/index.js";
import { MediaType, } from "../interfaces/index.js";
import { LcfVariant, LocalCloudFile } from "./localCloudFile.js";
import { Transcription } from "./transcription.js";
import { HasUsers } from "./hasUsers.js";
import { VariantMap } from "./variantMap.js";
import { VideoSettings } from "./videoSettings.js";
import { WebVTT } from "./webVTT.js";
import { TimerangeInformation } from "./timerangeInformation.js";
export class MediaAsset extends HasUsers {
    constructor() {
        super(...arguments);
        this[_a] = true; // Option 1
        this.sourceFile = LocalCloudFile.default();
        this.derivedFiles = [];
        this.isUploaded = false;
        this.mediaType = MediaType.null;
        this.value = "";
        this.size = 0;
        this.length = 0;
        this.durationInSeconds = 0;
    }
    get derivedFilesMap() {
        return VariantMap.fromIVariantArray(this.derivedFiles, (variants) => {
            this.derivedFiles = variants.map((x) => {
                return {
                    key: x.key,
                    value: LocalCloudFile.fromILocalCloudFile(x.value),
                };
            });
        }, (dto) => dto, (item) => item);
    }
    updateSubtitle(languageCode, updatedWebVtt) {
        let changed = false;
        const posterior = new WebVTT(updatedWebVtt);
        posterior.language = languageCode;
        const posteriorJson = JSON.stringify(posterior.toIWebVTT());
        const mainPrior = this.sourceFile.jsonData
            ? WebVTT.fromIWebVTT(JSON.parse(this.sourceFile.jsonData))
            : null;
        if ((mainPrior === null || mainPrior === void 0 ? void 0 : mainPrior.language) === languageCode) {
            const priorJson = this.sourceFile.jsonData;
            changed = !compareJsonStrings(priorJson, posteriorJson);
            if (changed) {
                console.log("updateSubtitle: main update", priorJson);
                console.log("updateSubtitle: main update with", posteriorJson);
                this.sourceFile.jsonData = posteriorJson;
            }
            return changed;
        }
        const variantPrior = this.sourceFile.variantMap.find((key, variant) => key.languageCode === variant.key.languageCode);
        if (variantPrior) {
            const priorJson = variantPrior.value.jsonData;
            changed = !compareJsonStrings(priorJson, posteriorJson);
            if (changed) {
                console.log("updateSubtitle: variant update", variantPrior.value.jsonData);
                console.log("updateSubtitle: variant update with", posteriorJson);
                variantPrior.value.jsonData = posteriorJson;
                this.sourceFile.variantMap.set(variantPrior.key, variantPrior.value);
            }
        }
        return changed;
    }
    toIMediaAsset() {
        var _b, _c, _d, _e, _f, _g, _h, _j;
        const result = {
            sourceFile: this.sourceFile.toILocalCloudFile(),
            derivedFiles: this.derivedFiles,
            proxyFile: (_b = this.proxyFile) === null || _b === void 0 ? void 0 : _b.toILocalCloudFile(),
            proxyFileLow: (_c = this.proxyFileLow) === null || _c === void 0 ? void 0 : _c.toILocalCloudFile(),
            audioFile: (_d = this.audioFile) === null || _d === void 0 ? void 0 : _d.toILocalCloudFile(),
            normalizedAudioFile: (_e = this.normalizedAudioFile) === null || _e === void 0 ? void 0 : _e.toILocalCloudFile(),
            thumbnailFile: (_f = this.thumbnailFile) === null || _f === void 0 ? void 0 : _f.toILocalCloudFile(),
            previewStripeImageFile: (_g = this.previewStripeImageFile) === null || _g === void 0 ? void 0 : _g.toILocalCloudFile(),
            transcription: (_h = this.transcription) === null || _h === void 0 ? void 0 : _h.toITranscription(),
            isUploaded: this.isUploaded,
            version: this.version,
            mediaType: this.mediaType,
            value: this.value,
            newRecord: this.newRecord,
            size: this.size,
            length: this.length,
            durationInSeconds: this.durationInSeconds,
            durationTicks: this.durationTicks,
            durationInFrames: this.durationInFrames,
            inPoint: this.inPoint,
            outPoint: this.outPoint,
            videoSettings: (_j = this.videoSettings) === null || _j === void 0 ? void 0 : _j.toIVideoSettings(),
            videoMetadata: this.videoMetadata,
            analysis: this.analysis,
            languageCode: this.languageCode,
            selectors: this.selectors,
        };
        return result;
    }
    trackingFunction(index, action) {
        return action.sourceFile.filename;
    }
    static fromIMediaAsset(data) {
        return plainToClass(MediaAsset, data);
    }
    static default() {
        return MediaAsset.fromIMediaAsset({
            sourceFile: LocalCloudFile.default(),
            derivedFiles: [],
            isUploaded: false,
            mediaType: MediaType.Video,
            value: "",
            size: 0,
            length: 0,
            durationInSeconds: 0,
        });
    }
}
_a = immerable;
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "sourceFile", void 0);
__decorate([
    Type(() => LcfVariant),
    __metadata("design:type", Array)
], MediaAsset.prototype, "derivedFiles", void 0);
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "proxyFile", void 0);
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "proxyFileLow", void 0);
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "audioFile", void 0);
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "normalizedAudioFile", void 0);
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "thumbnailFile", void 0);
__decorate([
    Type(() => LocalCloudFile),
    __metadata("design:type", LocalCloudFile)
], MediaAsset.prototype, "previewStripeImageFile", void 0);
__decorate([
    Type(() => Transcription),
    __metadata("design:type", Transcription)
], MediaAsset.prototype, "transcription", void 0);
__decorate([
    Type(() => VideoSettings),
    __metadata("design:type", Object)
], MediaAsset.prototype, "videoSettings", void 0);
__decorate([
    Type(() => TimerangeInformation),
    __metadata("design:type", Object)
], MediaAsset.prototype, "analysis", void 0);
export class MediaAssetVariant {
    constructor() {
        this.key = {};
        this.value = MediaAsset.default();
    }
}
__decorate([
    Type(() => MediaAsset),
    __metadata("design:type", MediaAsset)
], MediaAssetVariant.prototype, "value", void 0);
//# sourceMappingURL=mediaAsset.js.map