import "reflect-metadata";
import { immerable } from "immer";
import { Anchor } from "../enums/index.js";
import type { IClip } from "../interfaces/index.js";
import { FrameRange } from "./frameRange.js";
import { Transition } from "./transition.js";
import { Layer } from "./layer/layer.js";
export type ZPreference = "low" | "high";
export declare class TimeLineElement {
    [immerable]: boolean;
    id: string;
    private _libId;
    get libId(): string | null;
    set libId(value: string | null);
    linkedTo: string | null;
    renderDisabled: boolean;
    renderDisabledLocal: boolean;
    zIndex: number | undefined;
    zPreference: ZPreference;
    protected durationInSeconds: number | null;
    protected _absoluteRange: FrameRange;
    get absoluteRange(): FrameRange;
    set absoluteRange(value: FrameRange);
    protected _relativeRange: FrameRange;
    get relativeRange(): FrameRange;
    set relativeRange(value: FrameRange);
    private _cut;
    get cut(): FrameRange | null;
    set cut(value: FrameRange | null);
    get cutOrRange(): FrameRange;
    private _innerBoundaryRange;
    get innerBoundaryRange(): FrameRange | undefined;
    set innerBoundaryRange(value: FrameRange | undefined);
    private _outerBoundaryRange;
    get outerBoundaryRange(): FrameRange;
    set outerBoundaryRange(value: FrameRange);
    transitions: Transition[];
    get inTransitions(): Transition[];
    get outTransitions(): Transition[];
    get inTransition(): Transition | undefined;
    get outTransition(): Transition | undefined;
    get timing(): string;
    get isAnchorStart(): boolean;
    get isAnchorCenter(): boolean;
    get isAnchorEnd(): boolean;
    get isAnchorNone(): boolean;
    get cuttedFramesFront(): number;
    get cuttedFramesBack(): number;
    constructor(partial?: Partial<TimeLineElement>);
    moveToLeft(delta: number): void;
    moveToRight(delta: number): void;
    trimmLeft(delta: number, moveToRight?: boolean, absMoveToRight?: boolean): void;
    trimmRight(delta: number): void;
    setDuration(value: number, factor?: number): void;
    setTransitionsFromLayer(layer: Layer): void;
    setTransitionsFromIClip(iClip: IClip, fps: number, start: boolean, end: boolean): void;
    calculateAbsolutTimingOfTransitions(parent: TimeLineElement): void;
    transitionsByProperyName(property: string): Transition[];
    firstStartTransitionByProperyName(property: string): Transition | undefined;
    firstEndTransitionByProperyName(property: string): Transition | undefined;
    startTransitionsByProperyName(property: string): Transition[];
    endTransitionsByProperyName(property: string): Transition[];
    getTransitions(property: string, range: FrameRange): Transition[];
    setStartTransition(transition: Transition | undefined): void;
    setEndTransition(transition: Transition | undefined): void;
    clearStartEndTransitions(): void;
    removeStartTransitions(property?: string): void;
    removeEndTransitions(property?: string): void;
    getStartTransitions(): Transition[];
    getEndTransitions(): Transition[];
    addTransition(transition: Transition, anchor?: Anchor.Start | Anchor.End | Anchor.Center | undefined, offset?: number): void;
    private updateStartTransitions;
    private updateEndTransitions;
    private updateCenterTransitions;
    private cleanupStartTransition;
    private cleanupEndTransition;
}
//# sourceMappingURL=timeLineElement.d.ts.map