var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import { v4 } from "uuid";
import * as _ from "lodash";
import { immerable } from "immer";
import { Type } from "class-transformer";
import { Anchor } from "../enums/index.js";
import { FrameRange } from "./frameRange.js";
import { Transition } from "./transition.js";
export class TimeLineElement {
    get libId() {
        return this._libId;
    }
    set libId(value) {
        this._libId = value;
    }
    get absoluteRange() {
        return this._absoluteRange;
    }
    set absoluteRange(value) {
        this._absoluteRange = value;
    }
    get relativeRange() {
        return this._relativeRange;
    }
    set relativeRange(value) {
        this._relativeRange = value;
        this.updateStartTransitions();
        this.updateEndTransitions();
    }
    get cut() {
        return this._cut;
    }
    set cut(value) {
        this._cut = value;
    }
    get cutOrRange() {
        var _b;
        return (_b = this.cut) !== null && _b !== void 0 ? _b : FrameRange.fromStart(0, this._relativeRange.duration);
    }
    get innerBoundaryRange() {
        return this._innerBoundaryRange;
    }
    set innerBoundaryRange(value) {
        this._innerBoundaryRange = value;
    }
    get outerBoundaryRange() {
        return this._outerBoundaryRange;
    }
    set outerBoundaryRange(value) {
        this._outerBoundaryRange = value;
    }
    get inTransitions() {
        return this.getStartTransitions();
    }
    get outTransitions() {
        return this.getEndTransitions();
    }
    get inTransition() {
        return this.inTransitions.at(0);
    }
    get outTransition() {
        return this.outTransitions.at(0);
    }
    // #endregion
    // #region Computed properties
    get timing() {
        return `ABS: ${this.absoluteRange} REL: ${this.relativeRange} CUT: ${this.cut}`;
    }
    get isAnchorStart() {
        return this.relativeRange.anchor === Anchor.Start;
    }
    get isAnchorCenter() {
        return this.relativeRange.anchor === Anchor.Center;
    }
    get isAnchorEnd() {
        return this.relativeRange.anchor === Anchor.End;
    }
    get isAnchorNone() {
        return !this.relativeRange.anchor;
    }
    get cuttedFramesFront() {
        if (!this.cut)
            return 0;
        return this.cut.start;
    }
    get cuttedFramesBack() {
        if (!this.cut)
            return 0;
        return (this.outerBoundaryRange.duration -
            this.relativeRange.duration -
            this.cut.start);
    }
    // #endregion
    constructor(partial) {
        this[_a] = true; // Option 1
        this.id = v4().substring(0, 6);
        this._libId = null;
        // #region Lateral properties
        this.linkedTo = null;
        this.renderDisabled = false;
        this.renderDisabledLocal = false;
        this.zIndex = undefined;
        this.zPreference = "low";
        // #endregion
        // #region Temporal properties
        this.durationInSeconds = 0;
        this._absoluteRange = FrameRange.empty();
        this._relativeRange = FrameRange.empty();
        this._cut = null;
        this._innerBoundaryRange = undefined;
        this._outerBoundaryRange = FrameRange.positive;
        // #endregion
        // #region Transition properties
        this.transitions = [];
        if (partial) {
            Object.assign(this, partial);
            if (partial.relativeRange)
                this.relativeRange = FrameRange.fromIFrameRange(partial.relativeRange);
            if (partial.absoluteRange)
                this.absoluteRange = FrameRange.fromIFrameRange(partial.absoluteRange);
            if (partial.cut)
                this.cut = FrameRange.fromIFrameRange(partial.cut);
            if (partial.innerBoundaryRange)
                this.innerBoundaryRange = FrameRange.fromIFrameRange(partial.innerBoundaryRange);
            if (partial.outerBoundaryRange)
                this.outerBoundaryRange = FrameRange.fromIFrameRange(partial.outerBoundaryRange);
            if (partial.transitions)
                this.transitions = partial.transitions.map((t) => Transition.fromTransition(t));
        }
    }
    moveToLeft(delta) {
        this.absoluteRange = this.absoluteRange.add(delta * -1).round();
        this.relativeRange = this.relativeRange.add(delta * -1).round();
    }
    moveToRight(delta) {
        this.absoluteRange = this.absoluteRange.add(delta).round();
        this.relativeRange = this.relativeRange.add(delta).round();
    }
    trimmLeft(delta, moveToRight = false, absMoveToRight = false) {
        var _b, _c, _d, _e, _f, _g;
        const clone = ((_b = this.cut) !== null && _b !== void 0 ? _b : this.outerBoundaryRange).clone();
        this.cut = clone
            .addStart(delta)
            .pushApart(this.innerBoundaryRange)
            .clamp((_c = this.outerBoundaryRange) !== null && _c !== void 0 ? _c : FrameRange.positive)
            .round();
        if (moveToRight === true) {
            this.relativeRange = this.relativeRange
                .addStart(delta)
                .pushApart(this.innerBoundaryRange)
                .limitDuration((_d = this.outerBoundaryRange) === null || _d === void 0 ? void 0 : _d.duration)
                .round();
        }
        else {
            this.relativeRange = this.relativeRange
                .subtractEnd(delta)
                .pushApart(this.innerBoundaryRange)
                .limitDuration((_e = this.outerBoundaryRange) === null || _e === void 0 ? void 0 : _e.duration)
                .round();
        }
        if (absMoveToRight === true) {
            this.absoluteRange = this.absoluteRange
                .addStart(delta)
                .pushApart(this.innerBoundaryRange)
                .limitDuration((_f = this.outerBoundaryRange) === null || _f === void 0 ? void 0 : _f.duration)
                .round();
        }
        else {
            this.absoluteRange = this.absoluteRange
                .subtractEnd(delta)
                .pushApart(this.innerBoundaryRange)
                .limitDuration((_g = this.outerBoundaryRange) === null || _g === void 0 ? void 0 : _g.duration)
                .round();
        }
        if (this.cut)
            this.cut.duration = this.relativeRange.duration;
    }
    trimmRight(delta) {
        var _b, _c;
        const modifiedAbsolute = this.absoluteRange
            .subtractEnd(delta)
            .pushApart(this.innerBoundaryRange)
            .limitDuration((_b = this.outerBoundaryRange) === null || _b === void 0 ? void 0 : _b.duration)
            .round();
        this.absoluteRange = modifiedAbsolute;
        const modifiedRelative = this.relativeRange
            .subtractEnd(delta)
            .pushApart(this.innerBoundaryRange)
            .limitDuration((_c = this.outerBoundaryRange) === null || _c === void 0 ? void 0 : _c.duration)
            .round();
        this.relativeRange = modifiedRelative;
        if (this.cut && this.cut.duration !== modifiedRelative.duration) {
            this.cut.duration = modifiedRelative.duration;
        }
    }
    setDuration(value, factor = 1) {
        const duration = Math.round(value * factor);
        this.absoluteRange = this.absoluteRange.withDurationFromStart(duration, false);
        this.relativeRange = this.relativeRange.withDurationFromStart(duration, false);
    }
    // #region Transitions
    setTransitionsFromLayer(layer) {
        this.clearStartEndTransitions();
        const startAudio = layer.firstStartTransitionByProperyName("volume");
        if (startAudio)
            this.addTransition(startAudio, Anchor.Start);
        const endAudio = layer.firstEndTransitionByProperyName("volume");
        if (endAudio)
            this.addTransition(endAudio, Anchor.End);
        const startVideo = layer.firstStartTransitionByProperyName("opacity");
        if (startVideo)
            this.addTransition(startVideo, Anchor.Start);
        const endVideo = layer.firstEndTransitionByProperyName("opacity");
        if (endVideo)
            this.addTransition(endVideo, Anchor.End);
    }
    setTransitionsFromIClip(iClip, fps, start, end) {
        if (iClip === null || iClip === void 0 ? void 0 : iClip.transition) {
            const copyInTransition = true;
            const copyOutTransition = true;
            this.removeStartTransitions("volume");
            this.removeStartTransitions("opacity");
            if (start && copyInTransition) {
                const audioIn = Transition.audioInFromIClipTransition(iClip === null || iClip === void 0 ? void 0 : iClip.transition, fps);
                if (audioIn instanceof Transition)
                    this.addTransition(audioIn, Anchor.Start);
                const videoIn = Transition.videoInFromIClipTransition(iClip === null || iClip === void 0 ? void 0 : iClip.transition, fps);
                if (videoIn instanceof Transition)
                    this.addTransition(videoIn, Anchor.Start);
            }
            this.removeEndTransitions("volume");
            this.removeEndTransitions("opacity");
            if (end && copyOutTransition) {
                const audioOut = Transition.audioOutFromIClipTransition(iClip === null || iClip === void 0 ? void 0 : iClip.transition, fps);
                if (audioOut instanceof Transition)
                    this.addTransition(audioOut, Anchor.End);
                const videoOut = Transition.videoOutFromIClipTransition(iClip === null || iClip === void 0 ? void 0 : iClip.transition, fps);
                if (videoOut instanceof Transition)
                    this.addTransition(videoOut, Anchor.End);
            }
        }
    }
    calculateAbsolutTimingOfTransitions(parent) {
        const updatedTransitions = this.transitions.map((transition) => {
            const absoluteRange = transition.relativeRange.add(parent.absoluteRange.start);
            return transition.withAbsoluteRange(absoluteRange);
        });
        this.transitions = updatedTransitions;
    }
    transitionsByProperyName(property) {
        const transitionsByPropertyName = this.transitions.filter((t) => t.property === property);
        return transitionsByPropertyName;
    }
    firstStartTransitionByProperyName(property) {
        return this.startTransitionsByProperyName(property)
            .sort(Transition.compareRelativeStart)
            .at(0);
    }
    firstEndTransitionByProperyName(property) {
        return this.endTransitionsByProperyName(property)
            .sort(Transition.compareRelativeEnd)
            .reverse()
            .at(0);
    }
    startTransitionsByProperyName(property) {
        const transitionsByPropertyName = this.transitions.filter((t) => t.property === property &&
            (t.relativeRange.start === 0 || t.relativeRange.anchor === Anchor.Start));
        return transitionsByPropertyName;
    }
    endTransitionsByProperyName(property) {
        const transitionsByPropertyName = this.transitions.filter((t) => t.property === property &&
            (t.relativeRange.end === this.relativeRange.end ||
                t.relativeRange.anchor === Anchor.End));
        return transitionsByPropertyName;
    }
    getTransitions(property, range) {
        const transitionsByPropertyName = this.transitions.filter((t) => t.property === property);
        const ownTransitions = transitionsByPropertyName.filter((t) => t.absoluteRange.isInside(range.addPadding(-1)));
        const parentTransitions = [];
        const transitions = [...ownTransitions, ...parentTransitions];
        return transitions;
    }
    setStartTransition(transition) {
        if (!transition) {
            return;
        }
        this.removeStartTransitions(transition.property);
        let clone = _.cloneDeep(transition);
        clone.relativeRange = this.relativeRange.withDurationFromStart(clone.relativeRange.duration);
        clone = clone.toIn();
        this.transitions.push(clone);
    }
    setEndTransition(transition) {
        if (!transition) {
            return;
        }
        this.removeEndTransitions(transition.property);
        let transitionClone = _.cloneDeep(transition);
        transitionClone.relativeRange = this.relativeRange.withDurationFromEnd(transitionClone.relativeRange.duration);
        transitionClone = transitionClone.toOut();
        this.transitions.push(transitionClone);
    }
    // setStartTransition(transition: Transition | undefined) {
    //   if (!transition) {
    //     return;
    //   }
    //   this.removeStartTransitions(transition.property);
    //   const clone = produce(transition, (draftTransition) => {
    //     draftTransition.relativeRange = this.relativeRange.withDurationFromStart(
    //       draftTransition.relativeRange.duration
    //     );
    //     draftTransition = draftTransition.toIn();
    //   });
    //   this.transitions.push(clone);
    // }
    // setEndTransition(transition: Transition | undefined) {
    //   if (!transition) {
    //     return;
    //   }
    //   this.removeEndTransitions(transition.property);
    //   const clone = produce(transition, (draftTransition) => {
    //     draftTransition.relativeRange = this.relativeRange.withDurationFromEnd(
    //       draftTransition.relativeRange.duration
    //     );
    //     draftTransition = draftTransition.toOut();
    //   });
    //   this.transitions.push(clone);
    // }
    clearStartEndTransitions() {
        this.removeStartTransitions("volume");
        this.removeStartTransitions("opacity");
        this.removeEndTransitions("volume");
        this.removeEndTransitions("opacity");
    }
    removeStartTransitions(property) {
        const removers = property
            ? this.startTransitionsByProperyName(property)
            : this.getStartTransitions();
        this.transitions = this.transitions.filter((t) => !removers.includes(t));
    }
    removeEndTransitions(property) {
        const removers = property
            ? this.endTransitionsByProperyName(property)
            : this.getEndTransitions();
        this.transitions = this.transitions.filter((t) => !removers.includes(t));
    }
    getStartTransitions() {
        return this.transitions
            .filter((t) => t.relativeRange.start === 0 || t.relativeRange.anchor === Anchor.Start)
            .sort(Transition.compareRelativeEnd);
    }
    getEndTransitions() {
        return this.transitions
            .filter((t) => t.relativeRange.end === this.relativeRange.duration ||
            t.relativeRange.anchor === Anchor.End)
            .sort(Transition.compareRelativeStart);
    }
    addTransition(transition, anchor = undefined, offset = 0) {
        if (this instanceof Transition)
            throw new Error("Transition can not add a Transition.");
        const transitionClone = _.clone(transition);
        if (anchor === Anchor.Start)
            transitionClone.setRangeFromStart(offset);
        if (anchor === Anchor.End)
            transitionClone.setRangeFromEnd(this.relativeRange.duration - offset);
        if (anchor === Anchor.Center)
            transitionClone.setRangeFromCenter(this.relativeRange.center);
        transitionClone.clamp(this.relativeRange);
        this.transitions.push(transitionClone);
    }
    updateStartTransitions() {
        const startTransitions = this.getStartTransitions();
        startTransitions.forEach((t) => {
            t.relativeRange = this.relativeRange
                .withDurationFromStart(t.relativeRange.duration)
                .withAnchor(Anchor.Start);
        });
    }
    updateEndTransitions() {
        const endTransitions = this.getEndTransitions();
        endTransitions.forEach((t) => {
            t.relativeRange = this.relativeRange
                .withDurationFromEnd(t.relativeRange.duration)
                .withAnchor(Anchor.End);
        });
    }
    updateCenterTransitions() {
        // Const centerTransitions = this.transitions.filter(
        //   (t) => t.relativeRange.anchor === Anchor.Center
        // );
        // centerTransitions.forEach((t) => {
        //   t.relativeRange = this.relativeRange.withAnchor(Anchor.Center);
        // });
    }
    cleanupStartTransition(property) {
        const transitionsByPropertyName = this.startTransitionsByProperyName(property);
        // Get subset of array transitionsByPropertyName from index 1 to end
        const removers = transitionsByPropertyName.slice(1);
        // Remove all transitions in removers from transitions
        this.transitions = this.transitions.filter((t) => !removers.includes(t));
    }
    cleanupEndTransition(property) {
        const transitionsByPropertyName = this.endTransitionsByProperyName(property);
        // Get subset of array transitionsByPropertyName from index 1 to end
        const removers = transitionsByPropertyName.slice(1);
        // Remove all transitions in removers from transitions
        this.transitions = this.transitions.filter((t) => !removers.includes(t));
    }
}
_a = immerable;
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], TimeLineElement.prototype, "_absoluteRange", void 0);
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], TimeLineElement.prototype, "_relativeRange", void 0);
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], TimeLineElement.prototype, "_cut", void 0);
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], TimeLineElement.prototype, "_innerBoundaryRange", void 0);
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], TimeLineElement.prototype, "_outerBoundaryRange", void 0);
__decorate([
    Type(() => Transition),
    __metadata("design:type", Array)
], TimeLineElement.prototype, "transitions", void 0);
//# sourceMappingURL=timeLineElement.js.map