var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
import "reflect-metadata";
import { immerable } from "immer";
import { Type } from "class-transformer";
import { Time } from "./time.js";
export class TimeRange {
    get duration() {
        return this.end.subtract(this.start);
    }
    constructor(start, end) {
        this[_a] = true; // Option 1
        this.start = Time.min();
        this.end = Time.fromSeconds(1);
        if (start && end) {
            const startTime = Time.fromITime(start);
            const endTime = Time.fromITime(end);
            if (startTime.isGreaterThanOrEqualTo(endTime)) {
                throw new Error("Start time cannot be later than end time");
            }
            this.start = startTime;
            this.end = endTime;
        }
    }
    toString() {
        return `${this.start.toString()} - ${this.end.toString()}`;
    }
    // #region Comparison
    contains(time) {
        const t = Time.fromITime(time);
        return (t.isGreaterThanOrEqualTo(this.start) && t.isLessThanOrEqualTo(this.end));
    }
    before(time) {
        return Time.fromITime(time).isLessThan(this.start);
    }
    after(time) {
        return Time.fromITime(time).isGreaterThan(this.end);
    }
    // #endregion
    clone() {
        return new TimeRange(Time.fromITime(this.start), Time.fromITime(this.end));
    }
    static fromMilliseconds(start, end) {
        return new TimeRange(Time.fromMilliseconds(start), Time.fromMilliseconds(end));
    }
    static fromString(timeRangeString) {
        const [startString, endString] = timeRangeString.split(" - ");
        const start = Time.fromString(startString);
        const end = Time.fromString(endString);
        return new TimeRange(start, end);
    }
    static default() {
        return new TimeRange(Time.min(), Time.fromSeconds(1));
    }
}
_a = immerable;
__decorate([
    Type(() => Time),
    __metadata("design:type", Time)
], TimeRange.prototype, "start", void 0);
__decorate([
    Type(() => Time),
    __metadata("design:type", Time)
], TimeRange.prototype, "end", void 0);
//# sourceMappingURL=timerange.js.map