import "reflect-metadata";
import { immerable } from "immer";
import { Anchor, TransitionCurve } from "../enums/index.js";
import type { IClipTransition, ITransition } from "../interfaces/index.js";
import { FrameRange } from "./frameRange.js";
import { NumberRange } from "./number-range.js";
export declare class Transition {
    [immerable]: boolean;
    private _curve;
    get curve(): TransitionCurve;
    set curve(value: TransitionCurve);
    private _property;
    get property(): string;
    set property(value: string);
    private _valueRange;
    get valueRange(): NumberRange;
    set valueRange(value: NumberRange);
    private _absoluteRange;
    get absoluteRange(): FrameRange;
    set absoluteRange(value: FrameRange);
    private _relativeRange;
    get relativeRange(): FrameRange;
    set relativeRange(value: FrameRange);
    get halfDuration(): number;
    get isAnchorStart(): boolean;
    get isAnchorCenter(): boolean;
    get isAnchorEnd(): boolean;
    get isAnchorNone(): boolean;
    constructor(property: string, valueRange: NumberRange, duration: number, curve: TransitionCurve, anchor?: Anchor);
    set(transition: Transition): void;
    getAbsoluteRange(startOfParent: number): FrameRange;
    setRangeFromCenter(centerFrame: number): void;
    setRangeFromStart(startFrame: number): void;
    setRangeFromEnd(lastFrame: number): void;
    clamp(range?: FrameRange): void;
    calculateNumber(frame: number): number;
    calculateNormPosition(frame: number): number;
    setAbsoluteRange(offset: number): void;
    toITransition(): ITransition;
    clone(): this;
    toIn(): Transition;
    toOut(): Transition;
    withAbsoluteRange(range: FrameRange): Transition;
    toIClipTransition(fps: number): IClipTransition;
    isSymetricWith(other: Transition): boolean;
    toString: () => string;
    private calculator;
    static default(): Transition;
    static cutTransitionValue(frame: number, transitionStart: Transition | null, transitionEnd: Transition | null): number;
    static compareRelativeStart(a: Transition, b: Transition): number;
    static compareRelativeEnd(a: Transition, b: Transition): number;
    static compareRelativeDuration(a: Transition, b: Transition): number;
    static crossFadeTransitionValue(frame: number, transitionStart: Transition | null, transitionEnd: Transition | null, targetValue?: number): number;
    static linearInTransitionValue(frame: number, transitionStart: Transition | null, transitionEnd: Transition | null): number;
    static getActive(transitions: Transition[], frame: number): Transition | undefined;
    static calculateNumberFromTransitions(transitions: Transition[], frame: number): number;
    static activeTransitionFromTransitions(transitions: Transition[], frame: number): Transition | undefined;
    static getRangeInsideTransitions(transitions: Transition[], lastFrame: number): FrameRange;
    static fromITransitions(transitions: ITransition[] | null): Transition[];
    static fromITransition(iTransition: ITransition): Transition;
    static toITransitions(transitions: Transition[]): ITransition[] | null;
    static fromTransition(fromT: Transition): Transition;
    static audioIn(duration: number, fps?: number): Transition;
    static audioOut(duration: number, fps?: number): Transition;
    static audioInFromIClipTransition(iClipTrans: IClipTransition, fps: number): Transition | null;
    static audioOutFromIClipTransition(iClipTrans: IClipTransition, fps: number): Transition | null;
    static videoInFromIClipTransition(iClipTrans: IClipTransition, fps: number): Transition | null;
    static videoOutFromIClipTransition(iClipTrans: IClipTransition, fps: number): Transition | null;
}
export declare abstract class TransitionCalculator {
    abstract calculate(frame: number, range: FrameRange, valueFrom: number, valueTo: number): number;
}
export declare class CutTransitionCalculator extends TransitionCalculator {
    calculate(frame: number, range: FrameRange, valueFrom: number, valueTo: number): number;
}
export declare class LinearTransitionCalculator extends TransitionCalculator {
    calculate(frame: number, range: FrameRange, valueFrom: number, valueTo: number): number;
}
export declare class CubicTransitionCalculator extends TransitionCalculator {
    calculate(frame: number, range: FrameRange, valueFrom: number, valueTo: number): number;
}
export declare class QuadTransitionCalculator extends TransitionCalculator {
    calculate(frame: number, range: FrameRange, valueFrom: number, valueTo: number): number;
}
export declare function distinctTransitions(transitions: ITransition[]): ITransition[];
//# sourceMappingURL=transition.d.ts.map