var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
/* eslint-disable capitalized-comments */
import _ from "lodash";
import "reflect-metadata";
import { Type } from "class-transformer";
import { stringify } from "flatted";
import { interpolate, Easing } from "remotion";
import { immerable } from "immer";
import { Anchor, TransitionCurve } from "../enums/index.js";
import { castStringToEnum, clamp } from "../extensions/index.js";
import { Frame } from "./frame.js";
import { FrameRange } from "./frameRange.js";
import { NumberRange } from "./number-range.js";
export class Transition {
    get curve() {
        return this._curve;
    }
    set curve(value) {
        this._curve = value;
    }
    get property() {
        return this._property;
    }
    set property(value) {
        this._property = value;
    }
    get valueRange() {
        return this._valueRange;
    }
    set valueRange(value) {
        this._valueRange = value;
    }
    get absoluteRange() {
        return this._absoluteRange;
    }
    set absoluteRange(value) {
        this._absoluteRange = value;
    }
    get relativeRange() {
        return this._relativeRange;
    }
    set relativeRange(value) {
        this._relativeRange = value;
    }
    get halfDuration() {
        return this.relativeRange.duration / 2;
    }
    get isAnchorStart() {
        return this.relativeRange.anchor === Anchor.Start;
    }
    get isAnchorCenter() {
        return this.relativeRange.anchor === Anchor.Center;
    }
    get isAnchorEnd() {
        return this.relativeRange.anchor === Anchor.End;
    }
    get isAnchorNone() {
        return !this.relativeRange.anchor;
    }
    // eslint-disable-next-line max-params
    constructor(property, valueRange, duration, curve, anchor) {
        // Super();
        this[_a] = true; // Option 1
        this._curve = TransitionCurve.Cut;
        this._absoluteRange = FrameRange.empty();
        this._relativeRange = FrameRange.empty();
        this.toString = () => {
            return `TRS: ${this.property}   a:${this.relativeRange.anchor}   d: ${this.relativeRange.duration}   c: ${this.curve}    Val: ${this.valueRange.start} -> ${this.valueRange.end}`;
        };
        this._property = property;
        this._valueRange = valueRange;
        this._curve = curve;
        this.relativeRange = FrameRange.fromStart(0, duration).withAnchor(anchor);
    }
    set(transition) {
        this.valueRange = transition.valueRange;
        this.curve = transition.curve;
        this.relativeRange = transition.relativeRange;
    }
    getAbsoluteRange(startOfParent) {
        this.absoluteRange = this.relativeRange.add(startOfParent);
        return this.absoluteRange;
    }
    setRangeFromCenter(centerFrame) {
        this.relativeRange = FrameRange.fromCenter(centerFrame, this.relativeRange.duration).withAnchor(Anchor.Center);
    }
    setRangeFromStart(startFrame) {
        this.relativeRange = FrameRange.fromStart(startFrame, this.relativeRange.duration).withAnchor(Anchor.Start);
    }
    setRangeFromEnd(lastFrame) {
        this.relativeRange = FrameRange.fromEnd(lastFrame, this.relativeRange.duration).withAnchor(Anchor.End);
    }
    clamp(range) {
        if (!range)
            return;
        this.relativeRange = this.relativeRange.clamp(range);
    }
    calculateNumber(frame) {
        return this.calculator().calculate(frame, this.absoluteRange, this.valueRange.start, this.valueRange.end);
    }
    calculateNormPosition(frame) {
        return this.absoluteRange.calculateNormPosition(frame);
    }
    setAbsoluteRange(offset) {
        this.absoluteRange = this.relativeRange.add(offset);
    }
    toITransition() {
        return {
            property: this.property,
            valueFrom: this.valueRange.start,
            valueTo: this.valueRange.end,
            relativeRange: this.relativeRange.toIFrameRange(),
            curve: this.curve,
        };
    }
    clone() {
        return _.cloneDeep(this);
        // return new Transition(
        //   this.property,
        //   new NumberRange(this.valueRange.start, this.valueRange.end),
        //   this.relativeRange.duration,
        //   this.curve,
        //   this.relativeRange?.anchor ?? undefined
        // );
    }
    toIn() {
        const clone = this.clone();
        clone.valueRange = this.valueRange.toAscending();
        clone.relativeRange.anchor = Anchor.Start;
        return clone;
    }
    toOut() {
        const clone = this.clone();
        clone.valueRange = this.valueRange.toDescending();
        clone.relativeRange.anchor = Anchor.End;
        return clone;
    }
    withAbsoluteRange(range) {
        const clone = this.clone();
        clone.absoluteRange = range;
        return clone;
    }
    toIClipTransition(fps) {
        return {
            name: this.property,
            duration: Frame.secondsFromFrames(this.relativeRange.duration, fps),
            audioInCurve: this.curve,
            audioOutCurve: this.curve,
        };
    }
    isSymetricWith(other) {
        return (this.property === other.property &&
            this.curve === other.curve &&
            this.relativeRange.duration === other.relativeRange.duration);
    }
    calculator() {
        if (this.curve === TransitionCurve.Cut)
            return new CutTransitionCalculator();
        if (this.curve === TransitionCurve.Linear)
            return new LinearTransitionCalculator();
        if (this.curve === TransitionCurve.Cubic)
            return new CubicTransitionCalculator();
        if (this.curve === TransitionCurve.Quad)
            return new QuadTransitionCalculator();
        return new CutTransitionCalculator();
    }
    static default() {
        return new Transition("opacity", NumberRange.unitRange(), 1, TransitionCurve.Cut);
    }
    static cutTransitionValue(frame, transitionStart, transitionEnd) {
        if (transitionStart && transitionEnd) {
            return interpolate(frame, transitionStart.absoluteRange.getCutCenterInOutWithOther(transitionEnd.absoluteRange), [0, 1, 1, 0], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        if (transitionStart && !transitionEnd) {
            return interpolate(frame, transitionStart.absoluteRange.getCutCenter(), [0, 1], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        if (!transitionStart && transitionEnd) {
            return interpolate(frame, transitionEnd.absoluteRange.getCutCenter(), [1, 0], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        return 1;
    }
    static compareRelativeStart(a, b) {
        return FrameRange.compareStart(a.relativeRange, b.relativeRange);
    }
    static compareRelativeEnd(a, b) {
        return FrameRange.compareEnd(a.relativeRange, b.relativeRange);
    }
    static compareRelativeDuration(a, b) {
        return FrameRange.compareDuration(a.relativeRange, b.relativeRange);
    }
    static crossFadeTransitionValue(frame, transitionStart, transitionEnd, targetValue = 1) {
        if (transitionStart && transitionEnd) {
            return interpolate(frame, transitionStart.absoluteRange.getFadeInOutWithOther(transitionEnd.absoluteRange), [0, targetValue, targetValue, 0], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        if (transitionStart && !transitionEnd) {
            return interpolate(frame, [
                transitionStart.absoluteRange.start,
                transitionStart.absoluteRange.end,
            ], [0, targetValue], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        if (!transitionStart && transitionEnd) {
            return interpolate(frame, [transitionEnd.absoluteRange.start, transitionEnd.absoluteRange.end], [targetValue, 0], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        return 1;
    }
    static linearInTransitionValue(frame, transitionStart, transitionEnd) {
        if (transitionStart && transitionEnd) {
            return interpolate(frame, transitionStart.absoluteRange.getFadeInOutWithOther(transitionEnd.absoluteRange), [0, 1, 1, 1], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        if (transitionStart && !transitionEnd) {
            return interpolate(frame, [
                transitionStart.absoluteRange.start,
                transitionStart.absoluteRange.end,
            ], [0, 1], {
                extrapolateRight: "clamp",
                extrapolateLeft: "clamp",
            });
        }
        if (!transitionStart && transitionEnd) {
            return 1;
        }
        return 1;
    }
    static getActive(transitions, frame) {
        const activeTransitions = transitions.filter((t) => t.absoluteRange.containsFrame(frame));
        if (activeTransitions.length < 1)
            return undefined;
        return activeTransitions.at(0);
    }
    static calculateNumberFromTransitions(transitions, frame) {
        if (!transitions || transitions.length === 0)
            return 1;
        const activeTransitions = transitions.filter((t) => t.absoluteRange.containsFrame(frame));
        if (activeTransitions.length < 1)
            return 1;
        const activeTransition = activeTransitions.at(0);
        if (activeTransition) {
            const result = activeTransition.calculateNumber(frame);
            // Console.log(`Transition.calculateNumberFromTransitions ${result}`);
            return result;
        }
        return 1;
    }
    static activeTransitionFromTransitions(transitions, frame) {
        if (!transitions || transitions.length === 0)
            return undefined;
        const activeTransitions = transitions.filter((t) => t.absoluteRange.containsFrame(frame));
        if (activeTransitions.length < 1)
            return undefined;
        const activeTransition = activeTransitions.at(0);
        return activeTransition;
    }
    static getRangeInsideTransitions(transitions, lastFrame) {
        const starts = transitions.filter((t) => t.relativeRange.start === 0);
        starts.sort(Transition.compareRelativeStart);
        const first = starts.at(0);
        const ends = transitions.filter((t) => t.relativeRange.end === lastFrame);
        ends.sort(Transition.compareRelativeEnd);
        const last = ends.at(0);
        const result = FrameRange.empty();
        if (first)
            result.start = first.relativeRange.end;
        if (last)
            result.end = last.relativeRange.start;
        return result;
    }
    static fromITransitions(transitions) {
        if (!transitions)
            return [];
        return transitions.map((t) => Transition.fromITransition(t));
    }
    static fromITransition(iTransition) {
        const frames = iTransition.relativeRange.end - iTransition.relativeRange.start;
        const tansition = new Transition(iTransition.property, new NumberRange(iTransition.valueFrom, iTransition.valueTo), frames, iTransition.curve);
        tansition.relativeRange = FrameRange.fromIFrameRange(iTransition.relativeRange);
        return tansition;
    }
    static toITransitions(transitions) {
        if (!transitions)
            return null;
        return transitions.map((t) => t.toITransition());
    }
    static fromTransition(fromT) {
        return new Transition(fromT.property, new NumberRange(fromT.valueRange.start, fromT.valueRange.end), fromT.absoluteRange.duration, fromT.curve);
    }
    static audioIn(duration, fps) {
        let frames = clamp(duration, 1, Number.MAX_SAFE_INTEGER);
        if (fps) {
            frames = Frame.framesFromSeconds(duration, fps);
            frames = clamp(frames, 1, Number.MAX_SAFE_INTEGER);
        }
        const curve = TransitionCurve.Cubic;
        return new Transition("volume", NumberRange.unitRange(), frames, curve);
    }
    static audioOut(duration, fps) {
        let frames = clamp(duration, 1, Number.MAX_SAFE_INTEGER);
        if (fps) {
            frames = Frame.framesFromSeconds(duration, fps);
            frames = clamp(frames, 1, Number.MAX_SAFE_INTEGER);
        }
        const curve = TransitionCurve.Cubic;
        return new Transition("volume", NumberRange.unitRange().toDescending(), frames, curve);
    }
    static audioInFromIClipTransition(iClipTrans, fps) {
        if (iClipTrans.duration <= 0)
            return null;
        if (!iClipTrans.audioInCurve)
            return null;
        const frames = Frame.framesFromSeconds(iClipTrans.duration, fps);
        const curve = iClipTrans.audioInCurve === "losi"
            ? TransitionCurve.Cubic
            : castStringToEnum(iClipTrans.audioInCurve, TransitionCurve);
        return new Transition("volume", NumberRange.unitRange(), frames, curve, Anchor.Start);
    }
    static audioOutFromIClipTransition(iClipTrans, fps) {
        if (iClipTrans.duration <= 0)
            return null;
        if (!iClipTrans.audioOutCurve)
            return null;
        const frames = Frame.framesFromSeconds(iClipTrans.duration, fps);
        const curve = iClipTrans.audioOutCurve === "losi"
            ? TransitionCurve.Cubic
            : castStringToEnum(iClipTrans.audioOutCurve, TransitionCurve);
        return new Transition("volume", NumberRange.unitRange().toDescending(), frames, curve, Anchor.End);
    }
    static videoInFromIClipTransition(iClipTrans, fps) {
        if (iClipTrans.duration <= 0)
            return null;
        if (!iClipTrans.audioInCurve)
            return null;
        const frames = Frame.framesFromSeconds(iClipTrans.duration, fps);
        return new Transition("opacity", NumberRange.unitRange(), frames, TransitionCurve.Cut, Anchor.Start);
    }
    static videoOutFromIClipTransition(iClipTrans, fps) {
        if (iClipTrans.duration <= 0)
            return null;
        if (!iClipTrans.audioOutCurve)
            return null;
        const frames = Frame.framesFromSeconds(iClipTrans.duration, fps);
        return new Transition("opacity", NumberRange.unitRange().toDescending(), frames, TransitionCurve.Cut, Anchor.End);
    }
}
_a = immerable;
__decorate([
    Type(() => NumberRange),
    __metadata("design:type", NumberRange)
], Transition.prototype, "_valueRange", void 0);
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], Transition.prototype, "_absoluteRange", void 0);
__decorate([
    Type(() => FrameRange),
    __metadata("design:type", Object)
], Transition.prototype, "_relativeRange", void 0);
export class TransitionCalculator {
}
export class CutTransitionCalculator extends TransitionCalculator {
    calculate(frame, range, valueFrom, valueTo) {
        const cutRange = range.getCutCenter();
        const value = interpolate(frame, cutRange, [valueFrom, valueTo], {
            extrapolateRight: "clamp",
            extrapolateLeft: "clamp",
        });
        return value;
    }
}
export class LinearTransitionCalculator extends TransitionCalculator {
    calculate(frame, range, valueFrom, valueTo) {
        const fadeRange = range.getFade();
        return interpolate(frame, fadeRange, [valueFrom, valueTo], {
            extrapolateRight: "clamp",
            extrapolateLeft: "clamp",
        });
    }
}
export class CubicTransitionCalculator extends TransitionCalculator {
    calculate(frame, range, valueFrom, valueTo) {
        const fadeRange = range.getFade();
        return interpolate(frame, fadeRange, [valueFrom, valueTo], {
            easing: Easing.cubic,
            extrapolateRight: "clamp",
            extrapolateLeft: "clamp",
        });
    }
}
export class QuadTransitionCalculator extends TransitionCalculator {
    calculate(frame, range, valueFrom, valueTo) {
        const fadeRange = range.getFade();
        return interpolate(frame, fadeRange, [valueFrom, valueTo], {
            easing: Easing.quad,
            extrapolateRight: "clamp",
            extrapolateLeft: "clamp",
        });
    }
}
export function distinctTransitions(transitions) {
    const seen = new Set();
    return transitions.filter((t) => {
        const key = stringify(t);
        if (!seen.has(key)) {
            seen.add(key);
            return true;
        }
        return false;
    });
}
//# sourceMappingURL=transition.js.map