import "reflect-metadata";
import type { ICollectionDocument } from "../interfaces/index.js";
export declare class TrieNode {
    Value: string;
    IsEndOfWord: boolean;
    Children: Map<string, TrieNode>;
    Parent?: TrieNode;
    get directChildren(): TrieNode[];
    References: string[];
    constructor(Value?: string, IsEndOfWord?: boolean, parent?: TrieNode);
    initParents(): void;
    get Word(): string;
    get depth(): number;
    private getDepthRecursive;
    private concatCharsReverse;
    HasChild(value: string): boolean;
    HasChildren(): boolean;
    GetChild(value: string): TrieNode | undefined;
    GetChildren(levels?: number): TrieNode[];
    AddChild(value: string): void;
    HasReferences(): boolean;
    GetReferences(): string[];
    AddReference(reference: string): void;
    RemoveReference(reference: string): void;
}
export declare class Trie {
    Root: TrieNode;
    constructor(jsonString?: string, root?: TrieNode);
    Contains(word: string, reference?: string): boolean;
    GetReferences(word: string, canMissCharactersAtEnd?: number): string[];
    Insert(word: string, reference?: string): void;
    Remove(word: string, reference?: string): void;
    private _Remove;
    FindWordsWithReferences(searchTerm: string, canSkipCharacters?: number, canMissCharactersAtEnd?: number): Array<[string, string[]]>;
    FindLastNodesOf(searchTerm: string, canSkipCharacters?: number): TrieNode[];
    private FindLastNodeOf;
    FindWords(prefix: string): string[];
    private _FindWords;
    nodes(startNode?: TrieNode): TrieNode[];
    nodesWithMinDepthToLeaf(minDepth: number, startNode?: TrieNode): TrieNode[];
    depthToMaxLeafe(node: TrieNode): number;
    TraversePreOrder(visit: (node: TrieNode) => void): void;
    TraverseInOrder(visit: (node: TrieNode) => void): void;
    TraversePostOrder(visit: (node: TrieNode) => void): void;
    private TraversePreOrderRecursive;
    private TraverseInOrderRecursive;
    private TraversePostOrderRecursive;
    ToJson(): string;
    ToContainer(): ITrieDtoContainer;
    static FromJson(jsonObject: any): Trie | undefined;
}
export interface ITrieDtoContainer extends ICollectionDocument {
    data: string;
}
//# sourceMappingURL=trie.d.ts.map