var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var TrieNode_1, Trie_1;
/* eslint-disable default-param-last */
import "reflect-metadata";
import { parse, stringify } from "flatted";
import { JsonConvert, JsonConverter, JsonObject, JsonProperty, OperationMode, ValueCheckingMode, } from "json2typescript";
let MapTrieNodeConverter = class MapTrieNodeConverter {
    serialize(map) {
        const jsonConvert = new JsonConvert();
        jsonConvert.registerClasses(Trie, TrieNode);
        const obj = {};
        for (const kv of map) {
            if (kv[1] instanceof TrieNode)
                obj[kv[0]] = jsonConvert.serialize(kv[1], TrieNode);
            else
                obj[kv[0]] = kv[1];
        }
        return obj;
    }
    deserialize(object) {
        const jsonConvert = new JsonConvert();
        jsonConvert.registerClasses(Trie, TrieNode);
        const propNames = Object.getOwnPropertyNames(object);
        const kv = propNames.map((name) => {
            const prop = object[name];
            const des = jsonConvert.deserialize(prop, TrieNode);
            return [name, des];
        });
        return new Map(kv);
    }
};
MapTrieNodeConverter = __decorate([
    JsonConverter
], MapTrieNodeConverter);
let TrieNode = TrieNode_1 = class TrieNode {
    get directChildren() {
        return Array.from(this.Children.values());
    }
    constructor(Value = " ", IsEndOfWord = false, parent) {
        this.Value = "";
        this.IsEndOfWord = false;
        this.Children = new Map();
        this.References = [];
        this.Value = Value;
        this.Children = new Map();
        this.References = [];
        this.IsEndOfWord = IsEndOfWord;
        this.Parent = parent;
    }
    initParents() {
        for (const [ch, child] of this.Children) {
            child.Parent = this;
            child.initParents();
        }
    }
    get Word() {
        return this.concatCharsReverse();
    }
    get depth() {
        return this.getDepthRecursive();
    }
    getDepthRecursive() {
        var _a, _b;
        const fromParent = (_b = (_a = this.Parent) === null || _a === void 0 ? void 0 : _a.getDepthRecursive()) !== null && _b !== void 0 ? _b : 0 + 1;
        return fromParent;
    }
    concatCharsReverse() {
        var _a;
        const fromParent = (_a = this.Parent) === null || _a === void 0 ? void 0 : _a.concatCharsReverse().replace(" ", "");
        return fromParent === undefined ? this.Value : fromParent + this.Value;
    }
    HasChild(value) {
        return this.Children.has(value);
    }
    HasChildren() {
        return this.Children.size > 0;
    }
    GetChild(value) {
        return this.Children.get(value);
    }
    GetChildren(levels) {
        const directChildren = Array.from(this.Children.values());
        if (levels && levels > 0) {
            const subs = directChildren.flatMap((x) => x.GetChildren(levels - 1));
            return [...directChildren, ...subs];
        }
        return directChildren;
    }
    AddChild(value) {
        this.Children.set(value, new TrieNode_1(value, false, this));
    }
    HasReferences() {
        return this.References.length > 0;
    }
    GetReferences() {
        return this.References;
    }
    AddReference(reference) {
        this.References.push(reference);
    }
    RemoveReference(reference) {
        this.References = this.References.filter((r) => r !== reference);
    }
};
__decorate([
    JsonProperty("V", String),
    __metadata("design:type", String)
], TrieNode.prototype, "Value", void 0);
__decorate([
    JsonProperty("E", Boolean),
    __metadata("design:type", Boolean)
], TrieNode.prototype, "IsEndOfWord", void 0);
__decorate([
    JsonProperty("C", MapTrieNodeConverter),
    __metadata("design:type", Map)
], TrieNode.prototype, "Children", void 0);
__decorate([
    JsonProperty("R", [String]),
    __metadata("design:type", Array)
], TrieNode.prototype, "References", void 0);
TrieNode = TrieNode_1 = __decorate([
    JsonObject("TrieNode"),
    __metadata("design:paramtypes", [String, Boolean, TrieNode])
], TrieNode);
export { TrieNode };
let Trie = Trie_1 = class Trie {
    constructor(jsonString, root) {
        this.Root = new TrieNode(" ");
        this.Root = new TrieNode(" ");
        if (jsonString) {
            const jsonObject = parse(jsonString);
            const result = Trie_1.FromJson(jsonObject);
            if (result) {
                this.Root = result.Root;
            }
        }
        if (root)
            this.Root = root;
        this.Root.initParents();
    }
    Contains(word, reference) {
        if (word === null) {
            return false;
        }
        let current = this.Root;
        for (const ch of Array.from(word)) {
            if (!current.HasChild(ch)) {
                return false;
            }
            current = current.GetChild(ch);
        }
        if (reference) {
            const refs = this.GetReferences(word);
            return current.IsEndOfWord && refs.includes(reference);
        }
        return current.IsEndOfWord;
    }
    GetReferences(word, canMissCharactersAtEnd) {
        if (word === null) {
            return [];
        }
        let current = this.Root;
        for (const ch of Array.from(word)) {
            if (!current.HasChild(ch)) {
                return [];
            }
            current = current.GetChild(ch);
        }
        if (current.IsEndOfWord) {
            return current.GetReferences();
        }
        if (canMissCharactersAtEnd) {
            const subs = current.GetChildren(canMissCharactersAtEnd);
            const subReferences = subs.flatMap((x) => x.GetReferences());
            return [...current.GetReferences(), ...subReferences];
        }
        return [];
    }
    Insert(word, reference) {
        if (word === null) {
            return;
        }
        let current = this.Root;
        for (const ch of Array.from(word)) {
            if (!current.HasChild(ch)) {
                current.AddChild(ch);
            }
            current = current.GetChild(ch);
        }
        if (reference) {
            current.AddReference(reference);
        }
        current.IsEndOfWord = true;
    }
    Remove(word, reference) {
        if (word === null) {
            return;
        }
        this._Remove(word, this.Root, 0, reference);
    }
    _Remove(word, root, index, reference) {
        if (index >= word.length) {
            if (reference) {
                root.RemoveReference(reference);
            }
            if (!root.HasReferences() || reference === null) {
                root.IsEndOfWord = false;
                root.References = [];
            }
            return;
        }
        const ch = word[index];
        if (!root.HasChild(ch)) {
            return;
        }
        const child = root.GetChild(ch);
        this._Remove(word, child, index + 1, reference);
        if (child.IsEndOfWord || child.HasReferences() || child.Children.size > 0) {
            return;
        }
        root.Children.delete(ch);
    }
    FindWordsWithReferences(searchTerm, canSkipCharacters, canMissCharactersAtEnd) {
        const wordWithReferences = [];
        let words = [];
        const lastNode = this.FindLastNodeOf(searchTerm);
        if (Boolean(canSkipCharacters) &&
            canSkipCharacters &&
            canSkipCharacters > 0) {
            const lastNodes = this.FindLastNodesOf(searchTerm, canSkipCharacters);
            words = lastNodes.map((x) => x.Word);
        }
        else {
            if (!lastNode)
                return wordWithReferences;
            this._FindWords(lastNode, searchTerm, words);
        }
        for (const word of words) {
            const references = this.GetReferences(word, canMissCharactersAtEnd);
            if (references.length > 0)
                wordWithReferences.push([word, references]);
        }
        return wordWithReferences;
    }
    FindLastNodesOf(searchTerm, canSkipCharacters) {
        const result = [];
        const deepEnoughNodes = this.nodesWithMinDepthToLeaf(searchTerm.length + 1);
        const filteredSubs = deepEnoughNodes.filter((x) => x.depth < (canSkipCharacters !== null && canSkipCharacters !== void 0 ? canSkipCharacters : 0));
        const fromRoot = this.FindLastNodeOf(searchTerm);
        const fromSubNodes = filteredSubs
            .flatMap((x) => this.FindLastNodeOf(searchTerm, x))
            .filter((x) => Boolean(x));
        if (fromRoot)
            return [fromRoot, ...fromSubNodes];
        return fromSubNodes;
    }
    FindLastNodeOf(searchTerm, searchFromNode) {
        let current = searchFromNode !== null && searchFromNode !== void 0 ? searchFromNode : this.Root;
        for (const searchCharacter of Array.from(searchTerm)) {
            if (!current.HasChild(searchCharacter)) {
                return null;
            }
            current = current.GetChild(searchCharacter);
        }
        return current;
    }
    FindWords(prefix) {
        const words = [];
        const lastNode = this.FindLastNodeOf(prefix);
        if (!lastNode)
            return words;
        this._FindWords(lastNode, prefix, words);
        return words;
    }
    _FindWords(endOfWordNode, prefix, words) {
        if (endOfWordNode === null) {
            return;
        }
        if (endOfWordNode.IsEndOfWord) {
            words.push(prefix);
        }
        for (const [ch, childNode] of endOfWordNode.Children) {
            this._FindWords(childNode, prefix + ch, words);
        }
    }
    nodes(startNode) {
        const nodes = [];
        const root = startNode !== null && startNode !== void 0 ? startNode : this.Root;
        for (const entry of root.directChildren) {
            if (!nodes.includes(entry))
                nodes.push(entry);
            const subs = this.nodes(entry);
            for (const sub of subs) {
                if (!nodes.includes(sub))
                    nodes.push(sub);
            }
        }
        return nodes;
    }
    nodesWithMinDepthToLeaf(minDepth, startNode) {
        const nodes = [];
        const root = startNode !== null && startNode !== void 0 ? startNode : this.Root;
        const children = root.directChildren.map((c) => {
            return {
                node: c,
                depth: this.depthToMaxLeafe(c),
            };
        });
        const deepEnough = children.filter((x) => x.depth >= minDepth);
        for (const entry of deepEnough) {
            if (!nodes.includes(entry.node))
                nodes.push(entry.node);
            const subs = this.nodesWithMinDepthToLeaf(minDepth, entry.node);
            for (const sub of subs) {
                if (!nodes.includes(sub))
                    nodes.push(sub);
            }
        }
        return nodes;
    }
    depthToMaxLeafe(node) {
        let maxDepth = 0;
        for (const [ch, child] of node.Children) {
            const depth = this.depthToMaxLeafe(child);
            maxDepth = Math.max(maxDepth, depth);
        }
        return maxDepth + 1;
    }
    TraversePreOrder(visit) {
        this.TraversePreOrderRecursive(this.Root, visit);
    }
    TraverseInOrder(visit) {
        this.TraverseInOrderRecursive(this.Root, visit);
    }
    TraversePostOrder(visit) {
        this.TraversePostOrderRecursive(this.Root, visit);
    }
    TraversePreOrderRecursive(node, visit) {
        if (node === null) {
            return;
        }
        visit(node);
        for (const [ch, child] of node.Children) {
            this.TraversePreOrderRecursive(child, visit);
        }
    }
    TraverseInOrderRecursive(node, visit) {
        if (node === null) {
            return;
        }
        for (const [ch, child] of node.Children) {
            this.TraverseInOrderRecursive(child, visit);
        }
        visit(node);
    }
    TraversePostOrderRecursive(node, visit) {
        if (node === null) {
            return;
        }
        for (const [ch, child] of node.Children) {
            this.TraversePostOrderRecursive(child, visit);
        }
        visit(node);
    }
    ToJson() {
        const jsonConvert = new JsonConvert();
        jsonConvert.registerClasses(Trie_1, TrieNode);
        const jsonObject = jsonConvert.serialize(this, Trie_1);
        return stringify(jsonObject);
    }
    ToContainer() {
        return {
            data: this.ToJson(),
        };
    }
    static FromJson(jsonObject) {
        // Choose your settings
        // Check the detailed reference in the chapter "JsonConvert class properties and methods"
        const jsonConvert = new JsonConvert();
        jsonConvert.registerClasses(Trie_1, TrieNode);
        jsonConvert.operationMode = OperationMode.LOGGING; // Print some debug data
        jsonConvert.ignorePrimitiveChecks = false; // Don't allow assigning number to string etc.
        jsonConvert.valueCheckingMode = ValueCheckingMode.ALLOW_OBJECT_NULL; // Never allow null
        // Map to the country class
        let trie;
        try {
            trie = jsonConvert.deserializeObject(jsonObject, Trie_1);
        }
        catch (e) {
            console.log(e);
            return undefined;
        }
        return trie;
    }
};
__decorate([
    JsonProperty("Root", TrieNode),
    __metadata("design:type", TrieNode)
], Trie.prototype, "Root", void 0);
Trie = Trie_1 = __decorate([
    JsonObject("Trie"),
    __metadata("design:paramtypes", [String, TrieNode])
], Trie);
export { Trie };
//# sourceMappingURL=trie.js.map