var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a;
import _ from "lodash";
import { immerable } from "immer";
import { clamp } from "../extensions/number.extentions.js";
import { AnchorType } from "../enums/index.js";
import { BoxModel } from "./boxModel.js";
import { CssValue } from "./cssValue.js";
import { FrameRange } from "./frameRange.js";
import { parseSync, stringifySync, } from "./subtitle/index.js";
import { Position } from "./position.js";
import { NormalizedSize } from "./normalizedSize.js";
import { TimeRange } from "./timerange.js";
export class WebVTT {
    getNodeList(exclude) {
        return this.nodeList.map((node) => node.type === "cue" && (exclude === null || exclude === void 0 ? void 0 : exclude.includes("words"))
            ? Object.assign(Object.assign({}, node), { data: Object.assign(Object.assign({}, node.data), { words: [] }) }) : node);
    }
    get cues() {
        return this.nodeList.filter((node) => node.type === "cue");
    }
    get endInSeconds() {
        const milliseconds = this.cues.reduce((max, cue) => Math.max(max, cue.data.end), 0);
        return milliseconds / 1000;
    }
    constructor(obj) {
        this[_a] = true; // Option 1
        this.language = "";
        this.nodeList = [];
        const clone = _.cloneDeep(obj);
        Object.assign(this, clone);
    }
    withFrameTiming(fps, offset = 0) {
        const webVTT = new WebVTT();
        webVTT.language = this.language;
        webVTT.nodeList = this.getFrameCues(fps, offset);
        return webVTT;
    }
    inRange(range) {
        const webVTT = new WebVTT();
        webVTT.language = this.language;
        webVTT.nodeList = this.cues.filter((cue) => {
            return cue.data.start >= range.start && cue.data.end <= range.end;
        });
        return webVTT;
    }
    intersectRange(range, clamping) {
        const webVTT = new WebVTT();
        webVTT.language = this.language;
        webVTT.nodeList = this.cues
            .filter((cue) => {
            return cue.data.start <= range.end && cue.data.end >= range.start;
        })
            .map((cue) => {
            const start = clamping
                ? clamp(cue.data.start, range.start, range.end)
                : cue.data.start;
            const end = clamping
                ? clamp(cue.data.end, range.start, range.end)
                : cue.data.end;
            return Object.assign(Object.assign({}, cue), { data: Object.assign(Object.assign({}, cue.data), { start, end }) });
        });
        return webVTT;
    }
    getFrameCues(fps, offset = 0) {
        return this.cues.map((cue) => {
            const cueRange = FrameRange.fromMilliseconds(cue.data.start, cue.data.end, fps).add(offset);
            return Object.assign(Object.assign({}, cue), { data: Object.assign(Object.assign({}, cue.data), { start: cueRange.start, end: cueRange.end }) });
        });
    }
    getCueAtFrame(frame) {
        if (!this.nodeList)
            return undefined;
        const active = this.cues.find((cue) => {
            return cue.data.start <= frame && frame <= cue.data.end;
        });
        return active;
    }
    addCue(node) {
        this.nodeList.push({ type: "cue", data: Object.assign({}, node) });
    }
    addCueRange(nodes) {
        this.nodeList = this.nodeList.concat(nodes.map((node) => ({ type: "cue", data: Object.assign({}, node) })));
    }
    removeScriptTags() {
        this.nodeList = this.nodeList.map((node) => {
            if (node.type === "cue" && node.data.text.match(/<script.*<\/script>/g)) {
                node.data.text = node.data.text.replace(/<script.*<\/script>/g, "");
            }
            if (node.type === "header" && node.data.match(/<script.*<\/script>/g)) {
                node.data = node.data.replace(/<script.*<\/script>/g, "");
            }
            return node;
        });
    }
    removeWords() {
        this.nodeList.forEach((node) => {
            if (node.type === "cue") {
                node.data.words = [];
            }
        });
    }
    extractTexts() {
        return this.cues.map((cue) => cue.data.text);
    }
    toIWebVTT(exclude) {
        return {
            language: this.language,
            nodes: this.getNodeList(exclude),
        };
    }
    toIWebVTTMinimal() {
        return {
            language: this.language,
            nodes: this.nodeList.map((node) => {
                if (node.type === "cue")
                    return node.data.text;
                return node.data;
            }),
        };
    }
    toJson(minimal = false) {
        if (minimal)
            return JSON.stringify(this.toIWebVTTMinimal());
        return JSON.stringify(this.toIWebVTT());
    }
    toSrt() {
        const result = stringifySync(this.nodeList, { format: "SRT" });
        return result;
    }
    toVtt() {
        const result = stringifySync(this.nodeList, { format: "WebVTT" });
        return result;
    }
    toString() {
        let text = "";
        this.cues.forEach((cue) => {
            text += `${cue.data.text}\n`;
        });
        return text;
    }
    toTimerangeTexts(minTextLength) {
        const items = this.cues.map((cue) => {
            return {
                text: cue.data.text,
                timerange: TimeRange.fromMilliseconds(cue.data.start, cue.data.end),
            };
        });
        if (minTextLength) {
            return mergeUntilMinLength(items, minTextLength);
        }
        return items;
    }
    transform() { }
    clearText() {
        this.nodeList.forEach((node) => {
            if (node.type === "cue") {
                node.data.text = "";
            }
        });
    }
    overwriteSpatialProperties(prior) {
        let index = 0;
        this.nodeList.forEach((node) => {
            if (node.type === "cue") {
                const priorNode = prior.nodes[index];
                if (priorNode) {
                    node.data.settings = priorNode.data.settings;
                }
            }
            index++;
        });
    }
    clone() {
        return WebVTT.fromIWebVTT(this.toIWebVTT(), this.language);
    }
    difference(other) {
        const changes = [];
        this.nodeList.forEach((node, index) => {
            if (node.type === "cue") {
                const otherNode = other.nodeList[index];
                if (otherNode) {
                    const changedProperties = this.compareCues(node.data, otherNode.data);
                    if (changedProperties.length > 0) {
                        changes.push(Object.assign(Object.assign({}, node.data), { changedProperties }));
                    }
                }
            }
        });
        return changes;
    }
    compareCues(a, b) {
        const changedProperties = [];
        if (a.start !== b.start)
            changedProperties.push("start");
        if (a.end !== b.end)
            changedProperties.push("end");
        if (a.text !== b.text)
            changedProperties.push("text");
        if (a.settings !== b.settings)
            changedProperties.push("settings");
        return changedProperties;
    }
    static fromITranscriptionResult(transcript) {
        const cues = transcript.segments.map((segment) => {
            var _b, _c;
            return {
                start: secondsToMilliseconds(segment.start),
                end: secondsToMilliseconds(segment.end),
                text: segment.text,
                words: (_b = segment.words) === null || _b === void 0 ? void 0 : _b.map((word) => {
                    return {
                        start: secondsToMilliseconds(word.start),
                        end: secondsToMilliseconds(word.end),
                        word: word.word,
                    };
                }),
                speaker: (_c = segment.speaker) === null || _c === void 0 ? void 0 : _c.toString(),
            };
        });
        const webVTT = WebVTT.fromCues(cues);
        webVTT.language = transcript.language;
        return webVTT;
    }
    static fromString(webVTTString, language) {
        const webVTT = new WebVTT();
        webVTT.language = language !== null && language !== void 0 ? language : "";
        try {
            webVTT.nodeList = parseSync(webVTTString);
        }
        catch (error) {
            console.error(error);
            return null;
        }
        webVTT.removeScriptTags();
        return webVTT;
    }
    static fromHttp(source, language, signal) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield fetch(source, { signal });
            if (response.status < 200 || response.status >= 220) {
                return null;
            }
            const text = yield response.text();
            return WebVTT.fromString(text, language);
        });
    }
    static fromIWebVTT(iWebVTT, language) {
        var _b, _c;
        const webVTT = new WebVTT();
        if (!iWebVTT)
            return webVTT;
        webVTT.language = (_c = (_b = iWebVTT === null || iWebVTT === void 0 ? void 0 : iWebVTT.language) !== null && _b !== void 0 ? _b : language) !== null && _c !== void 0 ? _c : "";
        if (Array.isArray(iWebVTT.nodes)) {
            webVTT.nodeList = [...iWebVTT.nodes];
        }
        else {
            webVTT.nodeList = Object.values(iWebVTT.nodes);
        }
        return webVTT;
    }
    static fromJson(data) {
        const iWebVTT = JSON.parse(data);
        return WebVTT.fromIWebVTT(iWebVTT);
    }
    static fromCues(cues, language) {
        const webVTT = new WebVTT();
        webVTT.language = language !== null && language !== void 0 ? language : "";
        if (cues) {
            webVTT.addCueRange(cues);
        }
        return webVTT;
    }
}
_a = immerable;
export var DirectionSetting;
(function (DirectionSetting) {
    DirectionSetting[""] = "horizontal";
    DirectionSetting["rl"] = "rl";
    DirectionSetting["lr"] = "lr";
})(DirectionSetting || (DirectionSetting = {}));
export var LineAlignSetting;
(function (LineAlignSetting) {
    LineAlignSetting["start"] = "start";
    LineAlignSetting["center"] = "center";
    LineAlignSetting["end"] = "end";
})(LineAlignSetting || (LineAlignSetting = {}));
/// For example, for horizontal text, position:100%,line-right specifies that the right edge of the box is at the right edge of the screen, regardless of alignment.
export var PositionAlignSetting;
(function (PositionAlignSetting) {
    PositionAlignSetting["line-left"] = "line-left";
    PositionAlignSetting["center"] = "center";
    PositionAlignSetting["line-right"] = "line-right";
    PositionAlignSetting["auto"] = "auto";
})(PositionAlignSetting || (PositionAlignSetting = {}));
export var AlignSetting;
(function (AlignSetting) {
    AlignSetting["start"] = "start";
    AlignSetting["left"] = "left";
    AlignSetting["center"] = "center";
    AlignSetting["end"] = "end";
    AlignSetting["right"] = "right";
})(AlignSetting || (AlignSetting = {}));
export class VTTCue {
    get htmlText() {
        return this.text.replace(/\n/g, "<br>");
    }
    constructor(obj) {
        this.start = 0;
        this.end = 0;
        this.text = "";
        Object.assign(this, obj);
    }
    static fromVtt(vtt) {
        const result = new VTTCue({
            start: vtt.start,
            end: vtt.end,
            text: vtt.text,
        });
        Object.assign(result, vtt);
        return result;
    }
    static splitSettings(cueSettings) {
        if (!cueSettings)
            return [];
        const settingStrings = cueSettings.split(" ");
        return settingStrings.map((setting) => {
            const [key, value] = setting.split(":");
            return { key, value };
        });
    }
    static splitSettingsToString(splits) {
        if (!splits)
            return "";
        return splits.map((split) => `${split.key}:${split.value}`).join(" ");
    }
    // https://www.speechpad.com/captions/webvtt#toc_14
    static parseCueSettings(start, end, text, cueSettings) {
        const cue = new VTTCue({ start, end, text });
        if (!cueSettings)
            return cue;
        const settings = cueSettings.split(" ");
        for (const setting of settings) {
            const [key, value] = setting.split(":");
            const vals = value.split(",");
            switch (key) {
                case "vertical":
                    cue.vertical =
                        DirectionSetting[value];
                    break;
                case "snapToLines":
                    cue.snapToLines = value === "true";
                    break;
                case "line":
                    cue.percent("line", vals[0]);
                    if (vals.length > 1) {
                        cue.percent("lineAlign", vals[1]);
                    }
                    break;
                case "position":
                    cue.percent("position", vals[0]);
                    if (vals.length > 1) {
                        cue.positionAlign =
                            PositionAlignSetting[vals[1]];
                    }
                    break;
                case "lineAlign":
                    cue.lineAlign =
                        LineAlignSetting[value];
                    break;
                case "positionAlign":
                    cue.positionAlign =
                        PositionAlignSetting[value];
                    break;
                case "size":
                    cue.percent("size", value);
                    break;
                case "align":
                    cue.align = AlignSetting[value];
                    break;
                default:
                    break;
            }
        }
        return cue;
    }
    getBox(origin) {
        var _b;
        const box = origin !== null && origin !== void 0 ? origin : new BoxModel(Position.centerBottom(), new NormalizedSize(0.5, 1), "cue");
        // #region vertical
        if (this.line && this.line !== "auto")
            box.position.setYInPercent(this.line);
        if (this.lineAlign &&
            (!this.vertical || this.vertical === DirectionSetting[""])) {
            switch (this.lineAlign) {
                case LineAlignSetting.start:
                    box.position.anchorY = AnchorType.Top;
                    break;
                case LineAlignSetting.center:
                    box.position.anchorY = AnchorType.Center;
                    break;
                case LineAlignSetting.end:
                    box.position.anchorY = AnchorType.Bottom;
                    break;
                default:
                    break;
            }
        }
        if (this.position && this.position !== "auto")
            box.position.setXInPercent(this.position);
        // #endregion
        // #endregion horizontal
        if (this.positionAlign &&
            (!this.vertical || this.vertical === DirectionSetting[""])) {
            switch (this.positionAlign) {
                case PositionAlignSetting["line-left"]:
                    box.position.anchorX = AnchorType.Left;
                    break;
                case PositionAlignSetting.center:
                    box.position.anchorX = AnchorType.Center;
                    break;
                case PositionAlignSetting["line-right"]:
                    box.position.anchorX = AnchorType.Right;
                    break;
                default:
                    break;
            }
        }
        const widthPercent = (_b = this.size) !== null && _b !== void 0 ? _b : 65;
        box.size.setWidthInPercent(widthPercent);
        box.width = CssValue.fromPercent(widthPercent);
        // #endregion
        return box;
    }
    getTextAlign() {
        if (!this.align)
            return "center";
        switch (this.align) {
            case AlignSetting.start:
            case AlignSetting.left:
                return "left";
            case AlignSetting.center:
                return "center";
            case AlignSetting.end:
            case AlignSetting.right:
                return "right";
            default:
                return "center";
        }
    }
    set(key, v) {
        if (!this.get(key) && v !== "") {
            this[key] = v;
        }
    }
    // Return the value for a key, or a default value.
    // If 'defaultKey' is passed then 'dflt' is assumed to be an object with
    // a number of possible default values as properties where 'defaultKey' is
    // the key of the property that will be chosen; otherwise it's assumed to be
    // a single value.
    get(key, dflt, defaultKey) {
        if (defaultKey) {
            return this.has(key) ? this[key] : dflt[defaultKey];
        }
        return this.has(key) ? this[key] : dflt;
    }
    // Check whether we have a value for a key.
    has(k) {
        return k in this;
    }
    // Accept a setting if its one of the given alternatives.
    alt(k, v, a) {
        for (let n = 0; n < a.length; ++n) {
            if (v === a[n]) {
                this.set(k, v);
                break;
            }
        }
    }
    // Accept a setting if its a valid (signed) integer.
    integer(k, v) {
        if (/^-?\d+$/.test(v)) {
            this.set(k, parseInt(v, 10));
        }
    }
    // Accept a setting if its a valid percentage.
    percent(k, v) {
        // eslint-disable-next-line
        var m;
        if ((m = v.match(/^([\d]{1,3})(\.[\d]*)?%$/))) {
            const f = parseFloat(v);
            if (f >= 0 && f <= 100) {
                this.set(k, f);
                return true;
            }
        }
        return false;
    }
}
export var HorizontalAlignmentSettings;
(function (HorizontalAlignmentSettings) {
    HorizontalAlignmentSettings["LEFT"] = "position:10%,line-left align:start";
    HorizontalAlignmentSettings["CENTER"] = "";
    HorizontalAlignmentSettings["RIGHT"] = "position:90%,line-right align:end";
})(HorizontalAlignmentSettings || (HorizontalAlignmentSettings = {}));
export var SizeSettings;
(function (SizeSettings) {
    SizeSettings["SMALL"] = "size:45%";
    SizeSettings["MEDIUM"] = "size:65%";
    SizeSettings["LARGE"] = "size:80%";
})(SizeSettings || (SizeSettings = {}));
function secondsToMilliseconds(seconds) {
    return seconds * 1000;
}
export function mergeUntilMinLength(items, minTextLength) {
    const stack = [...items];
    const sackInverted = stack.reverse();
    const results = [];
    let mergeItem = {
        text: "",
        timerange: TimeRange.default(),
    };
    while (sackInverted.length > 0) {
        const item = sackInverted.pop();
        if (item) {
            if (mergeItem.text.length === 0)
                mergeItem.timerange = item.timerange.clone();
            if (mergeItem.text.length < minTextLength) {
                mergeItem.text += item.text + "\n";
                mergeItem.timerange.end = item.timerange.end;
            }
        }
        if (mergeItem.text.length >= minTextLength || sackInverted.length === 0) {
            results.push({
                text: mergeItem.text,
                timerange: mergeItem.timerange,
            });
            mergeItem = {
                text: "",
                timerange: TimeRange.default(),
            };
        }
    }
    return results;
}
//# sourceMappingURL=webVTT.js.map